'-------------------------------- Loop04 ---------------------------------'
'                                                                         '
' This program compares execution times for using a rolled up Simpson's   '
' Rule numerical integration to an algorithm with partially unrolled      '
' loops.                                                                  '
'                                                                         '
' The function integrated is x^5 from 0 to 0.9999 with 10,000 steps.      '
'                                                                         '
' NOTE: This version differs from the Windows Basic and C versions (as    '
'       discussed in the text in that only 5,000 iterations are done      '
'       (compared to the other versions doing 50,000 iterations).         '
'                                                                         '
'-------------------------------------------------------------------------'

'procedure declarations

'global declarations
'longs
DIM NumIts              AS LONG         'number of iterations
DIM counter             AS LONG         'iteration counter
DIM SimpCounter         AS LONG         'loop counter for the summation

'singles
DIM F(10000)            AS SINGLE       ' function to be integrated

DIM h.3                 AS SINGLE
DIM S0                  AS SINGLE
DIM S1                  AS SINGLE
DIM S2                  AS SINGLE

DIM Integral            AS SINGLE
DIM starttimer          AS SINGLE       'ticks at loop start
DIM stoptimer           AS SINGLE       'ticks at loop exit



'---------------------------- Initializations ------------------------------

'max 50000 iterations
NumIts = 5000


FOR counter = 0 TO 9999
    
	F(counter) = (CSNG(counter) / 10000#) ^ 5
    
NEXT counter


'------------------------------- End Initializations ---------------------

'clear the screen
CLS

'-------------------- rolled-up Simpson's Rule Integration ---------------

'display message
PRINT "Integrating f(x) with rolled-up algorithm ..."
PRINT


'start the timer
starttimer = TIMER

FOR counter = 1 TO NumIts
    
	' h/3, where h=(b-a)/n
	h.3 = .999 / 30000

	' sum the first and last function values
	S0 = F(0) + F(9999)

	' zero values for the running summations   
	S1 = 0
	S2 = 0
    
	'sum terms except first and last
	FOR SimpCounter = 1 TO 9997 STEP 2
    
	    S1 = S1 + F(SimpCounter)
	    S2 = S2 + F(SimpCounter + 1)
    
	NEXT SimpCounter

	' 'odd' terms * 2
	S1 = S1 * 2
	
	' 'even' terms * 4
	S2 = S2 * 4
    
	'full summation
	Integral = S0 + S1 + S2
	
	'multiply by step
	Integral = Integral * h.3

NEXT counter

'stop the timer
stoptimer = TIMER

'display the result
PRINT "Integrated f(x) 0 - 0.9999 "; NumIts; " times in"
PRINT stoptimer - starttimer; " seconds"
PRINT "Integral = "; Integral
PRINT



'-------------------- End rolled-up Simpson's Rule Integration -------------

'----------------- Simpson's Rule Integration Unrolled 4 terms -------------

'display message
PRINT "Integrating f(x) with loop partially unrolled 4 terms ..."
PRINT

'start the timer
starttimer = TIMER

FOR counter = 1 TO NumIts

	' h/3, where h=(b-a)/n
	h.3 = .999 / 30000

	'sum the first and last terms
	S0 = F(0) + F(9999)
	
	' zero values for the running summations
	S2 = 0
	S1 = 0
    
	'sum terms except first and last; note that the loop is stepped
	'by 8 so loop overhead is smaller (increments, compares and conditional
	'jumps are eliminated).
    
	FOR SimpCounter = 1 TO 9985 STEP 8
    
	    S1 = S1 + F(SimpCounter) + F(SimpCounter + 2) + F(SimpCounter + 4) + F(SimpCounter + 6)
	    S2 = S2 + F(SimpCounter + 1) + F(SimpCounter + 3) + F(SimpCounter + 5) + F(SimpCounter + 7)
    
	NEXT SimpCounter
	
	'cleanup
	S1 = S1 + F(9993) + F(9995) + F(9997)
	S2 = S2 + F(9994) + F(9996) + F(9998)

	' 'odd' terms * 2
	S1 = S1 * 2
	
	' 'even' terms * 4
	S2 = S2 * 4
    
	'total summation
	Integral = S0 + S1 + S2
	
	'multiply by h/3
	Integral = Integral * h.3

NEXT counter

'stop the timer
stoptimer = TIMER

'display the result
PRINT "Integrated f(x) 0 - 0.9999 "; NumIts; " times in"
PRINT stoptimer - starttimer; " seconds"
PRINT "Integral = "; Integral
PRINT

'-------------- End Simpson's Rule Integration Unrolled 4 terms ------------

'------------------ Simpson's Rule Integration Unrolled 5 terms ------------

'display message
PRINT "Integrating f(x) with loop partially unrolled 5 terms ..."
PRINT

'start the timer
starttimer = TIMER

FOR counter = 1 TO NumIts

	' h/3, where h=(b-a)/n
	h.3 = .999 / 30000

	'sum the first and last terms
	S0 = F(0) + F(9999)

	' zero values for the running summations   
	S1 = 0
	S2 = 0
    
	'sum terms except first and last; note that the loop is stepped
	'by 10 so loop overhead is smaller (increments, compares and conditional
	'jumps are eliminated).
    
	FOR SimpCounter = 1 TO 9981 STEP 10
    
	    S1 = S1 + F(SimpCounter) + F(SimpCounter + 2) + F(SimpCounter + 4) + F(SimpCounter + 6) + F(SimpCounter + 8)
	    S2 = S2 + F(SimpCounter + 1) + F(SimpCounter + 3) + F(SimpCounter + 5) + F(SimpCounter + 7) + F(SimpCounter + 9)
    
	NEXT SimpCounter
	
	'cleanup
	S1 = S1 + F(9991) + F(9993) + F(9995) + F(9997)
	S2 = S2 + F(9992) + F(9994) + F(9996) + F(9998)
    
	' 'odd' terms * 2
	S1 = S1 * 2
	
	' 'even' terms * 4
	S2 = S2 * 4
    
	'total summation
	Integral = S0 + S1 + S2
	
	'multiply by h/3
	Integral = Integral * h.3

NEXT counter

'stop the timer
stoptimer = TIMER

'display the result
PRINT "Integrated f(x) 0 - 0.9999 "; NumIts; " times in"
PRINT stoptimer - starttimer; " seconds"
PRINT "Integral = "; Integral
PRINT

'-------------- End Simpson's Rule Integration Unrolled 5 terms ------------

'------------------ Simpson's Rule Integration Unrolled 8 terms ------------

'display message
PRINT "Integrating f(x) with loop partially unrolled 8 terms ..."
PRINT

'start the timer
starttimer = TIMER

FOR counter = 1 TO NumIts

	' h/3, where h=(b-a)/n
	h.3 = .999 / 30000

	'sum the first and last terms
	S0 = F(0) + F(9999)
	
	' zero values for the running summations
	S1 = 0
	S2 = 0
    
	'sum terms except first and last; note that the loop is stepped
	'by 16 so loop overhead is smaller (increments, compares and conditional
	'jumps are eliminated).
	FOR SimpCounter = 1 TO 9969 STEP 16
    
	    S1 = S1 + F(SimpCounter) + F(SimpCounter + 2) + F(SimpCounter + 4) + F(SimpCounter + 6) + F(SimpCounter + 8) + F(SimpCounter + 10) + F(SimpCounter + 12) + F(SimpCounter + 14)
	    S2 = S2 + F(SimpCounter + 1) + F(SimpCounter + 3) + F(SimpCounter + 5) + F(SimpCounter + 7) + F(SimpCounter + 9) + F(SimpCounter + 11) + F(SimpCounter + 13) + F(SimpCounter + 15)
    
	NEXT SimpCounter
	
	'cleanup
	S1 = S1 + F(9985) + F(9987) + F(9989) + F(9991) + F(9993) + F(9995) + F(9997)
	    
	S2 = S2 + F(9986) + F(9988) + F(9990) + F(9992) + F(9994) + F(9996) + F(9998)

	' 'odd' terms * 2
	S1 = S1 * 2
	
	' 'even' terms * 4
	S2 = S2 * 4
    
	'total summation
	Integral = S0 + S1 + S2
	
	'multiply by h/3
	Integral = Integral * h.3

NEXT counter

'stop the timer
stoptimer = TIMER

'display the result
PRINT "Integrated f(x) 0 - 0.9999 "; NumIts; " times in"
PRINT stoptimer - starttimer; " seconds"
PRINT "Integral = "; Integral
PRINT

'-------------- End Simpson's Rule Integration Unrolled 8 terms ------------

' Keep display active until user presses a key
' this is needed when running in the dos subsystem of windows, else
' windows closes window immediately

' display a message
PRINT : PRINT "Press any key to end..."

'check keylatch; if empty, simply loop back
LoopHere:
In$ = INKEY$
IF In$ = "" THEN GOTO LoopHere

'end when keylatch contains any key
END


