'-------------------------------- RISC01 ---------------------------------'
'									  '
' This program compares the CISC style algorithm to the RISC style for    '
' the reduced Lennard Jones pair energy calculation.  This generic BASIC  '
' version of the program has no 'user interface' but simply executes      '
' both styles in sequence.						  '
'									  '
'-------------------------------------------------------------------------'

' global declarations
'longs
DIM counter     AS LONG         'iteration counter

'singles
DIM starttimer  AS SINGLE       'ticks at start of loop
DIM stoptimer   AS SINGLE       'ticks at loop exit

'doubles
DIM r           AS DOUBLE       'distance between particles
DIM r6          AS DOUBLE       'r^6
DIM r12         AS DOUBLE       'r^12
DIM E           AS DOUBLE       'computed energy


'clear the screen
CLS


'set distance to a made-up value for demo
r = 1.234

'-------------------------------- CISC Style Calculation ------------------

'display message
PRINT "Computing LJ Energy in CISC style ..."
PRINT

'start the timer
starttimer = TIMER

'begin loop for CISC style run
FOR counter = 1 TO 10000000

        'compute Energy in single equation...CISC style
        E = 1 / r ^ 12 - 1 / r ^ 6

NEXT counter

'stop the timer
stoptimer = TIMER

'display the result
PRINT "Computed E in Style Style in: "; stoptimer - starttimer; " seconds"
PRINT

'-------------------------------- End CISC Style Calculation ---------------

'-------------------------------- RISC Style Calculation -------------------

'display message
PRINT "Computing LJ Energy in RISC style ..."
PRINT

'start the timer
starttimer = TIMER

'begin loop for RISC style run
FOR counter = 1 TO 10000000

        'compute Energy in RISC style

        'compute r^6
        r6 = r * r * r * r * r * r

        'compute r^12
        r12 = r6 * r6

        'compute E
        E = 1 / r12 - 1 / r6

NEXT counter

'stop the timer
stoptimer = TIMER

'display the result
PRINT "Computed E in RISC style in: "; stoptimer - starttimer; " seconds"
PRINT

' ------------------------------- End RISC Style Calculation ---------------

' Keep display active until user presses a key
' this is needed when running in the dos subsystem of windows, else
' windows closes window immediately

' display a message
PRINT : PRINT "Press any key to end..."

'check keylatch; if empty, simply loop back
LoopHere:
A$ = INKEY$
IF A$ = "" THEN GOTO LoopHere

'end when keylatch contains any key
END

