'------------------------------ RISC02 -----------------------------------'
'									  '
' This program compares the CISC style algorithm to the RISC style for	  '
' the two dimensional distance calculation.  This generic BASIC		  '
' version of the program has no 'user interface' but simply executes      '
' both styles in sequence.						  '
' 									  '
'-------------------------------------------------------------------------'

' global declarations
'longs
DIM counter     AS LONG         ' iteration counter

'singles
DIM starttimer  AS SINGLE       ' ticks at start of loop
DIM stoptimer   AS SINGLE       ' ticks at loop exit

'doubles
DIM x1          AS DOUBLE       ' x coordinate of particle 1
DIM x2          AS DOUBLE       ' x coordinate of particle 2
DIM y1          AS DOUBLE       ' y coordinate of particle 1
DIM y2          AS DOUBLE       ' y coordinate of particle 2
DIM tempx       AS DOUBLE       ' delta-x, then (delta-x)^2
DIM tempy       AS DOUBLE       ' delta-y, then (delta-y)^2
DIM r           AS DOUBLE       ' computed distance

'clear the screen
CLS

' set coordinates to some made-up values for demo
x1 = 1.234
x2 = 2.675
y1 = .045
y2 = 2.073

'------------------------------ CISC Distance Calculation ------------------

'display message
PRINT "Computing Distance in CISC style algorithm ..."
PRINT

'start the timer
starttimer = TIMER

'begin loop for CISC calculation
FOR counter = 1 TO 10000000

	'compute distance in CISC style, a single equation
	r = SQR((x2 - x1) ^ 2 + (y2 - y1) ^ 2)

NEXT counter

'stop the timer
stoptimer = TIMER

'display the result
PRINT "r from CISC style in: "; stoptimer - starttimer; " seconds"
PRINT

'------------------------------ End CISC Distance Calculation ------------------

'------------------------------ RISC Distance Calculation ------------------

'display message
PRINT "Computing distance in RISC style algorithm ..."
PRINT

'start the timer
starttimer = TIMER

'begin loop for RISC calculation
FOR counter = 1 TO 10000000

	  'compute delta-x and (delta-x)^2
	  tempx = x2 - x1
	  tempx = tempx * tempx
	  
	  'compute delta-y and (delta-y)^2
	  tempy = y2 - y1
	  tempy = tempy * tempy

	  'compute distance from delta-x and delta-y
	  r = SQR(tempx + tempy)

NEXT counter

'stop the timer
stoptimer = TIMER

'display the result
PRINT "r from RISC style in: "; stoptimer - starttimer; " seconds"

'------------------------------ End RISC Distance Calculation ------------------

' Keep display active until user presses a key
' this is needed when running in the dos subsystem of windows, else
' windows closes window immediately

' display a message
PRINT : PRINT "Press any key to end..."

'check keylatch; if empty, simply loop back
LoopHere:
A$ = INKEY$
IF A$ = "" THEN GOTO LoopHere

'end when keylatch contains any key
END

