'----------------------------- RISC04 ------------------------------------'
'									  '
' This program compares the CISC style algorithm to the RISC		  '
' style for the Boltzmann factor calculation that uses the exp()	  '
' function. 								  '
'									  '
' This generic BASIC version of the program has no 'user interface'	  '
' but simply executes both styles in sequence.				  '
'									  '
'-------------------------------------------------------------------------'

' global declarations
'longs
DIM counter     AS LONG         ' iteration counter

'singles
DIM starttimer  AS SINGLE       ' ticks at start of loop
DIM stoptimer   AS SINGLE       ' ticks at loop exit

'doubles
DIM Temp        AS DOUBLE       ' temperature
DIM kBoltz      AS DOUBLE       ' Boltzmann's constant
DIM Energy      AS DOUBLE       ' Energy
DIM BoltzFactor AS DOUBLE       ' computed Boltzmann factor
DIM Argument    AS DOUBLE       ' temporary arguement for RISC calculation

'clear the screen
CLS

' initialize constant & set parameters to some made-up values for demo
Temp = 89.45
kBoltz = 1.381E-23
Energy = 4.52E-20


'------------------------------ CISC Boltzmann Calculation ------------------

'display message
PRINT "Computing Boltzmann Factor in CISC style algorithm ..."
PRINT

'start the timer
starttimer = TIMER

'begin loop for CISC calculation
FOR counter = 1 TO 10000000

	'compute the Boltzmann Factor in CISC Style
	BoltzFactor = EXP(-Energy / (kBoltz * Temp))

NEXT counter

'stop the timer
stoptimer = TIMER

'display the result
PRINT "Boltzmann Factor from CISC style in: "; stoptimer - starttimer; " seconds"
PRINT

'------------------------------ End CISC Boltzmann Calculation ------------------

'------------------------------ RISC Boltzmann Calculation ----------------------

'display message
PRINT "Computing Boltzmann Factor in RISC style algorithm ..."
PRINT

'start the timer
starttimer = TIMER

'begin loop for RISC calculation
FOR counter = 1 TO 10000000

	'RISC style Boltzmann factor;
	TempArg = -Energy / (kBoltz * Temp)
	BoltzFactor = EXP(TempArg)


NEXT counter

'stop the timer
stoptimer = TIMER

'display the result
PRINT "Boltzmann Factor from RISC style in: "; stoptimer - starttimer; " seconds"

'------------------------------ End RISC Boltzmann Calculation ------------------

' Keep display active until user presses a key
' this is needed when running in the dos subsystem of windows, else
' windows closes window immediately

' display a message
PRINT : PRINT "Press any key to end..."

'check keylatch; if empty, simply loop back
LoopHere:
A$ = INKEY$
IF A$ = "" THEN GOTO LoopHere

'end when keylatch contains any key
END

