'-------------------------------- LookUp02 -------------------------------'
'                                                                         '
' This program compares a floating point intensive calculation to a Look- '
' Up Table storing function values.                                       '
'                                                                         '
' NOTE: Due to array storage limitations, function in this version is     '
'       much smaller than in the Windows Basic and C version (that are    '
'       discussed in the text).                                           '
'                                                                         '
'-------------------------------------------------------------------------'

'declarations
'longs
DIM NumIts      AS LONG         'number of iterations
DIM counter     AS LONG         'iteration counter
DIM arg         AS LONG         'look-up index

'singles
DIM timestart   AS SINGLE       'ticks at start of loop
DIM timestop    AS SINGLE       'ticks at loop exit

'doubles
DIM kvar        AS DOUBLE       'Boltzmann's constant
DIM Expon(2830) AS DOUBLE       'The Look-Up Table for the Boltzmann Function
DIM Temp        AS DOUBLE       'temperature
DIM Energy      AS DOUBLE       'energy used in the Boltzmann function
DIM p           AS DOUBLE       'computed Boltzmann Factor
DIM PArg        AS DOUBLE       'arguement for Boltzmann Look-Up

'Clear the screen
CLS

'-------------------------------- Initializations -------------------------

'do 10000000 iterations of calculation
NumIts = 10000000

'Boltzmann's constant, Temp and Energy
kvar = 1.381E-23
Temp = 100
Energy = 4E-21

PRINT "Initialize Look-Up Array ..."
PRINT
    
'start the timer
timestart = TIMER
    
'need 2,830 Boltzmann factors for this example
FOR counter = 50 TO 2830
    
	'compute P(Counter/100)
	Expon(counter) = EXP(-counter / 100)
	    
NEXT counter

'stop the timer
timestop = TIMER
    
'display the results
PRINT "Look-Up Array initialized in"
PRINT timestop - timestart; " seconds."
PRINT
    
'---------------------------- End Initializations -------------------------

'---------------------------- Explicit Calculation ------------------------

'display message
PRINT "Starting explicit calculation of Boltzmann Factor ..."
PRINT

'start the timer
timestart = TIMER

'begin the explicit calculation
FOR counter = 1 TO NumIts

	'compute the Boltzmann Factor
	p = EXP(-Energy / (kvar * Temp))

NEXT counter

'stop the timer
timestop = TIMER

'display the result
PRINT "Explicitly computed Boltzmann Factor "; NumIts; " times in "
PRINT timestop - timestart; " seconds"
PRINT "Factor = "; p
PRINT

'------------------------ End Explicit Calculation ------------------------

'---------------------------- LookUp Calculation ------------------------

'display message
PRINT "Starting LookUp of Boltzmann Factor ..."
PRINT

'start the timer
timestart = TIMER


'begin the LookUp 'calculation'
FOR counter = 1 TO NumIts

	'compute the index
	arg = CLNG((Energy * 100) / (kvar * Temp))
	
	'look-up the Boltzmann Factor
	p = Expon(arg)

NEXT counter

'stop the timer
timestop = TIMER

'display the result
PRINT "Looked Up Boltzmann Factor "; NumIts; " times in "
PRINT timestop - timestart; " seconds"
PRINT "Factor = "; p
PRINT

'------------------------ End LookUp Calculation ------------------------

' Keep display active until user presses a key
' this is needed when running in the dos subsystem of windows, else
' windows closes window immediately

' display a message
PRINT : PRINT "Press any key to end..."

'check keylatch; if empty, simply loop back
LoopHere:
A$ = INKEY$
IF A$ = "" THEN GOTO LoopHere

'end when keylatch contains any key
END

