DECLARE FUNCTION LJFULL! (TotNum AS LONG, xa() AS SINGLE, ya() AS SINGLE, en() AS DOUBLE)
DECLARE FUNCTION LJEnergy! (moved AS LONG, Total AS LONG, xmoved AS SINGLE, ymoved AS SINGLE, xa() AS SINGLE, ya() AS SINGLE, en() AS DOUBLE)
'-------------------------------- Other02 --------------------------------'
'                                                                         '
' This program compares full energy calculation using nested loops to     '
' calculation of just the energy change which does not require nested     '
' loops. This version does not use a look up table for distance.          '
'                                                                         '
'-------------------------------------------------------------------------'

'global declarations
'longs
DIM counter     AS LONG         'iteration counter
DIM cNumInBox   AS LONG         'number of particles
DIM cBoxSide    AS LONG         'size of box
DIM NumConfigs  AS LONG         'number of configurations

'singles
DIM Sigma       AS SINGLE       'LJ Sigma parameter
DIM Depth       AS SINGLE       'LJ Well depth parameter
DIM timestart   AS SINGLE       'ticks at start of loop
DIM timestop    AS SINGLE       'ticks at loop exit
DIM x(20)       AS SINGLE       'x position of particle
DIM y(20)       AS SINGLE       'y position of particle

'doubles
DIM energy(354) AS DOUBLE       'Look-Up Table Entry for the LJ Energies as
				'as function of distance; distance mesh is 10pm
DIM TotalEnergy AS DOUBLE       'Total Energy

'clear the screen
CLS

'initialize number of configurations, number of particles and box size
NumConfigs = 50000
cNumInBox = 20
cBoxSide = 500

'initialize sigma (in pm) and depth (in J) with Helium parameters
Sigma = 258
Depth = 5.65E-22

'-------------------------------- Initialize Energy Array -----------------

DIM r           AS DOUBLE       'distance
DIM R6          AS DOUBLE       'distance^6
DIM R12         AS DOUBLE       'distance^12
DIM Sigma6      AS DOUBLE       'sigma^6
DIM Sigma12     AS DOUBLE       'sigma^12

PRINT "Initializing Look Up Table for energies"

'need an energy for all possible distances within box
FOR counter = 1 TO 354
    
	'distance mesh (or step) is 10 pm
	r = counter * 10
	
	'distance^6
	R6 = r * r * r * r * r * r
	
	'distance^12
	R12 = R6 * R6
	
	'Sigma^6
	Sigma6 = Sigma * Sigma * Sigma * Sigma * Sigma * Sigma
	
	'Sigma^12
	Sigma12 = Sigma6 * Sigma6
	
	'Compute LJ Energy
	energy(counter) = Sigma12 / R12 - Sigma6 / R6
	
	'Scale by well depth
	energy(counter) = energy(counter) * Depth
	
NEXT counter

PRINT "Finished initializing energy array"

'------------------------------ Initialize Particles ----------------------

'put particles initially at random positions

PRINT "Initializing 20 particles"
    
FOR counter = 1 TO 20      'for 20 particles

	      
	'put particles at random positions for demo
	x(counter) = 354 * RND
	y(counter) = 354 * RND
	    
	
NEXT counter

PRINT "Finished assigning initial particle positions"


'-------------------------------- Full Energy Calculation -----------------


'display message
PRINT "Starting full energy calculation with nested loop ..."
PRINT

'start the timer
timestart = TIMER

FOR counter = 1 TO NumConfigs

       'Calculate the Energy
	TotalEnergy = LJFULL(cNumInBox, x(), y(), energy())

	'Calculate another energy;  For this demo, simply call
	'LJFull function again.
	TotalEnergy = LJFULL(cNumInBox, x(), y(), energy())

NEXT counter

'stop the timer
timestop = TIMER

'display the result
PRINT "Computed full energy in "; timestop - timestart; " seconds"
PRINT

'--------------------------------- End Full Energy Calculation -------------

'--------------------------------- Energy Change Calculation ---------------

DIM SelectedParticle    AS LONG         'the index of the selected particle
DIM SelectedX           AS SINGLE       'x coordinate of selected particle
DIM SelectedY           AS SINGLE       'y coordinate of selected particle

DIM DeltaEnergy1        AS DOUBLE       'first energy
DIM DeltaEnergy2        AS DOUBLE       'second energy

'display message
PRINT "Starting energy change calculation with no nested loops ..."
PRINT

'start the timer
timestart = TIMER

'begin the unrolled calculation
FOR counter = 1 TO NumConfigs

	'Select a particle
	SelectedParticle = INT(cNumInBox * RND + 1)
	
	'Cache the particle x,y coordinates
	SelectedX = x(SelectedParticle)
	SelectedY = y(SelectedParticle)
 
	'Calculate energy
	DeltaEnergy1 = LJEnergy(SelectedParticle, cNumInBox, SelectedX, SelectedY, x(), y(), energy())
	
	'Calculate another energy; rather than selecting a random new x,y for
	'selected particle, simply switching x,y to 'assume' a new position.  Monte Carlo
	'does not do this.
	'Since x,y is different each time through loop, y,x will be also.
	'There is no performance cost in doing this since the program does not
	'"care" what parameters are passed to the function, so long as they are
	'the right type.
	DeltaEnergy2 = LJEnergy(SelectedParticle, cNumInBox, SelectedY, SelectedX, x(), y(), energy())

NEXT counter

'stop the timer
timestop = TIMER

'display the result
PRINT "Computed Energy Change in "; timestop - timestart; " seconds"
PRINT

'------------------------------------- End Unrolled Loop Calculation --------

' Keep display active until user presses a key
' this is needed when running in the dos subsystem of windows, else
' windows closes window immediately

' display a message
PRINT : PRINT "Press any key to end..."

'check keylatch; if empty, simply loop back
LoopHere:
A$ = INKEY$
IF A$ = "" THEN GOTO LoopHere

'end when keylatch contains any key
END

FUNCTION LJEnergy (moved AS LONG, Total AS LONG, xmoved AS SINGLE, ymoved AS SINGLE, xa() AS SINGLE, ya() AS SINGLE, en() AS DOUBLE)

       
	'longs
	DIM i           AS LONG         'particle counter
	DIM deltax      AS LONG         'difference between particle x's
	DIM deltay      AS LONG         'difference between particle y's
	DIM r           AS LONG         'distance between particles

	'singles
	DIM scurrentx   AS SINGLE       'x coordinate of 'current' particle
	DIM scurrenty   AS SINGLE       'y coordinate of 'current' particle

	'doubles
	DIM thisenergy  AS DOUBLE       'cummulative energy holder
	DIM tempr       AS DOUBLE       'temp for r computation

	'initialize to zero
	thisenergy = 0

	FOR i = 1 TO Total

		scurrentx = xa(i)
		scurrenty = ya(i)


		'avoid r=0
		IF i <> moved THEN

			'compute x difference
			deltax = ABS(scurrentx - xmoved)

			'compute y difference
			deltay = ABS(scurrenty - ymoved)

			'compute distance
			tempr = deltax * deltax + deltay * deltay
			tempr = SQR(tempr)
			r = INT(tempr / 10)

			'accumulate and lookup energy
			thisenergy = thisenergy + en(r)

		END IF


	NEXT i


	LJEnergy = thisenergy


END FUNCTION

FUNCTION LJFULL (TotNum AS LONG, xa() AS SINGLE, ya() AS SINGLE, en() AS DOUBLE)

	'longs
	
	DIM i           AS LONG         'i particle counter
	DIM j           AS LONG         'j particle counter
	DIM deltax      AS LONG         'x difference between i and j particles
	DIM deltay      AS LONG         'y difference between i and j particles
	DIM r           AS LONG         'r from rval lookup

	'doubles
	DIM thisenergy      AS DOUBLE       'cummulative holder for energy
	DIM tempr           AS DOUBLE       'temp value for r computation

	'initialize to zero
	thisenergy = 0

	FOR i = 1 TO TotNum - 1


		FOR j = i + 1 TO TotNum

			'x difference
			deltax = ABS(xa(i) - xa(j))

			'y difference
			deltay = ABS(ya(i) - ya(j))


			'look up distance
			tempr = deltax * deltax + deltay * deltay
			tempr = SQR(tempr)
			r = INT(tempr / 10)

			'look up energy
			thisenergy = thisenergy + en(r)

		NEXT j

	NEXT i

	LJFULL = thisenergy


END FUNCTION

