VERSION 5.00
Begin VB.Form frmData02 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "Loop Counter Type Demo using y=sin(x)"
   ClientHeight    =   4290
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   6285
   Icon            =   "Data02.frx":0000
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   4290
   ScaleWidth      =   6285
   StartUpPosition =   2  'CenterScreen
   Begin VB.CommandButton cmdClear 
      Caption         =   "Clear Text"
      Height          =   615
      Left            =   4560
      TabIndex        =   4
      Top             =   3480
      Width           =   1575
   End
   Begin VB.CommandButton cmdMeth4 
      Caption         =   "Long Loop Counter in Radians"
      Height          =   615
      Left            =   4560
      TabIndex        =   3
      Top             =   2640
      Width           =   1575
   End
   Begin VB.CommandButton cmdMeth3 
      Caption         =   "Double Loop Counter in Radians"
      Height          =   615
      Left            =   4560
      TabIndex        =   2
      Top             =   1800
      Width           =   1575
   End
   Begin VB.CommandButton cmdMeth2 
      Caption         =   "Long Loop Counter in Degrees"
      Height          =   615
      Left            =   4560
      TabIndex        =   1
      Top             =   960
      Width           =   1575
   End
   Begin VB.CommandButton cmdMeth1 
      Caption         =   "Double Loop Counter in Degrees"
      Height          =   615
      Left            =   4560
      TabIndex        =   0
      Top             =   120
      Width           =   1575
   End
End
Attribute VB_Name = "frmData02"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'
'This demo explores the trade-off in using doubles for loop
'counters vs. type conversion.  This is sometimes needed when
'a particular function takes a double as an argument (or any
'type other than long).
'
'For demo purposes, the sin() function is used.  The sin(x) is computed
'iteratively (1000 times) from 0 to 360 degrees in 0.01 degree increments.
'The iteration counter (1 to 1000) can used as a long, since it does
'not appear in the computational steps.
'
'There are two key ways this simple algorithm can be examined.  First,
'the sin() function takes only values in radians, so the conversion
'from degree to radian is examined.  Second, the sin() function
'takes a double and returns a double.  This allows the examination
'of type conversion when a long is used for the argument counter. This is
'important in this procedure since in degrees, the long can be incremented by
'one, whereas in radians, another value must be used. (The situation would,
'of course, be reversed if it was desired to have the sin() at, say, 0.01
'radians....).
'
'To ensure that type conversions are done in a 'controlled' way,
'most of the constants needed in the code are declared as constants
'of proper 'type' rather than entered into the source code as immediates.
'The reason for this is to eliminate 'hidden' type conversions.  Recall
'that the compiler does insert 'immediate' data when variables
'declared as constants appear; we are simply ensuring proper type.
'For example, if we used the value 0.01, the compiler might insert
'0.01 as a 32 bit value, forcing a 'hidden' 32 bit to 64 bit conversion
'when the value is used.  By declaring Const cDegStep as Double = 0.01,
'we force all references to this immediate to be quad words (64 bits).

'Type conversions are general performance killers that creep into a program
'in sometimes hard-to-find places.  By not using a Strong Typing paradigm,
'BASIC is particularly susceptible to this.
'
Option Explicit

'Module Globals
'Constants
Const cNumIts As Long = 1000        'number of iterations, just a counter to
                                    'stretch the execution time to several seconds

Const cDeg2Rad As Double = 0.017453292  'Pi/180 to convert degrees to radians
Const ccDeg2Rad As Double = 0.00017453292   '0.01 degrees in redians
Const c2Pi As Double = 6.283185307      '2 * Pi
Const cDegStep As Double = 0.01         'Degree step size, the 'precision' we wish
                                        'to compute sin(angle)
Const cRadStep As Double = 0.0000001      'Converts the radian LONG to Double in Meth4

'Singles
Dim starttimer As Single        'ticks when routine starts
Dim stoptimer As Single         'ticks when rountine stops

'Doubles
Dim dummy As Double     'dummy variable to take sin(angle)

'Clears the screen for doing multiple runs
Private Sub cmdClear_Click()

    Cls

End Sub

'Method1 computes sin(angle) from 0 to 360 degrees in 0.01 degree increments.
'In Method1, the loop is a double, in contrast with the conventional wisdom
'to ALWAYS use Long Integers as loop counters.  Using a double in the loop
'eliminates a Type Conversion from Long to Double.
'
'The step size is a fraction of a degree, and the sin()
'function takes a double.  Both of these considerations
'provide the motivation for using a double precision variable as
'the loop counter.
    
Private Sub cmdMeth1_Click()

    Dim ItCounter As Long       'just an interaction counter

    Dim x As Double             'actual argument counter in degrees
    
    'start the timer
    starttimer = Timer
    
    For ItCounter = 1 To cNumIts
    
        'counter and step are doubles
        For x = 0 To 360 Step cDegStep
    
            'sin() gets double, no conversion necessary
            dummy = Sin(x * cDeg2Rad)
        
        Next x
    
    Next ItCounter
    
    'stop the timer
    stoptimer = Timer
    
    'Display the result
    Print
    Print "Using a Double loop counter in degrees units, "
    Print "Computed y=sin(x) "; cNumIts; " times "
    Print "in "; stoptimer - starttimer; " seconds."

End Sub

'Method2 performs the same computation as Method1, except that the
'argument loop counter is a Long Integer running from 0 to 36000,
'then has the counter multiplied by 0.01 (incorporated into the conversion
'value ccDeg2Rad).
'
'This approach follows the conventional wisdom that loop counters
'should ALWAYS be Long Integers.  However, it forces a type conversion
'from Long Integer to Double BEFORE the argument can be passed to
'the sin() function.

Private Sub cmdMeth2_Click()

    Dim ItCounter As Long       'iteration counter
    Dim Counter As Long         'actual argument counter, as Long
    
    'start the timer
    starttimer = Timer
    
    For ItCounter = 1 To cNumIts
    
        'loop using a Long
        For Counter = 0 To 36000 Step 1
    
            'sin() gets double, so counter needs conversion
            'in test, it made no difference if CDbl included explicitly
            'but is included as 'better style.'
            dummy = Sin(CDbl(Counter) * ccDeg2Rad)
        
        Next Counter
    
    Next ItCounter
    
    stoptimer = Timer       'stop the timer
    
    'Display the Results
    Print
    Print "Using a LONG loop counter in degree units, "
    Print "Computed y=sin(x) "; cNumIts; " times "
    Print "in "; stoptimer - starttimer; " seconds."


End Sub

'Method3 is similar to Method1, except the computation is done
'directly in "radian space."  The step size is still 'specified'
'as 0.01 degrees.
'
'Method3 elimates the conversion from degree to radians, so it is
'obvious that is should run faster than Method1.  A Double is used
'as the argument counter, so no type conversion is necessary.

Private Sub cmdMeth3_Click()

    Dim ItCounter As Long       'iteration counter

    Dim arg1 As Double          'actual argument as double, also
                                'the argument counter
    
    'start the timer
    starttimer = Timer
    
    'compute multiple times
    For ItCounter = 1 To cNumIts
    
        'no degree to radian conversion needed, so float math
        'eliminated from inside the loop!
    
        'counter and step are double
        For arg1 = 0 To c2Pi Step ccDeg2Rad
    
            'sin() gets double
            dummy = Sin(arg1)
        
        Next arg1
    
    Next ItCounter
    
    'stop the timer
    stoptimer = Timer
    
    'Display the result
    Print
    Print "Using a DOUBLE loop counter in radian units, "
    Print "Computed y=sin(x) "; cNumIts; " times "
    Print "in "; stoptimer - starttimer; " seconds."

End Sub

'Method4 is similar to Method2, except that it is performed in
''radian space.'  The argument counter is a Long which is converted
'to a double.
'
'It may seem curious that this method is the slowest of the four.
'That is, it does not follow the 'pattern' that Meth3 is faster
'than Meth1, suggesting that Meth4 should be faster than Meth2.
'The reason for this is that, in the compiled object code, an actual ADD opcode must be used
'vs. an INC opcode.

Private Sub cmdMeth4_Click()

    Dim ItCounter As Long       'iteration counter
    Dim Counter As Long         'argument counter as long

    Dim arg1 As Double          'actual argument variable, in radians
    
    'start the timer
    starttimer = Timer
    
    For ItCounter = 1 To cNumIts
    
        'counter and step are longs
        For Counter = 0 To 62831853 Step 1745
    
            'sin() gets double, type conversion is needed
            dummy = Sin(CDbl(Counter) * cRadStep)
        
        Next Counter
    
    Next ItCounter
    
    'stop the timer
    stoptimer = Timer
    
    'display the result
    Print
    Print "Using a LONG loop counter in radian units, "
    Print "Computed y=sin(x) "; cNumIts; " times "
    Print "in "; stoptimer - starttimer; " seconds."


End Sub
