VERSION 5.00
Begin VB.Form frmData07 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "Data07 - Cache Variable vs. Structure Access Demo"
   ClientHeight    =   3195
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   5055
   Icon            =   "Data07.frx":0000
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   3195
   ScaleWidth      =   5055
   StartUpPosition =   2  'CenterScreen
   Begin VB.CommandButton cmdClear 
      Caption         =   "Clear Text"
      Height          =   615
      Left            =   3240
      TabIndex        =   2
      Top             =   1920
      Width           =   1455
   End
   Begin VB.CommandButton cmdCached 
      Caption         =   "Compute using Cache Variables"
      Height          =   615
      Left            =   3240
      TabIndex        =   1
      Top             =   1080
      Width           =   1455
   End
   Begin VB.CommandButton cmdMembers 
      Caption         =   "Compute using Member Access"
      Height          =   615
      Left            =   3240
      TabIndex        =   0
      Top             =   240
      Width           =   1455
   End
End
Attribute VB_Name = "frmData07"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'This program demonstrates using of simple 'cache' variables
'to store object members vs. accessing the object members
'directly using the "." notation.
'
'In this demo, only some simple floating point math is done
'on the data to force several repeated memory accesses.

Option Explicit

'module global declarations
'constants
Const cNumIts As Long = 20000000   'number of iterations

'longs
Dim ItCounter As Long           'iteration counter

'singles
Dim starttimer As Single        'system 'ticks' at start of loop
Dim stoptimer As Single         'system 'ticks' at loop exit
Dim a As Single                 'dummy results for several
Dim b As Single                 'math operations
Dim c As Single

'early object binding to class
Dim ThisStructure As CAStructure     'AStructure is declared in
                                    'the module Data07Declares

'performs some simple math after caching the data in "simple"
'variables
Private Sub cmdCached_Click()

    'local declarations
    Dim x As Single             'simple cache variable for member .x
    Dim y As Single             'simple cache variable for member .y
    Dim z As Single             'simple cache variable for member .z

    'start the timer
    starttimer = Timer

    'cache the data; this is included inside the timing loop
    'to make a 'fair' comparison, though these assignments are
    'neglible compared to the loop execution
    x = ThisStructure.x         'cache .x
    y = ThisStructure.y         'cache .y
    z = ThisStructure.z         'cache .z
    
    For ItCounter = 1 To cNumIts
    
        'do some simple floating point math to force some
        'data access
        a = x + y + z
        b = x * y * z
        c = x * x + y * y + z * z

    Next ItCounter

    'stop the timer
    stoptimer = Timer

    'display the results
    Print "Using data cached in simple variables,"
    Print "computed some math "; cNumIts; " times in"
    Print stoptimer - starttimer; " seconds."

End Sub

'clears the print area for multiple runs
Private Sub cmdClear_Click()

    Cls

End Sub

Private Sub cmdMembers_Click()

    'start the timer
    starttimer = Timer

    For ItCounter = 1 To cNumIts

        'do some floating point math to force memory references
        a = ThisStructure.x + ThisStructure.y + ThisStructure.z
        b = ThisStructure.x * ThisStructure.y * ThisStructure.z
        c = ThisStructure.x * ThisStructure.x + ThisStructure.y * _
            ThisStructure.y + ThisStructure.z * ThisStructure.z

    Next ItCounter

    'stop the timer
    stoptimer = Timer

    'display the results
    Print "Using member access directly,"
    Print "computed some math "; cNumIts; " times in"
    Print stoptimer - starttimer; " seconds."
    Print

End Sub

'initialization up start-up
Private Sub Form_Load()

    'instantiate the class
    Set ThisStructure = New CAStructure

    'assign some values
    With ThisStructure

        .x = 123.4
        .y = 16.453
        .z = -0.00345
    
    End With

End Sub
