VERSION 5.00
Begin VB.Form frmFunction01 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "Function01 - Function Calls vs. Inline Code Demo"
   ClientHeight    =   3195
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   5025
   Icon            =   "frmFunction01.frx":0000
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   3195
   ScaleWidth      =   5025
   StartUpPosition =   2  'CenterScreen
   Begin VB.CommandButton cmdClear 
      Caption         =   "Clear Text"
      Height          =   495
      Left            =   3480
      TabIndex        =   2
      Top             =   1800
      Width           =   1335
   End
   Begin VB.CommandButton cmdInLine 
      Caption         =   "Repetitive Inline Code"
      Height          =   495
      Left            =   3480
      TabIndex        =   1
      Top             =   1080
      Width           =   1335
   End
   Begin VB.CommandButton cmdFunctionCall 
      Caption         =   "Repetitive Function Calls"
      Height          =   495
      Left            =   3480
      TabIndex        =   0
      Top             =   360
      Width           =   1335
   End
End
Attribute VB_Name = "frmFunction01"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'This program compares the use of inline functions
'to function calling.

Option Explicit

'global declarations
'constants
Const cNumIts As Long = 100000000

'This function would do the LJ Energy computation; in this demo,
'no computational work is being done.  However, to properly 'stress'
'the calling provisions in the compiler, the local variables must
'be declared and a return value must be generated.
'
'These operations require stack manipulations so that when a function
'is iteratively called, cycles are wasted pushing and popping data.

Private Function Calc_LJEnergy(R As Double) As Double

    'local declarations
    'doubles
    Dim r1 As Double        'used internally in the optimized LJ calc
    Dim r2 As Double
    
    'return a value
    Calc_LJEnergy = 0
    
End Function

Private Sub cmdClear_Click()

    Cls

End Sub

'This procedure implements the repetitive calling of the Calc_LJEnergy
'function (which in this demo, does no computational work).  A double
'passed and a double is returned.

Private Sub cmdFunctionCall_Click()

    'local declarations
    'longs
    Dim Counter As Long             'loop counter
    
    'singles
    Dim StartTimer As Single        'system ticks at start of loop
    Dim StopTimer As Single         'system ticks at end of loop
    
    'doubles
    Dim R As Double                 'distance, a double to pass to function
    Dim Energy As Double            'energy, holder for function return value
    
    'initialize a value
    R = 0                           'explicitly initialize R
    
    'start the timer
    StartTimer = Timer
    
    For Counter = 1 To cNumIts
    
        'do the function call, pushing R on the stack
        'a double is popped from the stack as return value
        Energy = Calc_LJEnergy(R)

    Next Counter
    
    'stop the timer
    StopTimer = Timer
    
    
    'display the result
    Print cNumIts; " function calls made in"
    Print StopTimer - StartTimer; " seconds."
    Print
    

End Sub

'Inline version...declarations are done OUTSIDE main loop and there
'is no parameter or return value passing.

Private Sub cmdInLine_Click()

    'local declarations
    'longs
    Dim Counter As Long             'iteration counter
    
    'singles
    Dim StartTimer As Single        'ticks at start of loop
    Dim StopTimer As Single         'ticks at loop exit
    
    'doubles
    Dim R As Double                 'values would be needed
    Dim r1 As Double                'for the inline energy calc
    Dim r2 As Double
    Dim Energy As Double            'computed energy
    
    'start the timer
    StartTimer = Timer
    
    For Counter = 1 To cNumIts
        
        ' no work in loop; calc code goes here
        'assign a value, since that at least would be done
        Energy = 0
        
    Next Counter
    
    'stop the timer
    StopTimer = Timer
    
    'display the result
    Print cNumIts; " Inline computations in"
    Print StopTimer - StartTimer; " seconds."
    Print

End Sub
