VERSION 5.00
Begin VB.Form frmFunction05 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "Function05 - Demo's the use of Global Data vs. Passing Parameters"
   ClientHeight    =   3285
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   6525
   Icon            =   "Function05.frx":0000
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   3285
   ScaleWidth      =   6525
   StartUpPosition =   2  'CenterScreen
   Begin VB.CommandButton cmdClear 
      Caption         =   "Clear Text"
      Height          =   615
      Left            =   4800
      TabIndex        =   3
      Top             =   2520
      Width           =   1695
   End
   Begin VB.CommandButton cmdByRefCall 
      Caption         =   "Call using PASSED Parameter, By Reference"
      Height          =   615
      Left            =   4800
      TabIndex        =   2
      Top             =   960
      Width           =   1695
   End
   Begin VB.CommandButton cmdByValCall 
      Caption         =   "Call using PASSED Parameter, By Value"
      Height          =   615
      Left            =   4800
      TabIndex        =   1
      Top             =   240
      Width           =   1695
   End
   Begin VB.CommandButton cmdGlobalCall 
      Caption         =   "Call using GLOBAL data access"
      Height          =   615
      Left            =   4800
      TabIndex        =   0
      Top             =   1680
      Width           =   1695
   End
End
Attribute VB_Name = "frmFunction05"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'This program demonstrates the use of global variables
'as faster code than passing parameters to a function.
'
'No work is done in the functions...this compares passing
'data to using global data (which does not require being passed
'on the stack) rather than the actual work done.

Option Explicit

'Global Declarations
'Constants
Const cNumIts As Long = 50000000     'number of iterations

'Singles
Dim StartTimer As Single        'system ticks at start of loop
Dim StopTimer As Single         'system ticks at loop exit

'Doubles
Dim Energy As Double        'global for computed energy
Dim r As Double             'global for 3d distance
Dim delX As Double          'global for x2-x1
Dim delY As Double          'global for y2-y1
Dim delZ As Double          'global for z2-z1

'This procedure computes the distance and energy by calling
'a function via passing parameters by reference.

Private Sub cmdByRefCall_Click()
    
    'Local declarations
    'Longs
    Dim Counter As Long             'iteration counter
    
    'Doubles
    Dim RefEnergy As Double     'return value for the call
    
    StartTimer = Timer          'start the timer
    
    For Counter = 1 To cNumIts
    
        'call the 'BYREF' function
        RefEnergy = Compute_Energy_Ref(delX, delY, delZ)
    
    Next Counter

    StopTimer = Timer       'stop the timer
    
    'display the results
    Print
    Print "Called Energy Function by reference "; cNumIts; " times in"
    Print StopTimer - StartTimer; " seconds."

End Sub

'This procedure computes the distance and energy by calling
'a function with BYVAL parameter passing.

Private Sub cmdByValCall_Click()

    'Local Declarations
    'Longs
    Dim Counter As Long             'iteration counter
    
    'Doubles
    Dim ValEnergy As Double         'return value for computed energy
    
    StartTimer = Timer      'start the timer
    
    For Counter = 1 To cNumIts

        'call the "BYVAL" function
        ValEnergy = Compute_Energy_Val(delX, delY, delZ)

    Next Counter
    
    StopTimer = Timer          'stop the timer
    
    'Display the results
    Print
    Print "Called Energy Function by value "; cNumIts; " times in"
    Print StopTimer - StartTimer; " seconds."

End Sub

'clears the output for multiple tests

Private Sub cmdClear_Click()
    
    Cls
    
End Sub

'This procedure computes the distance and energy using a function
'with no parameters passed; rather, the function works with and on
'global data.

Private Sub cmdGlobalCall_Click()

    'Local Declarations
    'longs
    Dim Counter As Long         'iteration counter
    
    StartTimer = Timer          'start the timer
    
    For Counter = 1 To cNumIts
    
        'call the procedure that works on and with global data
        'note that the return value is a global as well, so the
        'subroutine does not have to push a return value on the stack
        Call Compute_Energy_Global
    
    Next Counter

    StopTimer = Timer       'stop the timer
    
    Print
    Print "Called Energy Function using global data "; cNumIts; " times in"
    Print StopTimer - StartTimer; " seconds."

End Sub

'initialization code...all that is needed is to initialize
'the coordinate differences.

Private Sub Form_Load()

    delX = 1.567
    delY = 2.066
    delZ = 1.439

End Sub

'procedure to compute distance and energy using global
'data...not a function since no parameter is passed or
'returned.
'
'There is no dynamic allocation of local data space in this routine.

Private Sub Compute_Energy_Global()


End Sub

'Function to compute the distance and energy using
'BYVAL parameter passing, and passing the result in the
'return value.
'
'Memory is allocated at each call for the passed parameters
'as well as any locally declared variables.

Private Function Compute_Energy_Val(ByVal dX As Double, _
            ByVal dY As Double, ByVal dZ As Double) As Double

    'local declarations
    'Doubles
    Dim RVal As Double      'holder for the distance

    'return the value
    Compute_Energy_Val = RVal

End Function

'Function to compute the distance and energy using
'BYREF parameter passing, and passing the result in the
'return value.
'
'Memory is allocated at each call for the passed parameters
'as well as any locally declared variables.

Private Function Compute_Energy_Ref(ByRef dX As Double, _
            ByRef dY As Double, ByRef dZ As Double) As Double

    'Local Declarations
    'Doubles
    Dim RRef As Double
    

    'return the value
    Compute_Energy_Ref = RRef

End Function
