VERSION 5.00
Begin VB.Form frmLoop01 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "Loop01 - Rolled and Unrolled Function Assignment"
   ClientHeight    =   5280
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   6615
   Icon            =   "loop01.frx":0000
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   5280
   ScaleWidth      =   6615
   StartUpPosition =   2  'CenterScreen
   Begin VB.CommandButton Command3 
      Caption         =   "Clear Text"
      Height          =   855
      Left            =   5040
      TabIndex        =   2
      Top             =   4200
      Width           =   1455
   End
   Begin VB.CommandButton Command2 
      Caption         =   "Unrolled Loop"
      Height          =   975
      Left            =   5040
      TabIndex        =   1
      Top             =   3000
      Width           =   1455
   End
   Begin VB.CommandButton Command1 
      Caption         =   "Loop Rolled Up"
      Height          =   975
      Left            =   5040
      TabIndex        =   0
      Top             =   1800
      Width           =   1455
   End
   Begin VB.Label Label1 
      Caption         =   "Assign Function f(r) = exp(-r)"
      Height          =   855
      Left            =   5040
      TabIndex        =   3
      Top             =   360
      Width           =   1455
   End
End
Attribute VB_Name = "frmLoop01"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
' This primitive program is not using any algorithmic
' or programmatic optimizations, it simply serves to show the cost of
' loop overhead.  An exponential computation (ie, inverse natural log)
' is done 10,000,000 times with double precision arguements.  In a real
' computation, if expvalu were constant, it would be declared as such
' so compiler could insert data instead of memory pointers.
'
' Since many programmers like to program in a 'vector' style, using many small loops
' to keep track of 'dimensions,' loop overhead can add up.
'
' The executable was compiled in VB 6.0 Pro with following compiler settings:
'
' (1) Native Code, optimized for fast
' (2) No Advanced Options
'
' Execution times on Test machine were:
'
'   Rolled Loop (with actual looping):      6.2 seconds
'   Unrolled Loop (no loop overhead):       4.8 seconds
'
' representing a gain of 23%.

Option Explicit

Dim counter As Long
Dim counter2 As Long

Dim timestop As Single
Dim timestart As Single

Dim expvalu As Double
Dim r(10) As Double

' command1 computes exp(-argument*r) for r 1 through 10 10,000,000 times
' in a loop.

Private Sub Command1_Click()

    expvalu = 3.13      'just a value to use
            
    timestart = Timer   ' start timer
    
    For counter = 1 To 10000000 Step 1
    
        For counter2 = 1 To 10
        
            r(counter2) = Exp(-counter2 * expvalu)  'just a sample calculation
            
        Next counter2
        
    Next counter
    
    timestop = Timer    ' stop timer
    
    Print "Calculated exp(-r) vs. r 10,000,000 times in"
    Print: Print Str(timestop - timestart) & "  seconds"
    Print

End Sub

' command2 executes the same calculation the same number of times, but
' with no loop overhead.  Hard-Coding like this is obviously more work,
' but where pure performance is needed, the gains will add up.  Again, since
' much programming uses many many such small loops throughout, eliminating can
' show quite an overall benefit.  In the context of vectorization, it is certainly
' possible to hard code rather than use for-next or do loops.

Private Sub Command2_Click()
    
    expvalu = 3.13      'set to a value...
        
    timestart = Timer   ' start the timer
    
    For counter = 1 To 10000000 Step 1
    
        r(1) = Exp(-1 * expvalu)    ' we could slick this further by eliminating
                                    ' multiplication by 1!!
        r(2) = Exp(-2 * expvalu)
        r(3) = Exp(-3 * expvalu)
        r(4) = Exp(-4 * expvalu)
        r(5) = Exp(-5 * expvalu)
        r(6) = Exp(-6 * expvalu)
        r(7) = Exp(-7 * expvalu)
        r(8) = Exp(-8 * expvalu)
        r(9) = Exp(-9 * expvalu)
        r(10) = Exp(-10 * expvalu)
        
    Next counter
    
    timestop = Timer    ' stop the timer
    
    Print "Calculated exp(-r) vs. r 10,000,000 times in"
    Print: Print Str(timestop - timestart) & "  seconds"
    Print

End Sub

' merely clears screen for multiple test runs; due to os
' overhead, better picture arises from averaging several
' runs

Private Sub Command3_Click()

    Cls

End Sub
