VERSION 5.00
Begin VB.Form frmData03 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "Loop03 - Generic Rolled vs. Explicit Unrolled Matrix Multiplication"
   ClientHeight    =   4470
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   6180
   Icon            =   "Data03.frx":0000
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   4470
   ScaleWidth      =   6180
   StartUpPosition =   3  'Windows Default
   Begin VB.CommandButton cmdClearText 
      Caption         =   "Clear Text"
      Enabled         =   0   'False
      Height          =   735
      Left            =   4080
      TabIndex        =   3
      Top             =   3000
      Width           =   1935
   End
   Begin VB.CommandButton cmdExplUnrolled 
      Caption         =   "Function for Explicit Matrix Multiply, Unrolled Loop"
      Enabled         =   0   'False
      Height          =   735
      Left            =   4080
      TabIndex        =   2
      Top             =   2040
      Width           =   1935
   End
   Begin VB.CommandButton cmdExplRolled 
      Caption         =   "Function for Explicit Matrix Multiply, Rolled Loop"
      Enabled         =   0   'False
      Height          =   735
      Left            =   4080
      TabIndex        =   1
      Top             =   1080
      Width           =   1935
   End
   Begin VB.CommandButton cmdnxnMatrix 
      Caption         =   "Function for n x n Matrix Multiply (Loop Rolled)"
      Height          =   735
      Left            =   4080
      TabIndex        =   0
      Top             =   120
      Width           =   1935
   End
End
Attribute VB_Name = "frmData03"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'
' This program compares the execution times for using a generic
' nxn matrix multiplication function to explicit functions, such as
' 4x4 and 8x8.
'
' In general, the elements of the nxn result matrix C (C = A * B) can be
' written
'
'           c(j,k) = SUM [a(j,l) * b(l,k)], summing l = 1 to n
'
' for j and k = 1 to n.  The generic function uses variables for
' j, k and l in the loop counters.  Explicit functions use hard-coded
' values for j, k and l if rolled up, and hard-codes summations if
' unrolled.
'
'
' Since the size of the matrices in the generic
' function is a variable, the loop MUST be rolled up.  This can be
' costly for 'small' loops (say < 20).
'
' By using explicit functions, the loops can be unrolled, since the size
' of the calculation within each function is not variable.  That is, the
' array elements can be hard-coded.  This gives a significant performance
' gain, especially for 'small' loops.
'
' The UNROLLED calculation took a while to type the code.
' This illustrates again the difference between high performance at
' run-time vs. short development time.
'
' For demo purposes, code randomly selects 4x4 or 8x8 matrix multiplication
' to show that the decision logic to select the correct unrolled function
' does not cost too much for the loss of generality over the nxn function.
' In random order, about half the calculations are 4x4 and about half are 20x20.
'
Option Explicit

'Global Declarations

'Contants
Const NumIts As Long = 500000         ' 500000 total matrix multiplications

'Integers
Dim MTypes(500000) As Integer       ' holder for random 4 or 8 so that each
                                    ' routine uses the same set of random numbers
                                    ' this is done so that there is no bias since
                                    ' compute times for 4x4 and 8x8 are different

'Singles
Dim starttimer As Single            ' time at start of loop
Dim stoptimer As Single             ' time at end of loop
Dim A(8, 8) As Single             ' Matrix A
Dim B(8, 8) As Single             ' Matrix B
Dim C(8, 8) As Single             ' Matrix C = A * B

'Clears the screen so multiple runs can be checked
Private Sub cmdClearText_Click()

    Cls

End Sub

'Calls the EXPLICIT matrix multiplier functions with ROLLED Loops
Private Sub cmdExplRolled_Click()

    'local declarations
    'longs
    Dim ItCounter As Long       'iteration loop variable
    Dim called20 As Long        'number of times 20x20 is called
    Dim called4 As Long         'number of times 4x4 is called
    
    'start the timer
    starttimer = Timer
    
    For ItCounter = 1 To NumIts
        
        'logic to select which function to call
        Select Case MTypes(ItCounter)
        
            Case 8  '8x8
        
                'call the explicit rolled function
                Call Matrix_Mult_8x8_Rolled
        
                'increment the 8x8 counter
                called20 = called20 + 1
        
            Case 4  '4x4
        
                'call the explicit rolled function
                Call Matrix_Mult_4x4_Rolled
        
                'increment the 4x4 counter
                called4 = called4 + 1
        
        End Select
    
    Next ItCounter

    'stop the timer
    stoptimer = Timer
    
    'Display results
    Print "Rolled 8x8 called "; called20; " times"
    Print "Rolled 4x4 called "; called4; " times"
    Print stoptimer - starttimer; "Seconds"
    Print
    
End Sub

'Calls the Explicit 8x8 or 4x4 Functions using UNROLLED LOOPS
Private Sub cmdExplUnrolled_Click()

    'local declarations
    'longs
    Dim ItCounter As Long       'iteration loop variable
    Dim called20 As Long        'number of times 8x8 is called
    Dim called4 As Long         'number of times 4x4 is called
    
    'Display computation type
    Print "Computing Explicit Matrix Multiplications "
    Print "with UNROLLED Loops ..."
    
    'start the timer
    starttimer = Timer
    
    For ItCounter = 1 To NumIts
        
        'logic to select which function to call
        Select Case MTypes(ItCounter)
        
            Case 8  '8x8
        
                'call the explicit unrolled function
                Call Matrix_Mult_8x8_Unrolled
        
                'increment the call counter
                called20 = called20 + 1
        
            Case 4  '4x4
        
                'call the explicit unrolled function
                Call Matrix_Mult_4x4_UnRolled
        
                'increment the call counter
                called4 = called4 + 1
        
        End Select
    
    Next ItCounter
    
    'stop the timer
    stoptimer = Timer
    
    'display the results
    Print "Unrolled 8x8 called "; called20; " times"
    Print "Unrolled 4x4 called "; called4; " times in "
    Print stoptimer - starttimer; "Seconds"
    Print
    
End Sub

'calls the generic nxn matrix multiplication function with matrix
'size as a calling parameter
Private Sub cmdnxnMatrix_Click()
    
    Dim ItCounter As Long       'iteration loop variable
    Dim called20 As Long        'number of times nxn is called with dimension 20
    Dim called4 As Long         'number of times nxn is called with dimension 4
    
    'display computation type
    Print "Computing nxn Matrix Multiply "
    Print "with decision logic ..."
    
    starttimer = Timer          'start the timer
    
    For ItCounter = 1 To NumIts     'do NumIts matrix multiplies
        
        Select Case MTypes(ItCounter)   ' logic to select calling parameter
                                        ' the calling code could be streamlined to
                                        ' single line
                                        '
                                        ' nxn_Matrix_Mult(MTypes(ItCounter))
                                        '
                                        ' but decision logic would still be needed
                                        ' to increment either called20 or called4
                                        ' counters.
                                        '
                                        ' Since all three 'versions' use the exact same
                                        ' random number sequence and the same decision
                                        ' logic, this code really is comparing the compute
                                        ' times of the FUNCTIONS themselves (albeit, the
                                        ' nxn is having to pass a parameter).
                                        
            Case 8  '8x8
        
                Call nxn_Matrix_Mult(8)    'call nxn with dimension 20
        
                called20 = called20 + 1     ' increment the 20x20 counter
        
            Case 4  '4x4
        
                Call nxn_Matrix_Mult(4)     'call nxn with dimension 4
                
                called4 = called4 + 1       ' increment the 4x4 counter
        
        End Select
    
    Next ItCounter
    
    'stop the timer
    stoptimer = Timer

    'display the result
    Print "nxn( 8) called "; called20; " times"
    Print "nxn( 4) called "; called4; " times in"
    Print stoptimer - starttimer; "Seconds"
    Print
    
    'do the rolled generic nxn call without decision logic
    'the same random number array is used, so bias is eliminated
    'note that the call counters are not used here
    
    'display type of computation
    Print "Computing nxn Matrix Multiply"
    Print "without decision logic ..."
    
    'start the timer
    starttimer = Timer
    
    For ItCounter = 1 To NumIts
    
        'call the generic routine, using the array value directly
        Call nxn_Matrix_Mult(MTypes(ItCounter))
        
    Next ItCounter
    
    'stop the timer
    stoptimer = Timer
    
    'display the result
    Print "Executed in "
    Print stoptimer - starttimer; "seconds"
    Print
    
End Sub

'Procedure to compute nxn matrix multiplication
'uses globals for the matrices and the result
'the dimension "n" is passed as parameter
'
'For all j and k <= n,
'
'       c(j,k) = SUM [ a(j,l) + b(l,k) ], SUM l = 1 to n
'
'so this procedure need THREE NESTED LOOPS of size n (and therefore
'the computation scales as n^3).
Private Sub nxn_Matrix_Mult(ByVal Dimen As Long)

    Dim j As Long       ' j loop variable
    Dim k As Long       ' k loop variable
    Dim l As Long       ' l loop variable
    
    'loop over j
    For j = 1 To Dimen
    
        'loop over k
        For k = 1 To Dimen
        
            'zero out c(j,k)
            C(j, k) = 0
            
            'compute c(j,k)using rolled loop
            For l = 1 To Dimen
            
                C(j, k) = C(j, k) + A(j, l) * B(l, k)
                
            Next l
            
        Next k
        
    Next j

End Sub

'Entry Procedure ... doing some initialization
Private Sub Form_Load()

    Dim j As Long       'j loop counter
    Dim k As Long       'k loop counter
    Dim ItCounter As Long       'iteration loop variable
    
    'set A and B to 8x8 matrices
    'for the 4x4 computations, just use
    'the 4x4 portion in upper left 'quadrant'
    For j = 1 To 8
                        
        For k = 1 To 8
        
            'assign random values to A and B to 'simulate'
            'a real calculation
            A(j, k) = 20000 * Rnd
            B(j, k) = 20000 * Rnd

        Next k
        
    Next j
    
    'assign random 4 or 8 to MTypes, which will be used
    'to select 4x4 or 8x8
    For ItCounter = 1 To NumIts
    
        MTypes(ItCounter) = Int(2 * Rnd) + 1
        
        'assign actual dimension to array for the nxn
        'call without decision logic
        Select Case MTypes(ItCounter)
        
        Case 1
        
            MTypes(ItCounter) = 8
            
        Case 2
        
            MTypes(ItCounter) = 4
            
        End Select
    
    Next ItCounter
    
    'enable the other command buttons
    cmdClearText.Enabled = True
    cmdExplRolled.Enabled = True
    cmdExplUnrolled.Enabled = True
    cmdnxnMatrix.Enabled = True

End Sub

'Explicit 4x4 matrix multiplication with loops ROLLED-UP
'This is similar to the nxn Procedure, but does not involve
'passing a parameter and loop ranges are hard-coded.
Private Sub Matrix_Mult_4x4_Rolled()

    Dim j As Long       'j loop variable
    Dim k As Long       'k loop variable
    Dim l As Long       'l loop variable
    
    For j = 1 To 4
    
        For k = 1 To 4
        
            'zero C(j,k)
            C(j, k) = 0
            
            'compute c(j,k) in rolled loop
            For l = 1 To 4
            
                C(j, k) = C(j, k) + A(j, l) * B(l, k)
                
            Next l
            
        Next k
        
    Next j


End Sub

'Explicit 8x8 matrix multiplication with loops ROLLED-UP
'This is similar to the nxn Procedure, but does not involve
'passing a parameter and loop ranges are hard-coded.
Private Sub Matrix_Mult_8x8_Rolled()

    Dim j As Long       'j loop variable
    Dim k As Long       'k loop variable
    Dim l As Long       'l loop variable
    
    For j = 1 To 8
    
        For k = 1 To 8
        
            'zero c(j,k)
            C(j, k) = 0
            
            'Compute c(j,k) in rolled loop
            For l = 1 To 8
            
                C(j, k) = C(j, k) + A(j, l) * B(l, k)
                
            Next l
            
        Next k
        
    Next j

End Sub

'Procedure to compute 4x4 matrix multiplication with the all loops
'UNROLLED.  While longer, hard to read code, this eliminates all loop
'overhead, which can be significant for small loops that contain
'few lines of code within the loop.
Private Sub Matrix_Mult_4x4_UnRolled()

    'j = 1
    C(1, 1) = A(1, 1) * B(1, 1) + A(1, 2) * B(2, 1) + A(1, 3) * B(3, 1) + A(1, 4) * B(4, 1)
    C(1, 2) = A(1, 1) * B(1, 2) + A(1, 2) * B(2, 2) + A(1, 3) * B(3, 2) + A(1, 4) * B(4, 2)
    C(1, 3) = A(1, 1) * B(1, 3) + A(1, 2) * B(2, 3) + A(1, 3) * B(3, 3) + A(1, 4) * B(4, 3)
    C(1, 4) = A(1, 1) * B(1, 4) + A(1, 2) * B(2, 4) + A(1, 3) * B(3, 4) + A(1, 4) * B(4, 4)
    
    'j = 2
    C(2, 1) = A(2, 1) * B(1, 1) + A(2, 2) * B(2, 1) + A(2, 3) * B(3, 1) + A(2, 4) * B(4, 1)
    C(2, 2) = A(2, 1) * B(1, 2) + A(2, 2) * B(2, 2) + A(2, 3) * B(3, 2) + A(2, 4) * B(4, 2)
    C(2, 3) = A(2, 1) * B(1, 3) + A(2, 2) * B(2, 3) + A(2, 3) * B(3, 3) + A(2, 4) * B(4, 3)
    C(2, 4) = A(2, 1) * B(1, 4) + A(2, 2) * B(2, 4) + A(2, 3) * B(3, 4) + A(2, 4) * B(4, 4)
    
    'j=3
    C(3, 1) = A(3, 1) * B(1, 1) + A(3, 2) * B(2, 1) + A(3, 3) * B(3, 1) + A(3, 4) * B(4, 1)
    C(3, 2) = A(3, 1) * B(1, 2) + A(3, 2) * B(2, 2) + A(3, 3) * B(3, 2) + A(3, 4) * B(4, 2)
    C(3, 3) = A(3, 1) * B(1, 3) + A(3, 2) * B(2, 3) + A(3, 3) * B(3, 3) + A(3, 4) * B(4, 3)
    C(3, 4) = A(3, 1) * B(1, 4) + A(3, 2) * B(2, 4) + A(3, 3) * B(3, 4) + A(3, 4) * B(4, 4)
    
    'j=4
    C(4, 1) = A(4, 1) * B(1, 1) + A(4, 2) * B(2, 1) + A(4, 3) * B(3, 1) + A(4, 4) * B(4, 1)
    C(4, 2) = A(4, 1) * B(1, 2) + A(4, 2) * B(2, 2) + A(4, 3) * B(3, 2) + A(4, 4) * B(4, 2)
    C(4, 3) = A(4, 1) * B(1, 3) + A(4, 2) * B(2, 3) + A(4, 3) * B(3, 3) + A(4, 4) * B(4, 3)
    C(4, 4) = A(4, 1) * B(1, 4) + A(4, 2) * B(2, 4) + A(4, 3) * B(3, 4) + A(4, 4) * B(4, 4)
    

End Sub

'Procedure to compute 8x8 matrix multiplication with the all loops
'UNROLLED.  While longer, hard to read code, this eliminates all loop
'overhead, which can be significant for small loops that contain
'few lines of code within the loop.
Private Sub Matrix_Mult_8x8_Unrolled()

    'j = 1
    C(1, 1) = A(1, 1) * B(1, 1) + A(1, 2) * B(2, 1) + A(1, 3) * B(3, 1) + A(1, 4) * B(4, 1) + A(1, 5) * B(5, 1) + A(1, 6) * B(6, 1) + A(1, 7) * B(7, 1) + A(1, 8) * B(8, 1)
    C(1, 2) = A(1, 1) * B(1, 2) + A(1, 2) * B(2, 2) + A(1, 3) * B(3, 2) + A(1, 4) * B(4, 2) + A(1, 5) * B(5, 2) + A(1, 6) * B(6, 2) + A(1, 7) * B(7, 2) + A(1, 8) * B(8, 2)
    C(1, 3) = A(1, 1) * B(1, 3) + A(1, 2) * B(2, 3) + A(1, 3) * B(3, 3) + A(1, 4) * B(4, 3) + A(1, 5) * B(5, 3) + A(1, 6) * B(6, 3) + A(1, 7) * B(7, 3) + A(1, 8) * B(8, 3)
    C(1, 4) = A(1, 1) * B(1, 4) + A(1, 2) * B(2, 4) + A(1, 3) * B(3, 4) + A(1, 4) * B(4, 4) + A(1, 5) * B(5, 4) + A(1, 6) * B(6, 4) + A(1, 7) * B(7, 4) + A(1, 8) * B(8, 4)
    C(1, 5) = A(1, 1) * B(1, 5) + A(1, 2) * B(2, 5) + A(1, 3) * B(3, 5) + A(1, 4) * B(4, 5) + A(1, 5) * B(5, 5) + A(1, 6) * B(6, 5) + A(1, 7) * B(7, 5) + A(1, 8) * B(8, 5)
    C(1, 6) = A(1, 1) * B(1, 6) + A(1, 2) * B(2, 6) + A(1, 3) * B(3, 6) + A(1, 4) * B(4, 6) + A(1, 5) * B(5, 6) + A(1, 6) * B(6, 6) + A(1, 7) * B(7, 6) + A(1, 8) * B(8, 6)
    C(1, 7) = A(1, 1) * B(1, 7) + A(1, 2) * B(2, 7) + A(1, 3) * B(3, 7) + A(1, 4) * B(4, 7) + A(1, 5) * B(5, 7) + A(1, 6) * B(6, 7) + A(1, 7) * B(7, 7) + A(1, 8) * B(8, 7)
    C(1, 8) = A(1, 1) * B(1, 8) + A(1, 2) * B(2, 8) + A(1, 3) * B(3, 8) + A(1, 4) * B(4, 8) + A(1, 5) * B(5, 8) + A(1, 6) * B(6, 8) + A(1, 7) * B(7, 8) + A(1, 8) * B(8, 8)
    
    'j = 2
    C(2, 1) = A(2, 1) * B(1, 1) + A(2, 2) * B(2, 1) + A(2, 3) * B(3, 1) + A(2, 4) * B(4, 1) + A(2, 5) * B(5, 1) + A(2, 6) * B(6, 1) + A(2, 7) * B(7, 1) + A(2, 8) * B(8, 1)
    C(2, 2) = A(2, 1) * B(1, 2) + A(2, 2) * B(2, 2) + A(2, 3) * B(3, 2) + A(2, 4) * B(4, 2) + A(2, 5) * B(5, 2) + A(2, 6) * B(6, 2) + A(2, 7) * B(7, 2) + A(2, 8) * B(8, 2)
    C(2, 3) = A(2, 1) * B(1, 3) + A(2, 2) * B(2, 3) + A(2, 3) * B(3, 3) + A(2, 4) * B(4, 3) + A(2, 5) * B(5, 3) + A(2, 6) * B(6, 3) + A(2, 7) * B(7, 3) + A(2, 8) * B(8, 3)
    C(2, 4) = A(2, 1) * B(1, 4) + A(2, 2) * B(2, 4) + A(2, 3) * B(3, 4) + A(2, 4) * B(4, 4) + A(2, 5) * B(5, 4) + A(2, 6) * B(6, 4) + A(2, 7) * B(7, 4) + A(2, 8) * B(8, 4)
    C(2, 5) = A(2, 1) * B(1, 5) + A(2, 2) * B(2, 5) + A(2, 3) * B(3, 5) + A(2, 4) * B(4, 5) + A(2, 5) * B(5, 5) + A(2, 6) * B(6, 5) + A(2, 7) * B(7, 5) + A(2, 8) * B(8, 5)
    C(2, 6) = A(2, 1) * B(1, 6) + A(2, 2) * B(2, 6) + A(2, 3) * B(3, 6) + A(2, 4) * B(4, 6) + A(2, 5) * B(5, 6) + A(2, 6) * B(6, 6) + A(2, 7) * B(7, 6) + A(2, 8) * B(8, 6)
    C(2, 7) = A(2, 1) * B(1, 7) + A(2, 2) * B(2, 7) + A(2, 3) * B(3, 7) + A(2, 4) * B(4, 7) + A(2, 5) * B(5, 7) + A(2, 6) * B(6, 7) + A(2, 7) * B(7, 7) + A(2, 8) * B(8, 7)
    C(2, 8) = A(2, 1) * B(1, 8) + A(2, 2) * B(2, 8) + A(2, 3) * B(3, 8) + A(2, 4) * B(4, 8) + A(2, 5) * B(5, 8) + A(2, 6) * B(6, 8) + A(2, 7) * B(7, 8) + A(2, 8) * B(8, 8)
    
    'j=3
    C(3, 1) = A(3, 1) * B(1, 1) + A(3, 2) * B(2, 1) + A(3, 3) * B(3, 1) + A(3, 4) * B(4, 1) + A(3, 5) * B(5, 1) + A(3, 6) * B(6, 1) + A(3, 7) * B(7, 1) + A(3, 8) * B(8, 1)
    C(3, 2) = A(3, 1) * B(1, 2) + A(3, 2) * B(2, 2) + A(3, 3) * B(3, 2) + A(3, 4) * B(4, 2) + A(3, 5) * B(5, 2) + A(3, 6) * B(6, 2) + A(3, 7) * B(7, 2) + A(3, 8) * B(8, 2)
    C(3, 3) = A(3, 1) * B(1, 3) + A(3, 2) * B(2, 3) + A(3, 3) * B(3, 3) + A(3, 4) * B(4, 3) + A(3, 5) * B(5, 3) + A(3, 6) * B(6, 3) + A(3, 7) * B(7, 3) + A(3, 8) * B(8, 3)
    C(3, 4) = A(3, 1) * B(1, 4) + A(3, 2) * B(2, 4) + A(3, 3) * B(3, 4) + A(3, 4) * B(4, 4) + A(3, 5) * B(5, 4) + A(3, 6) * B(6, 4) + A(3, 7) * B(7, 4) + A(3, 8) * B(8, 4)
    C(3, 5) = A(3, 1) * B(1, 5) + A(3, 2) * B(2, 5) + A(3, 3) * B(3, 5) + A(3, 4) * B(4, 5) + A(3, 5) * B(5, 5) + A(3, 6) * B(6, 5) + A(3, 7) * B(7, 5) + A(3, 8) * B(8, 5)
    C(3, 6) = A(3, 1) * B(1, 6) + A(3, 2) * B(2, 6) + A(3, 3) * B(3, 6) + A(3, 4) * B(4, 6) + A(3, 5) * B(5, 6) + A(3, 6) * B(6, 6) + A(3, 7) * B(7, 6) + A(3, 8) * B(8, 6)
    C(3, 7) = A(3, 1) * B(1, 7) + A(3, 2) * B(2, 7) + A(3, 3) * B(3, 7) + A(3, 4) * B(4, 7) + A(3, 5) * B(5, 7) + A(3, 6) * B(6, 7) + A(3, 7) * B(7, 7) + A(3, 8) * B(8, 7)
    C(3, 8) = A(3, 1) * B(1, 8) + A(3, 2) * B(2, 8) + A(3, 3) * B(3, 8) + A(3, 4) * B(4, 8) + A(3, 5) * B(5, 8) + A(3, 6) * B(6, 8) + A(3, 7) * B(7, 8) + A(3, 8) * B(8, 8)
    
    'j=4
    C(4, 1) = A(4, 1) * B(1, 1) + A(4, 2) * B(2, 1) + A(4, 3) * B(3, 1) + A(4, 4) * B(4, 1) + A(4, 5) * B(5, 1) + A(4, 6) * B(6, 1) + A(4, 7) * B(7, 1) + A(4, 8) * B(8, 1)
    C(4, 2) = A(4, 1) * B(1, 2) + A(4, 2) * B(2, 2) + A(4, 3) * B(3, 2) + A(4, 4) * B(4, 2) + A(4, 5) * B(5, 2) + A(4, 6) * B(6, 2) + A(4, 7) * B(7, 2) + A(4, 8) * B(8, 2)
    C(4, 3) = A(4, 1) * B(1, 3) + A(4, 2) * B(2, 3) + A(4, 3) * B(3, 3) + A(4, 4) * B(4, 3) + A(4, 5) * B(5, 3) + A(4, 6) * B(6, 3) + A(4, 7) * B(7, 3) + A(4, 8) * B(8, 3)
    C(4, 4) = A(4, 1) * B(1, 4) + A(4, 2) * B(2, 4) + A(4, 3) * B(3, 4) + A(4, 4) * B(4, 4) + A(4, 5) * B(5, 4) + A(4, 6) * B(6, 4) + A(4, 7) * B(7, 4) + A(4, 8) * B(8, 4)
    C(4, 5) = A(4, 1) * B(1, 5) + A(4, 2) * B(2, 5) + A(4, 3) * B(3, 5) + A(4, 4) * B(4, 5) + A(4, 5) * B(5, 5) + A(4, 6) * B(6, 5) + A(4, 7) * B(7, 5) + A(4, 8) * B(8, 5)
    C(4, 6) = A(4, 1) * B(1, 6) + A(4, 2) * B(2, 6) + A(4, 3) * B(3, 6) + A(4, 4) * B(4, 6) + A(4, 5) * B(5, 6) + A(4, 6) * B(6, 6) + A(4, 7) * B(7, 6) + A(4, 8) * B(8, 6)
    C(4, 7) = A(4, 1) * B(1, 7) + A(4, 2) * B(2, 7) + A(4, 3) * B(3, 7) + A(4, 4) * B(4, 7) + A(4, 5) * B(5, 7) + A(4, 6) * B(6, 7) + A(4, 7) * B(7, 7) + A(4, 8) * B(8, 7)
    C(4, 8) = A(4, 1) * B(1, 8) + A(4, 2) * B(2, 8) + A(4, 3) * B(3, 8) + A(4, 4) * B(4, 8) + A(4, 5) * B(5, 8) + A(4, 6) * B(6, 8) + A(4, 7) * B(7, 8) + A(4, 8) * B(8, 8)
    
    'j=5
    C(5, 1) = A(5, 1) * B(1, 1) + A(5, 2) * B(2, 1) + A(5, 3) * B(3, 1) + A(5, 4) * B(4, 1) + A(5, 5) * B(5, 1) + A(5, 6) * B(6, 1) + A(5, 7) * B(7, 1) + A(5, 8) * B(8, 1)
    C(5, 2) = A(5, 1) * B(1, 2) + A(5, 2) * B(2, 2) + A(5, 3) * B(3, 2) + A(5, 4) * B(4, 2) + A(5, 5) * B(5, 2) + A(5, 6) * B(6, 2) + A(5, 7) * B(7, 2) + A(5, 8) * B(8, 2)
    C(5, 3) = A(5, 1) * B(1, 3) + A(5, 2) * B(2, 3) + A(5, 3) * B(3, 3) + A(5, 4) * B(4, 3) + A(5, 5) * B(5, 3) + A(5, 6) * B(6, 3) + A(5, 7) * B(7, 3) + A(5, 8) * B(8, 3)
    C(5, 4) = A(5, 1) * B(1, 4) + A(5, 2) * B(2, 4) + A(5, 3) * B(3, 4) + A(5, 4) * B(4, 4) + A(5, 5) * B(5, 4) + A(5, 6) * B(6, 4) + A(5, 7) * B(7, 4) + A(5, 8) * B(8, 4)
    C(5, 5) = A(5, 1) * B(1, 5) + A(5, 2) * B(2, 5) + A(5, 3) * B(3, 5) + A(5, 4) * B(4, 5) + A(5, 5) * B(5, 5) + A(5, 6) * B(6, 5) + A(5, 7) * B(7, 5) + A(5, 8) * B(8, 5)
    C(5, 6) = A(5, 1) * B(1, 6) + A(5, 2) * B(2, 6) + A(5, 3) * B(3, 6) + A(5, 4) * B(4, 6) + A(5, 5) * B(5, 6) + A(5, 6) * B(6, 6) + A(5, 7) * B(7, 6) + A(5, 8) * B(8, 6)
    C(5, 7) = A(5, 1) * B(1, 7) + A(5, 2) * B(2, 7) + A(5, 3) * B(3, 7) + A(5, 4) * B(4, 7) + A(5, 5) * B(5, 7) + A(5, 6) * B(6, 7) + A(5, 7) * B(7, 7) + A(5, 8) * B(8, 7)
    C(5, 8) = A(5, 1) * B(1, 8) + A(5, 2) * B(2, 8) + A(5, 3) * B(3, 8) + A(5, 4) * B(4, 8) + A(5, 5) * B(5, 8) + A(5, 6) * B(6, 8) + A(5, 7) * B(7, 8) + A(5, 8) * B(8, 8)
    
    'j = 6
    C(6, 1) = A(6, 1) * B(1, 1) + A(6, 2) * B(2, 1) + A(6, 3) * B(3, 1) + A(6, 4) * B(4, 1) + A(6, 5) * B(5, 1) + A(6, 6) * B(6, 1) + A(6, 7) * B(7, 1) + A(6, 8) * B(8, 1)
    C(6, 2) = A(6, 1) * B(1, 2) + A(6, 2) * B(2, 2) + A(6, 3) * B(3, 2) + A(6, 4) * B(4, 2) + A(6, 5) * B(5, 2) + A(6, 6) * B(6, 2) + A(6, 7) * B(7, 2) + A(6, 8) * B(8, 2)
    C(6, 3) = A(6, 1) * B(1, 3) + A(6, 2) * B(2, 3) + A(6, 3) * B(3, 3) + A(6, 4) * B(4, 3) + A(6, 5) * B(5, 3) + A(6, 6) * B(6, 3) + A(6, 7) * B(7, 3) + A(6, 8) * B(8, 3)
    C(6, 4) = A(6, 1) * B(1, 4) + A(6, 2) * B(2, 4) + A(6, 3) * B(3, 4) + A(6, 4) * B(4, 4) + A(6, 5) * B(5, 4) + A(6, 6) * B(6, 4) + A(6, 7) * B(7, 4) + A(6, 8) * B(8, 4)
    C(6, 5) = A(6, 1) * B(1, 5) + A(6, 2) * B(2, 5) + A(6, 3) * B(3, 5) + A(6, 4) * B(4, 5) + A(6, 5) * B(5, 5) + A(6, 6) * B(6, 5) + A(6, 7) * B(7, 5) + A(6, 8) * B(8, 5)
    C(6, 6) = A(6, 1) * B(1, 6) + A(6, 2) * B(2, 6) + A(6, 3) * B(3, 6) + A(6, 4) * B(4, 6) + A(6, 5) * B(5, 6) + A(6, 6) * B(6, 6) + A(6, 7) * B(7, 6) + A(6, 8) * B(8, 6)
    C(6, 7) = A(6, 1) * B(1, 7) + A(6, 2) * B(2, 7) + A(6, 3) * B(3, 7) + A(6, 4) * B(4, 7) + A(6, 5) * B(5, 7) + A(6, 6) * B(6, 7) + A(6, 7) * B(7, 7) + A(6, 8) * B(8, 7)
    C(6, 8) = A(6, 1) * B(1, 8) + A(6, 2) * B(2, 8) + A(6, 3) * B(3, 8) + A(6, 4) * B(4, 8) + A(6, 5) * B(5, 8) + A(6, 6) * B(6, 8) + A(6, 7) * B(7, 8) + A(6, 8) * B(8, 8)
    
    'j=7
    C(7, 1) = A(7, 1) * B(1, 1) + A(7, 2) * B(2, 1) + A(7, 3) * B(3, 1) + A(7, 4) * B(4, 1) + A(7, 5) * B(5, 1) + A(7, 6) * B(6, 1) + A(7, 7) * B(7, 1) + A(7, 8) * B(8, 1)
    C(7, 2) = A(7, 1) * B(1, 2) + A(7, 2) * B(2, 2) + A(7, 3) * B(3, 2) + A(7, 4) * B(4, 2) + A(7, 5) * B(5, 2) + A(7, 6) * B(6, 2) + A(7, 7) * B(7, 2) + A(7, 8) * B(8, 2)
    C(7, 3) = A(7, 1) * B(1, 3) + A(7, 2) * B(2, 3) + A(7, 3) * B(3, 3) + A(7, 4) * B(4, 3) + A(7, 5) * B(5, 3) + A(7, 6) * B(6, 3) + A(7, 7) * B(7, 3) + A(7, 8) * B(8, 3)
    C(7, 4) = A(7, 1) * B(1, 4) + A(7, 2) * B(2, 4) + A(7, 3) * B(3, 4) + A(7, 4) * B(4, 4) + A(7, 5) * B(5, 4) + A(7, 6) * B(6, 4) + A(7, 7) * B(7, 4) + A(7, 8) * B(8, 4)
    C(7, 5) = A(7, 1) * B(1, 5) + A(7, 2) * B(2, 5) + A(7, 3) * B(3, 5) + A(7, 4) * B(4, 5) + A(7, 5) * B(5, 5) + A(7, 6) * B(6, 5) + A(7, 7) * B(7, 5) + A(7, 8) * B(8, 5)
    C(7, 6) = A(7, 1) * B(1, 6) + A(7, 2) * B(2, 6) + A(7, 3) * B(3, 6) + A(7, 4) * B(4, 6) + A(7, 5) * B(5, 6) + A(7, 6) * B(6, 6) + A(7, 7) * B(7, 6) + A(7, 8) * B(8, 6)
    C(7, 7) = A(7, 1) * B(1, 7) + A(7, 2) * B(2, 7) + A(7, 3) * B(3, 7) + A(7, 4) * B(4, 7) + A(7, 5) * B(5, 7) + A(7, 6) * B(6, 7) + A(7, 7) * B(7, 7) + A(7, 8) * B(8, 7)
    C(7, 8) = A(7, 1) * B(1, 8) + A(7, 2) * B(2, 8) + A(7, 3) * B(3, 8) + A(7, 4) * B(4, 8) + A(7, 5) * B(5, 8) + A(7, 6) * B(6, 8) + A(7, 7) * B(7, 8) + A(7, 8) * B(8, 8)
    
    'j=8
    C(8, 1) = A(8, 1) * B(1, 1) + A(8, 2) * B(2, 1) + A(8, 3) * B(3, 1) + A(8, 4) * B(4, 1) + A(8, 5) * B(5, 1) + A(8, 6) * B(6, 1) + A(8, 7) * B(7, 1) + A(8, 8) * B(8, 1)
    C(8, 2) = A(8, 1) * B(1, 2) + A(8, 2) * B(2, 2) + A(8, 3) * B(3, 2) + A(8, 4) * B(4, 2) + A(8, 5) * B(5, 2) + A(8, 6) * B(6, 2) + A(8, 7) * B(7, 2) + A(8, 8) * B(8, 2)
    C(8, 3) = A(8, 1) * B(1, 3) + A(8, 2) * B(2, 3) + A(8, 3) * B(3, 3) + A(8, 4) * B(4, 3) + A(8, 5) * B(5, 3) + A(8, 6) * B(6, 3) + A(8, 7) * B(7, 3) + A(8, 8) * B(8, 3)
    C(8, 4) = A(8, 1) * B(1, 4) + A(8, 2) * B(2, 4) + A(8, 3) * B(3, 4) + A(8, 4) * B(4, 4) + A(8, 5) * B(5, 4) + A(8, 6) * B(6, 4) + A(8, 7) * B(7, 4) + A(8, 8) * B(8, 4)
    C(8, 5) = A(8, 1) * B(1, 5) + A(8, 2) * B(2, 5) + A(8, 3) * B(3, 5) + A(8, 4) * B(4, 5) + A(8, 5) * B(5, 5) + A(8, 6) * B(6, 5) + A(8, 7) * B(7, 5) + A(8, 8) * B(8, 5)
    C(8, 6) = A(8, 1) * B(1, 6) + A(8, 2) * B(2, 6) + A(8, 3) * B(3, 6) + A(8, 4) * B(4, 6) + A(8, 5) * B(5, 6) + A(8, 6) * B(6, 6) + A(8, 7) * B(7, 6) + A(8, 8) * B(8, 6)
    C(8, 7) = A(8, 1) * B(1, 7) + A(8, 2) * B(2, 7) + A(8, 3) * B(3, 7) + A(8, 4) * B(4, 7) + A(8, 5) * B(5, 7) + A(8, 6) * B(6, 7) + A(8, 7) * B(7, 7) + A(8, 8) * B(8, 7)
    C(8, 8) = A(8, 1) * B(1, 8) + A(8, 2) * B(2, 8) + A(8, 3) * B(3, 8) + A(8, 4) * B(4, 8) + A(8, 5) * B(5, 8) + A(8, 6) * B(6, 8) + A(8, 7) * B(7, 8) + A(8, 8) * B(8, 8)
    

End Sub

