VERSION 5.00
Begin VB.Form frmLoop04 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "Loop04 - Rolled vs. Partially Unrolled Simpson's Rule Integration"
   ClientHeight    =   3900
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   6300
   Icon            =   "Loop04.frx":0000
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   3900
   ScaleWidth      =   6300
   StartUpPosition =   2  'CenterScreen
   Begin VB.CommandButton cmdUnroll8 
      Caption         =   "Integrate x^5 with loop unrolled by eight terms"
      Height          =   615
      Left            =   4080
      TabIndex        =   4
      Top             =   2400
      Width           =   2055
   End
   Begin VB.CommandButton cmdUnroll4 
      Caption         =   "Integrate x^5 with loop unrolled by four terms"
      Height          =   615
      Left            =   4080
      TabIndex        =   3
      Top             =   960
      Width           =   2055
   End
   Begin VB.CommandButton cmdRolled 
      Caption         =   "Integrate x^5 using fully rolled loops"
      Height          =   615
      Left            =   4080
      TabIndex        =   2
      Top             =   240
      Width           =   2055
   End
   Begin VB.CommandButton cmdClearText 
      Caption         =   "Clear Text"
      Height          =   615
      Left            =   4080
      TabIndex        =   1
      Top             =   3120
      Width           =   2055
   End
   Begin VB.CommandButton cmdSimpsons 
      Caption         =   "Integrate x^5 with loop unrolled by five terms"
      Height          =   615
      Left            =   4080
      TabIndex        =   0
      Top             =   1680
      Width           =   2055
   End
End
Attribute VB_Name = "frmLoop04"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
' This program compares the execution times for a repetitive Simpson's Rule
' integration with the summation loop fully rolled-up or partially unrolled.
' The technique of partial unrolling is useful in cases where full unrolling
' is not feasible.
'
' Recall that the integral, I, of a function that is divided into 2n steps
' (Simpson's Rule requires an even number of steps) can be computed by
'
'               I = h/3 * (s0 + 2s1 + 4s2)
'
' where h = (b-a)/2n, s0 = f(0) + f(2n), s1 = f(1) + f(3) + ...f(2n-1) and
' s(2) = f(2) + f(4) + ... f(2n-2).
'
' On the test computer, the partially unrolled code executed three times faster
' than the fully rolled code.
'
' This equated to 168,000 cycles for the rolled code for one integration compared
' to 60,000 cycles for one integration for the unrolled version.

Option Explicit

Const cNumIts As Long = 50000   ' number of iterations

Dim F(10000) As Single          ' function to be integrated

Private Sub cmdClearText_Click()

    Cls

End Sub

'This procedure performs the Simpson's rule integration using
'the fully rolled-up loops.  In general, this procedure trades
'short source code and quick typing time for execution time

Private Sub cmdRolled_Click()

    'local declarations
    'longs
    Dim SimpCounter As Long     'loop counter for the summation
    Dim ItCounter As Long       'iteration counter
    
    'singles
    Dim h_3 As Single           'Simpson's rule parameters
    Dim S0 As Single
    Dim S1 As Single
    Dim S2 As Single
    Dim Integral As Single
    Dim StartTimer As Single    'system ticks at start of loop
    Dim StopTimer As Single     'system ticks at loop exit
    
    'start the timer
    StartTimer = Timer
    
    For ItCounter = 1 To cNumIts
    
        h_3 = 0.999 / 30000     ' h/3, where h=(b-a)/n
    
        S0 = F(0) + F(9999)     ' sum the first and last function values
        S1 = 0                  ' zero values for the running summations to follow
        S2 = 0
    
        'sum terms except first and last
        For SimpCounter = 1 To 9997 Step 2
    
            S1 = S1 + F(SimpCounter)
            S2 = S2 + F(SimpCounter + 1)
    
        Next SimpCounter

        ' 'odd' terms * 2
        S1 = S1 * 2
        
        ' 'even' terms * 4
        S2 = S2 * 4
    
        'full summation
        Integral = S0 + S1 + S2
        
        'multiply by step
        Integral = Integral * h_3
    
    Next ItCounter
    
    'stop the timer
    StopTimer = Timer
    
    'display the results
    Print "By fully rolled-up loops:"
    Print "Integral = "; Integral; " "; cNumIts; " times in"
    Print StopTimer - StartTimer; " seconds."
    Print


End Sub

'This procedure computes the integral using a partially unrolled loop: the
'loop is unrolled with five terms explicitly coded and the loop is stepped
'accordingly.  Partial unrolling reaches a diminishing return.
'
Private Sub cmdSimpsons_Click()

    Dim SimpCounter As Long     'loop counter for the summation
    Dim ItCounter As Long       'iteration counter
    
    Dim h_3 As Single           'Simpson's Rule parameters
    Dim S0 As Single
    Dim S1 As Single
    Dim S2 As Single
    Dim Integral As Single      'Integral result
    Dim StartTimer As Single    'clock ticks at start of loop
    Dim StopTimer As Single     'clock ticks at loop exit
    
    StartTimer = Timer          'start the timer
    
    For ItCounter = 1 To cNumIts
    
        h_3 = 0.999 / 30000     ' h/3, where h=(b-a)/n
    
        S0 = F(0) + F(9999)     'sum the first and last terms
        S1 = 0                  ' zero values for the running summations
        S2 = 0
    
        'sum terms except first and last; note that the loop is stepped
        'by 10 so loop overhead is smaller (increments, compares and conditional
        'jumps are eliminated).
    
        For SimpCounter = 1 To 9981 Step 10
    
            S1 = S1 + F(SimpCounter) + F(SimpCounter + 2) + F(SimpCounter + 4) + F(SimpCounter + 6) + F(SimpCounter + 8)
            S2 = S2 + F(SimpCounter + 1) + F(SimpCounter + 3) + F(SimpCounter + 5) + F(SimpCounter + 7) + F(SimpCounter + 9)
    
        Next SimpCounter
        
        'cleanup
        S1 = S1 + F(9991) + F(9993) + F(9995) + F(9997)
        S2 = S2 + F(9992) + F(9994) + F(9996) + F(9998)
    
        ' 'odd' terms * 2
        S1 = S1 * 2
        
        ' 'even' terms * 4
        S2 = S2 * 4
    
        'total summation
        Integral = S0 + S1 + S2
        
        'multiply by h/3
        Integral = Integral * h_3
    
    Next ItCounter
    
    'stop the timer
    StopTimer = Timer
    
    'display the result
    Print "By partially unrolling the loop (five steps unrolled):"
    Print "Integral = "; Integral; " "; cNumIts; " times in"
    Print StopTimer - StartTimer; " seconds."
    Print

End Sub

'This procedure computes the integral using a partially unrolled loop: the
'loop is unrolled with eight terms explicitly coded and the loop is stepped
'accordingly.  Partial unrolling reaches a diminishing return.
Private Sub cmdUnroll8_Click()

    'local declarations
    'longs
    Dim SimpCounter As Long     'loop counter for the summation
    Dim ItCounter As Long       'iteration counter
    
    'singles
    Dim h_3 As Single           'Simpson's Rule parameters
    Dim S0 As Single
    Dim S1 As Single
    Dim S2 As Single
    Dim Integral As Single      'Integral result
    Dim StartTimer As Single    'clock ticks at start of loop
    Dim StopTimer As Single     'clock ticks at loop exit
    
    'start the timer
    StartTimer = Timer
    
    For ItCounter = 1 To cNumIts
    
        ' h/3, where h=(b-a)/n
        h_3 = 0.999 / 30000
    
        'sum the first and last terms
        S0 = F(0) + F(9999)
        
        ' zero values for the running summations
        S1 = 0
        S2 = 0
    
        'sum terms except first and last; note that the loop is stepped
        'by 16 so loop overhead is smaller (increments, compares and conditional
        'jumps are eliminated).
        For SimpCounter = 1 To 9969 Step 16
    
            S1 = S1 + F(SimpCounter) + F(SimpCounter + 2) + F(SimpCounter + 4) + F(SimpCounter + 6) _
                + F(SimpCounter + 8) + F(SimpCounter + 10) + F(SimpCounter + 12) + F(SimpCounter + 14)
            S2 = S2 + F(SimpCounter + 1) + F(SimpCounter + 3) + F(SimpCounter + 5) + F(SimpCounter + 7) _
                + F(SimpCounter + 9) + F(SimpCounter + 11) + F(SimpCounter + 13) + F(SimpCounter + 15)
    
        Next SimpCounter
        
        'cleanup
        S1 = S1 + F(9985) + F(9987) + F(9989) + F(9991) + F(9993) _
            + F(9995) + F(9997)
            
        S2 = S2 + F(9986) + F(9988) + F(9990) + F(9992) + F(9994) _
            + F(9996) + F(9998)

        ' 'odd' terms * 2
        S1 = S1 * 2
        
        ' 'even' terms * 4
        S2 = S2 * 4
    
        'total summation
        Integral = S0 + S1 + S2
        
        'multiply by h/3
        Integral = Integral * h_3
    
    Next ItCounter
    
    'stop the timer
    StopTimer = Timer
    
    'display the result
    Print "By partially unrolling the loop (eight steps unrolled):"
    Print "Integral = "; Integral; " "; cNumIts; " times in"
    Print StopTimer - StartTimer; " seconds."
    Print

End Sub

'This procedure computes the integral using a partially unrolled loop: the
'loop is unrolled with four terms explicitly coded and the loop is stepped
'accordingly.  Partial unrolling reaches a diminishing return.

Private Sub cmdUnroll4_Click()

    'local declarations
    'longs
    Dim SimpCounter As Long     'loop counter for the summation
    Dim ItCounter As Long       'iteration counter
    
    'singles
    Dim h_3 As Single           'Simpson's Rule parameters
    Dim S0 As Single
    Dim S1 As Single
    Dim S2 As Single
    Dim Integral As Single      'Integral result
    Dim StartTimer As Single    'clock ticks at start of loop
    Dim StopTimer As Single     'clock ticks at loop exit
    
    'start the timer
    StartTimer = Timer
    
    For ItCounter = 1 To cNumIts
    
        ' h/3, where h=(b-a)/n
        h_3 = 0.999 / 30000
    
        'sum the first and last terms
        S0 = F(0) + F(9999)
        
        ' zero values for the running summations
        S2 = 0
        S1 = 0
    
        'sum terms except first and last; note that the loop is stepped
        'by 8 so loop overhead is smaller (increments, compares and conditional
        'jumps are eliminated).
    
        For SimpCounter = 1 To 9985 Step 8
    
            S1 = S1 + F(SimpCounter) + F(SimpCounter + 2) + F(SimpCounter + 4) + F(SimpCounter + 6)
            S2 = S2 + F(SimpCounter + 1) + F(SimpCounter + 3) + F(SimpCounter + 5) + F(SimpCounter + 7)
    
        Next SimpCounter
        
        'cleanup
        S1 = S1 + F(9993) + F(9995) + F(9997)
        S2 = S2 + F(9994) + F(9996) + F(9998)

        ' 'odd' terms * 2
        S1 = S1 * 2
        
        ' 'even' terms * 4
        S2 = S2 * 4
    
        'total summation
        Integral = S0 + S1 + S2
        
        'multiply by h/3
        Integral = Integral * h_3
    
    Next ItCounter
    
    'stop the timer
    StopTimer = Timer
    
    'display the result
    Print "By partially unrolling the loop (four steps unrolled):"
    Print "Integral = "; Integral; " "; cNumIts; " times in"
    Print StopTimer - StartTimer; " seconds."
    Print

End Sub

'initialization code; initialize function x^5

Private Sub Form_Load()

    Dim Counter As Single
    
    For Counter = 0 To 9999
    
        F(Counter) = (Counter / 10000) ^ 5
    
    Next Counter

End Sub
