VERSION 5.00
Begin VB.Form frmRisc05 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "RISC05 - Combined Distance - Energy RISC Calculation Revisited"
   ClientHeight    =   3195
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   6435
   Icon            =   "frmRisc05.frx":0000
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   3195
   ScaleWidth      =   6435
   StartUpPosition =   2  'CenterScreen
   Begin VB.CommandButton cmdClearText 
      Caption         =   "Clear Text"
      Height          =   615
      Left            =   4320
      TabIndex        =   2
      Top             =   2040
      Width           =   1695
   End
   Begin VB.CommandButton cmdRISC2 
      Caption         =   "r and E as RISC without sqr(r) and r^2"
      Height          =   615
      Left            =   4320
      TabIndex        =   1
      Top             =   1200
      Width           =   1695
   End
   Begin VB.CommandButton cmdRISC1 
      Caption         =   "r and E as RISC with sqr(r) and r^2"
      Height          =   735
      Left            =   4320
      TabIndex        =   0
      Top             =   240
      Width           =   1695
   End
End
Attribute VB_Name = "frmRisc05"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'This program demonstrates the performance improvement that
'can be made when an algorithm is profiled for redundant
'or unneeded computation.
'
'In the combined distance and LJ Energy computation, the distance
'r is computed using the sqr() function.  When r is used to compute
'the Lennard Jones Energy, r^6 and r^12 are computed.
'
'In particular, the sqr() function is "wasted."  It is noted that
'r^6 = (r^3)^2, so cycles are saved by not square-rooting then squaring
'again.
'
'The net algorithm speed increases by a factor of 2.

Option Explicit

'Global Declarations
'Constants
Const cNumIts As Long = 10000000        'number of iterations
Const x1 As Double = 1.234              'just some points to use
Const x2 As Double = 2.678
Const y1 As Double = 0.345
Const y2 As Double = 1.529

'Singles
Dim starttimer As Single        'System ticks at start of loop
Dim stoptimer As Single         'System ticks at loop exit

'clears the output for multiple runs
Private Sub cmdClearText_Click()

    Cls

End Sub

' RISC Algorithm from RISC03 combining the r and E calculation
' but computes the square of the square root

Private Sub cmdRISC1_Click()

    'Local Declarations
    'Longs
    Dim counter As Long     'iteration counter

    'Doubles
    Dim TempX As Double     'holder for (x2-x1)^2
    Dim TempY As Double     'holder for (y2-y1)^2
    Dim r1 As Double        'holder for r^6
    Dim r2 As Double        'holder for r^12
    Dim Energy As Double    'computed LJ energy
    Dim r As Double         'computed 2d distance
    
    starttimer = Timer      'start the timer
    
    For counter = 1 To cNumIts
    
        'compute r in RISC style of RISC02
        TempX = x1 - x2
        TempX = TempX * TempX
        TempY = y1 - y2
        TempY = TempY * TempY
        TempY = TempY + TempX
    
        'actual r is computed
        r = Sqr(TempY)
    
        'compute LJ Energy in RISC style of RISC01
        r1 = r * r * r * r * r * r
        r2 = r1 * r1
        
        Energy = 1 / r2 - 1 / r1
        
    Next counter
    
    stoptimer = Timer   'stop the timer
    
    'Display the result
    Print
    Print "Computed r and E "; cNumIts; " times as RISC with "
    Print "squaring the square root in"
    Print stoptimer - starttimer; " seconds"
    Print "E = "; Energy
    Print

End Sub

'Combined r and E RISC computation further optimized by
'elimating the squaring of the square root

Private Sub cmdRISC2_Click()
    
    'Local Declarations
    'Longs
    Dim counter As Long     'iteration counter
 
    'Doubles
    Dim TempX As Double     'holder for (x2-x1)^2
    Dim TempY As Double     'holder for (y2-y1)^2
    Dim r1 As Double        'holder for r^6
    Dim r2 As Double        'holder for r^12
    Dim Energy As Double    'computed energy
    Dim r As Double         'computed 2d distance squared
    
    starttimer = Timer      'start the timer
    
    For counter = 1 To cNumIts
    
        'compute r^2 (not r!!) in RISC Style of RISC02
        TempX = x1 - x2
        TempX = TempX * TempX
        TempY = y1 - y2
        TempY = TempY * TempY
        TempY = TempY + TempX
        
        'don't need to compute sqr(r) then re-square it in the
        'energy calculation.  r^2 is used to compute r^6 directly
        'as r^6=(r^2)^3; entering the next step, TempY=r^2
        
        r1 = TempY * TempY * TempY      'r1 = r^6
        r2 = r1 * r1                    'r2 = (r^6)^2 = r^12
        
        Energy = 1 / r2 - 1 / r1
        
    Next counter
    
    stoptimer = Timer       'stop the timer
    
    'Display the results
    Print
    Print "Computed r and E "; cNumIts; " times as RISC without "
    Print "squaring the square root in"
    Print stoptimer - starttimer; " seconds"
    Print "E = "; Energy

End Sub
