VERSION 5.00
Begin VB.Form frmOther02 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "Other02 - Nested Loops vs Energy Change Only Demo"
   ClientHeight    =   3195
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   5505
   Icon            =   "Other02.frx":0000
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   3195
   ScaleWidth      =   5505
   StartUpPosition =   3  'Windows Default
   Begin VB.CommandButton cmdEnFull 
      Caption         =   "Compute Full LJ Energy"
      Height          =   735
      Left            =   3480
      TabIndex        =   1
      Top             =   240
      Width           =   1455
   End
   Begin VB.CommandButton cmdEnChange 
      Caption         =   "Compute LJ Energy CHANGE only"
      Height          =   735
      Left            =   3480
      TabIndex        =   0
      Top             =   1680
      Width           =   1455
   End
   Begin VB.Label Label2 
      Height          =   855
      Left            =   240
      TabIndex        =   3
      Top             =   360
      Width           =   2775
   End
   Begin VB.Label Label1 
      Height          =   735
      Left            =   240
      TabIndex        =   2
      Top             =   1680
      Width           =   2775
   End
End
Attribute VB_Name = "frmOther02"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'
'This program SIMULATES a 1-particle homogeneous Monte Carlo using two algorithms to
''calculate' the LJ Energy: the two particle energy is computed each iteration
'using the nested x,y loops, and the energy change only is computed using a single
'particle loop counter.
'
'Note that the performance gain is not linear.  The 'Full Energy' computation
'scales roughly as N^2 vs. N^1 scaling for the single particle routine.
'Therefore, the gain improves for calculations involving more particles.

Option Explicit

'Global Declarations
'Constants
Const cNumInBox As Long = 20        'Number of particles in 'actual' box
Const cBoxSide As Long = 500        'Actual box is 5000pm x 5000pm
                                    'To save memory, precision is 10 pm.
Const NumConfigs As Long = 50000    'Number of Configurations
              
'Singles
Dim Sigma As Single                 'LJ Sigma parameter
Dim Depth As Single                 'LJ Well depth parameter
Dim Rval(354, 354) As Single        'Look-Up Table Entry for precomputed distances
Dim StartTimer As Single            'Stores the time at entry of loop
Dim StopTimer As Single             'Stores the time as loop exit

'Doubles
Dim Energy(354) As Double           'Look-Up Table Entry for the LJ Energies as
                                    'as function of distance; distance mesh is 10pm
Dim TotalEnergy As Double           'Total Energy

'User Defined Types
Private Type Particle               'Structure to hold particle data; just a bit neater
                                    'to store this way, and sets stage for OOP if desired
    x As Single                     'x coordinate
    y As Single                     'y coordinate
    
End Type

Dim Particles(20) As Particle       'need twenty particles in box

'Initializes the Energy(r); using LJ parameters for Helium for this demo
'RISC style of Chapter 8 is used.
Private Sub Init_Energy()

    'Procedure Declarations
    'Longs
    Dim Counter As Long     'Distance counter, mesh (or step size) is 10 pm
    
    'Doubles
    Dim R As Double         'Cache variable for distance
    Dim R6 As Double        'Distance^6
    Dim R12 As Double       'Distance^12
    Dim Sigma6 As Double    'LJ Sigma^6
    Dim Sigma12 As Double   'LJ Sigma^12
    
    'need an energy for all possible distances within box
    For Counter = 1 To 354
    
        'distance mesh (or step) is 10 pm
        R = Counter * 10
        
        'distance^6
        R6 = R * R * R * R * R * R
        
        'distance^12
        R12 = R6 * R6
        
        'Sigma^6
        Sigma6 = Sigma * Sigma * Sigma * Sigma * Sigma * Sigma
        
        'Sigma^12
        Sigma12 = Sigma6 * Sigma6
        
        'Compute LJ Energy
        Energy(Counter) = Sigma12 / R12 - Sigma6 / R6
        
        'Scale by well depth
        Energy(Counter) = Energy(Counter) * Depth
        
    Next Counter

End Sub

'Compute energies using Energy Change Only.  This calculation scales
'roughly as N^1.  Note that the code is 'more obscure' compared to the
'LJ energy equations written on paper.
Private Sub cmdEnChange_Click()

    'Procedure Declarations
    'Longs
    Dim Configs As Long             'configuration counter
    Dim SelectedParticle As Long    'Index for randomly selected particle
    Dim SelectedType As Long        'Cache variable for selected particle type
    
    'Singles
    Dim SelectedX As Single         'Cache variable for selected particle x coordinate
    Dim SelectedY As Single         'Cache variable for selected particle y coordinate
    
    'Doubles
    Dim DeltaEnergy1 As Double       'The Energy Change, or "static part" as referenced
                                     'in Chapter 10
    Dim DeltaEnergy2 As Double
    
    'display a message
    Label1.Caption = "Computing ..."
    
    'let windows catch up
    DoEvents
    
    'start the timer
    StartTimer = Timer
    
    For Configs = 1 To NumConfigs
    
        'Select a particle
        SelectedParticle = Int(cNumInBox * Rnd + 1)
        
        'Cache the particle x,y coordinates
        SelectedX = Particles(SelectedParticle).x
        SelectedY = Particles(SelectedParticle).y
 
        'Calculate energy
        DeltaEnergy1 = LJ_Energy(SelectedParticle, cNumInBox, _
                    SelectedX, SelectedY)
        
        'Calculate another energy; rather than selecting a random new x,y for
        'selected particle, simply switching x,y to 'assume' a new position.  Monte Carlo
        'does not do this.
        'Since x,y is different each time through loop, y,x will be also.
        'There is no performance cost in doing this since the program does not
        '"care" what parameters are passed to the function, so long as they are
        'the right type.
        DeltaEnergy2 = LJ_Energy(SelectedParticle, cNumInBox, _
                    SelectedY, SelectedX)
        
    Next Configs
    
    'stop the timer
    StopTimer = Timer
    
    'Display timer results
    Label1.Caption = Str$(StopTimer - StartTimer) & " seconds for Energy Change Only"

End Sub


'Initialize the Look-Up Array for distances
Private Sub Init_Rval()

    'Private Declarations

    'Longs
    Dim rCountx As Long     'x coordinate counter
    Dim rCounty As Long     'y coordinate counter
        
    'Singles
    Dim tempx As Single     'cache variable for x
    Dim tempy As Single     'cache variable for y
    
    For rCountx = 1 To 354

        'distance mesh is 10 pm
        tempx = rCountx * 10
        
        'square "delta-x"
        tempx = tempx * tempx
        
        For rCounty = 1 To 354
        
            'distance mesh is 10 pm
            tempy = rCounty * 10
            
            'square "delta-y"
            tempy = tempy * tempy

            'compute distance
            Rval(rCountx, rCounty) = Int(Sqr(tempx + tempy) * 0.1)
            
        Next rCounty
    
    Next rCountx

End Sub

'Initialize 20 particles of random type (1,2 or 4) at random positions
'It is not necessary to pick random types; in a real procedure, 'block' types
'may be used; eg, particles 1-5 type 1, particles 6-10 type 2, etc for whatever
'model is under investigation.
'Random types are used here to further stress the branch predictions.
Private Sub Init_Particles()

    'Local Declarations
    'Longs
    Dim PCounter As Long        'particle counter
    
    For PCounter = 1 To 20      'for 20 particles

        With Particles(PCounter)
        
            'put particles at random positions for demo
            .x = 354 * Rnd()
            .y = 354 * Rnd()
            
        End With
        
    Next PCounter

End Sub

'Function LJ_Energy Computes the 1-particle LJ Energy...called "changed part"
'in the Chapter 10.

Private Function LJ_Energy(MovedIndex As Long, TotalNum As Long, _
            XMoved As Single, YMoved As Single) As Double
            
    'Private Declarations
            
    'Longs
    Dim I As Long           'particle counter
    Dim deltaX As Long      'difference between particle x's
    Dim deltaY As Long      'difference between particle y's
    Dim R As Long           'Distance between particles
                
    'Singles
    Dim sCurrentX As Single 'x coordinate of 'current' particle
    Dim sCurrentY As Single 'y coordinate of 'current' particle
    Dim BCX As Single       'x for particle in boundary box
    Dim BCY As Single       'y for particle in boundary box
    
    'Doubles
    Dim ThisEnergy As Double    'accumulative energy holder
    
    'initiaze to zero
    ThisEnergy = 0
    
    For I = 1 To TotalNum
     
        'cache x,y coordinates
        sCurrentX = Particles(I).x
        sCurrentY = Particles(I).y
        
        'avoid R=0, ie same particle
        If I <> MovedIndex Then
            
            'compute x difference
            deltaX = Abs(sCurrentX - XMoved)
            
            'compute y difference
            deltaY = Abs(sCurrentY - YMoved)
                
            'lookup distance
            R = Rval(deltaX, deltaY)
            R = Int(R \ 10)
        
            'accumulate and lookup energy
            ThisEnergy = ThisEnergy + Energy(R)
        
        End If
        
    Next I
    
    LJ_Energy = ThisEnergy
    
End Function

'Computes the full LJ energy using nested loops.
Private Sub cmdEnFull_Click()
            
    'Procedure Declarations
    'Longs
    Dim Configs As Long             'configuration counter
    Dim SelectedParticle As Long    'Index for randomly selected particle
      
    Label2.Caption = "Computing ..."
    DoEvents                        'allow windows to process events, specifically updating
                                    'label2.caption BEFORE entering main compute loop
    StartTimer = Timer              'start the timer
    
    For Configs = 1 To NumConfigs
        
        'Calculate the Energy
        TotalEnergy = LJ_Full(cNumInBox)
        
        'Calculate another energy;  For this demo, simply call
        'LJ_Full function again.
        TotalEnergy = LJ_Full(cNumInBox)
           
    Next Configs
    
    'stop the timer
    StopTimer = Timer
    
    'Display Timer Results
    Label2.Caption = Str$(StopTimer - StartTimer) & " seconds for Full Energy in nested Loops"
    

End Sub

'Entry point for program
Private Sub Form_Load()

    'These values for Sigma and Depthp are for Helium
    Sigma = 258         'in pm
    Depth = 5.65E-22    'in Joules
    
    'Initialize the LookUp Table
    Init_Rval
    Init_Energy
    Init_Particles

End Sub

'Calculate the Full LJ Energy for all particles in box and
'in BC boxes, but the symmetry of Rij=Rji is used.  This routine
'scales approximately as TotNum^2, so performance degrades parabolically
'as the number of particles increases.
'This demo shows that nested loops should be avoided if an alternative
'algorithm can be constructed.

Private Function LJ_Full(TotNum As Long) As Double

    'Private Declarations
    
    'Longs
    Dim I As Long           'Particle I counter
    Dim J As Long           'Particle J counter
    Dim deltaX As Long      'Difference between x of I and J particles
    Dim deltaY As Long      'Difference between y of I and J particles
    Dim R As Long           'R from Rval lookup table
                
    'Singles
    Dim sXi As Single       'cache variable for x coordinate of particle I
    Dim sYi As Single       'cache variable for y coordinate of particle I
    Dim sXj As Single       'cache variable for x coordinate of particle J
    Dim sYj As Single       'cache variable for y coordinate of particle J
    
    Dim BCX As Single       'holds x,y for particles in 'boundary boxes'
    Dim BCY As Single
    
    'Doubles
    Dim ThisEnergy As Double    'running variable for energy
    
    'Initialize to zero
    ThisEnergy = 0
    
    For I = 1 To TotNum - 1
    
        'cache x,y values
        sXi = Particles(I).x
        sYi = Particles(I).y
        
        'do for all J > I
        For J = I + 1 To TotNum
        
            'cache x,y values
            sXj = Particles(J).x
            sYj = Particles(J).y
            
            'x difference
            deltaX = Abs(sXi - sXj)
            
            'y difference
            deltaY = Abs(sYi - sYj)
                
            'lookup distance rather than compute
            R = Rval(deltaX, deltaY)
            R = Int(R \ 10)
        
            'lookup and accumulate energy
            ThisEnergy = ThisEnergy + Energy(R)
        
        Next J
    
    Next I
    
    'function value = Inner computed value
    LJ_Full = ThisEnergy

End Function


