// Data04.c implementation file
//
// This program is Linux equivalent of the Windows Console App. demo'd
// in Data04
//
// To perform some computational work, the program selects a random
// distance between 'sigma' and 7 angstroms, and computes the Lennard
// Jones pair energy.  Helium LJ parameters are used.

#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

//Defines
#define NumIts 100000000		//number of iterations
#define sigma  (float)258		//lennard jones sigma parameter
#define depth  (float)10.22		//lennard jones depth parameter

//global declarations
//int xk;				//'last' random number

double energy;

int main()
{
	//local declarations
	//integers
	int i;		//iteration counter
	int rnum;	//random number
	int xk;		//'last' random number
	int m;		//random generator period
	int a;		//defines the generator
	
	//floats
	float r;		//distance between particles	
	float r_6;		//distance^6
	float r_12;		//distance^12
	float sigma_6;		//sigma^6
	float sigma_12;		//sigma^12
	//float energy;		//computed energy

	//doubles
	double starttime;	//ticks at start of loop
	double stoptime;	//ticks at end of loop
	double elapsedtime;	//seconds during loop

	//character strings
	char ch;			//input string to end

	//need structures for the timer functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

	//seed the random number generator, definition and period
	xk = 200;
	m = 16777216;
	a = 25;

	//clear the current console display
	system("clear");

	//diplay 'computing' message
	printf("Computing iterative Lennard Jones Energies ...\n\n");

	//start the timer
	gettimeofday(&_tstart, &tz);
	
	//compute sigma^6, per Chapter 8
	sigma_6 = sigma * sigma * sigma * sigma * sigma * sigma;

	//compute sigma^12, per Chapter 8
	sigma_12 = sigma_6 * sigma_6;

	//start iteration
	for(i=0; i<NumIts; ++i){
		
		//get a random r between 0 and 1, per Chapter 6
		rnum = (25 * xk) % 16777216;
		r = (float)rnum/(float)16777216;
		
		//increment the recursive index value
		xk = rnum;
		
		//get r between 0 and 7
		r = r * 7 + 1;

		//compute r^6, per Chapter 8
		r_6 = r * r * r * r * r * r;
	
		//compute r^12, per Chapter 8
		r_12 = r_6 * r_6;

		//compute energy
		energy = sigma_12/r_12 - sigma_6/r_6;
		energy *= depth;

	}

	//stop the timer
	gettimeofday(&_tstop, &tz);

	//compute the elapsed time
	starttime = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	stoptime = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	elapsedtime = (float)(stoptime - starttime);

	//display result
	printf("%s%g%s", "Elapsed time for Linux Console App: ",elapsedtime," seconds.\n\n");

	//display closing message and wait for <enter> before exiting
	printf("Press <Enter> to end");
	scanf("%c", &ch);

	//clear the current display
	system("clear");

	//exit
	return 0;
}
