// Data01.c : implementation file
//
// This program compares code requiring type conversions to that
// with properly typed variables.
//
// The rolled routine is in fnA().
// The unrolled routine is in fnB().


#include <sys/time.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

//defines
#define NumIts 10000000		//number of iterations

//functions
void fnA();			
void fnB();			

// computes x^y with poor declarations that force type conversions
void fnA()
{

	//general local declarations
	//integers
	int counter;				// iteration counter
	int x;						// base
	int y;						// exponent
	int result;					// calculation result
	
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

	//assign base
	x = (int)2;

	//assign exponent
	y = (int)4;
	
	//display entry message
	printf("\nComputing 2^4 with type conversions...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(counter=1; counter <=NumIts; ++counter){
        
		//compute x^y
		result = (int)pow((double)y, (double)x);
    
    }
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	
}//end fnA()

//compute x^y with properly typed data
void fnB()
{

	//general local declarations
	//integers
	int		counter;			// iteration counter
	
	//doubles
	double	x;					// base
	double  y;					// exponent
	double  result;				// calculation result

	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

	//assign base
	x = (double)2;

	//assign exponent
	y = (double)4;
	
	//display entry message
	printf("\nComputing 2^4 without type conversions...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(counter=1; counter <=NumIts; ++counter){
        
		//compute x^y
		result = pow(y, x);
    
    }
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	
}//end fnB()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
	system("clear");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tx^y with poor data typing, forcing conversions\n");
		printf("2\tx^y with properly typed data\n");
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
	
			case '2': fnB();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

}
