// Data03.c : implementation file
//
//
// This program demonstrates the performance difference between using
// variable data and 'constant' data.  The variables generally force
// a memory reference compared to the immediate data reference possible
// with constants.
//
// In C, immediate data are generally declared as macros, using #define. 
//
// This demo uses an empty for loop that is executed repeatedly.  Getting
// machine/platform uniform data in this demo may be difficult due to
// data caching.
//
// A Loop with variable data (requiring a memory reference) is in fnA().
// A Loop with fixed data (using immediate, compiled-in data) is in fnB().


#include <sys/time.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

//defines
#define NumIts		10000000	// number of iterations
#define CSTART		1			// fixed loop start
#define CSTOP		20			// fixed loop end
#define CSTEPSIZE   1			// fixed loop step

//global declarations
//integers
int StartCounter;            //memory loop start
int StopCounter;             //memory loop stop
int StepSize;                //memory loop step size
int Counter;                 //test loop counter

//functions
void fnA();			
void fnB();			

// loop with memory access for parameters
void fnA()
{

	//general local declarations
	//integers
	int itcounter;				//iteration counter	
	
	//variables for time execution time
	//floats
	float ElapsedTime;			//ticks during loop
	
	//doubles
	double t1;					//ticks at start of loop
	double t2;					//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

	//initialize the data
	StartCounter = CSTART;
	StopCounter = CSTOP;
	StepSize = CSTEPSIZE;
	
	//display entry message
	printf("\nLoop Overhead with Memory References...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(itcounter=1; itcounter <= NumIts; ++itcounter){
        
		for(Counter=StartCounter; Counter <= StopCounter; Counter += StepSize){ 

		}
    
    }
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	
}//end fnA()

//repeatedly do for loop with immediate data as loop boundaries
void fnB()
{

	//general local declarations
	//integers
	int itcounter;				//iteration counter

	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;
	
	//display entry message
	printf("\nLoop Overhead with Immediate Data...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(itcounter=1; itcounter <= NumIts; ++itcounter){
        
		for(Counter=CSTART; Counter <= CSTEPSIZE; Counter += CSTEPSIZE){ 

		}
    
    }
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	
}//end fnB()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
	system("clear");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tLoop with Memory Reference\n");
		printf("2\tLoop with Immediate Data\n");
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
	
			case '2': fnB();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

}
