// Data05.c : implementation file
//
// This program compares memory reference times to global data vs.
//	local data.
//
// fnA() uses local data
// fnB() uses global data

#include <sys/time.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

//defines
#define NumIts		50000000	// number of iterations

//global data
//integers
int gCounter;				//global interation counter

//doubles
double gAngle;				//global angle
double gResult;				//global sin(x) result

//functions
void fnA();			
void fnB();			

// compute using LOCAL scope
void fnA()
{

	//general local declarations
	//integers
	int lcounter;				//iteration counter	

	//doubles
	double lAngle;				//local angle
	double lResult;				//local sin(x) result

	//variables for time execution time
	//floats
	float ElapsedTime;			//ticks during loop
	
	//doubles
	double t1;					//ticks at start of loop
	double t2;					//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

	//display entry message
	printf("\nCompute sin(x) with locally declared data...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(lcounter=1; lcounter <=NumIts; ++lcounter){
        
		//assign the angle
		lAngle = (double)0.785398163; //(double)rand()/RAND_MAX;

		//compute sin(angle)
		lResult = sin(lAngle);
    
    }
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	
}//end fnA()

//compute sin(x) using global data
void fnB()
{

	//general local declarations

	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;
	
	//display entry message
	printf("\nCompute sin(x) with global data...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(gCounter=1; gCounter <=NumIts; ++gCounter){
        
		//assign the angle
		gAngle = (double)0.785398163; //(double)rand()/RAND_MAX

		//compute sin(angle)
		gResult = sin(gAngle);
    
    }
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	
}//end fnB()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
	system("clear");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tCompute with local data\n");
		printf("2\tCompute with global data\n");
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
	
			case '2': fnB();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

}
