// Function08.c : implementation file
//
// This program compares the use of recursive function calls to 
// solving a recursive problem iteratively.  The factorial of 100
// is computed in this demo.
//
// The recursive function call is in fnA().
// The iterative function call is in fnB().

#include <sys/time.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

//defines
#define NumIts 1000000		//number of iterations

//functions
void fnA();			
void fnB();
void fnC();			

double Recurs_Fact(int Arg1);

//compute 100! in recursive function call
void fnA()
{

	//general local declarations
	//integers
	int counter;			  // iteration counter

	//doubles
	double Result;			  //return value
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;
	
	//display entry message
	printf("\nComputing Factorial using Recursive Function call ...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(counter=1; counter <= NumIts; ++counter){

        //call the recursive function
		Result = Recurs_Fact(100);

    }
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	printf("%s%g%s", "Computed Factorial = ", Result, "\n");

}//end fnA()

//iterative computation of 100!
void fnB()
{
	//general local declarations
	//integers
	int counter;			  // iteration counter
	int fact;				  //factorial 'counter'

	//doubles
	double Result;			  //return value
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;
	
	//display entry message
	printf("\nComputing Factorial Iteratively ...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//initialize Result
		Result = 1;

        //compute factorial iteratively
		for(fact = 1; fact<=100; fact++){

			Result *= fact;

		}

    }
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	printf("%s%g%s", "Computed Factorial = ", Result, "\n");

}  // End fnB()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
	system("clear");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tCompute Factorial using Recursive Function Call\n");
		printf("2\tIterative Inline Factorial Calculation\n");
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
	
			case '2': fnB();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

} // End Main

//function to compute factorial recursively
double Recurs_Fact(int Arg1)
{

	double answer;

	if(Arg1==1) return(1);
	answer = Recurs_Fact(Arg1 -1) * Arg1;

	return(answer);

}
