
// Loop01.c : implementation file
//
// This program demonstrates loop unrolling for a simple function 
// assignment.
//
// The rolled routine is in fnA().
// The unrolled routine is in fnB().


#include <sys/time.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

//defines
#define NumIts 10000000		//number of iterations

//functions
void fnA();			
void fnB();			

//assigns array via rolled loop
void fnA()
{

	//general local declarations
	//integers
	int i;					//iteration counter
	int fncounter;			//function assignment counter

	//doubles
	double expvalu;			//arguement factor
	double r[11];				//assigned function value
	
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

	//just a value to use
	expvalu = 3.13;
	
	//display entry message
	printf("\nAssigning array with rolled loop...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(i=1; i<= NumIts; ++i){
	
		for(fncounter=1; fncounter <= 10; ++fncounter){


			//assign the function array using the counter
			//as the independent variable
			r[fncounter] = exp(-fncounter * expvalu);
            
        }

	}
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	
}//end fnA()

void fnB()
{

	//general local declarations
	//integers
	int i;					//iteration counter

	//doubles
	double expvalu;			//arguement factor
	double r[11];				//assigned function value
	
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

	//just a value to use
	expvalu = 3.13;
	
	//display entry message
	printf("\nAssigning array with unrolled loop...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(i=1; i<= NumIts; ++i){

			//assign the function array using explicit code
			r[1] = exp(-1 * expvalu);
            r[2] = exp(-2 * expvalu);
			r[3] = exp(-3 * expvalu);
			r[4] = exp(-4 * expvalu);
			r[5] = exp(-5 * expvalu);
			r[6] = exp(-6 * expvalu);
			r[7] = exp(-7 * expvalu);
			r[8] = exp(-8 * expvalu);
			r[9] = exp(-9 * expvalu);
			r[10] = exp(-10 * expvalu);

	}
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	
}//end fnB()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
	system("clear");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tExponential Function Assignment with Rolled Loop\n");
		printf("2\tExponential Function Assignment with Unrolled Loop\n");
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
	
			case '2': fnB();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

}
