
//RISC01 demonstrates the use of simplified math source code versus
//complex code (via a call to a math function).  This particular demo uses 
//a reduced Lennard Jones pair energy calculation.


#include <sys/time.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

//defines
#define NumIts 10000000		//number of iterations

//functions
void fnCISC();			//LJ energy in CISC style
void fnRISC();			//LJ energy in RISC style

//computes reduced LJ Energy in 'CISC' style
void fnCISC()
{

	//local declarations
	//integers
	int i;			//iteration counter
	
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double r;		//distance between particles
	double energy;		//computed energy
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

	//initialize r to just a value to use
	r = (double)1.234;
	
	
	//display entry message
	printf("\nComputing LJ Energies in CISC Style...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(i=1; i<= NumIts; ++i){
	
		//compute the energy
		energy = 1/pow(r,(double)12) - 1/pow(r,(double)6);

	}
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	printf("%s%g%s", "Energy = ", energy, " \n\n");
	
}//end fnCISC()

void fnRISC()
{

	//local declarations
	//integers
	int i;			//iteration counter
	
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double r;		//distance between particles
	double r6;		//r^6
	double r12;		//r^12
	double energy;		//computed energy
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

	//initialize r to just a value to use
	r = (double)1.234;
	
	//display entry message
	printf("\nComputing LJ Energies in RISC Style...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(i=1; i<= NumIts; ++i){
	
		//compute r^6 from r
		r6 = r * r * r * r * r * r;
		
		//compute r^12 from r^6
		r12 = r6 * r6;
		
		//compute the energy
		energy = 1/r12 - 1/r6;
	}
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	printf("%s%g%s", "Energy = ", energy, " \n\n");
	
}//end fnRISC()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
	system("clear");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tLJ Energies in CISC Style\n");
		printf("2\tLJ Energies in RISC Style\n");
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);

		//inchar = getchar();

		//call the user selected function
		switch	(inchar){

			case '1': fnCISC();
				break;
	
			case '2': fnRISC();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

}
