// RISC04.c : implementation file
//

// This program compares the performance of the CISC style and RISC style
// code for the Boltzmann factor function.  In this case, the CISC style
// is faster.
//
// The CISC routine is in fnA().
// The RISC routine is in fnB().


#include <sys/time.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

//defines
#define NumIts 10000000		//number of iterations
#define Temp   89.45		// Temperature
#define kBoltz 1.381E-23	// Boltzmann's constant
#define Energy 4.52E-20		// energy

//functions
void fnA();			
void fnB();			

//CISC style calculation
void fnA()
{

	//general local declarations
	//integers
	int counter;		// iteration counter

	//doubles
	double BoltzFactor;	// result of exp(arg) calculation
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;
	
	//display entry message
	printf("\nCISC style Boltzmann calculation ...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(counter=1; counter <= NumIts; ++counter){
    
		// CISC Boltzman calc
        	BoltzFactor = exp(-Energy / (kBoltz * Temp));   

        }
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	
}//end fnA()

// RISC style Boltzmann factor calculation
void fnB()
{

	//general local declarations
	//integers
	int counter;			//iteration counter

	//doubles
	double BoltzFactor;	// result of exp(arg) calc
	double Temp1;		// intermediate variable
	double arg1;		// actual argument passed to exp() function
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;
	
	//display entry message
	printf("\nRISC Style Boltzmann Factor...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(counter=1; counter <= NumIts; ++counter){
    

        	//compute factor in RISC style
		Temp1 = kBoltz * Temp;		
		arg1 = -Energy / Temp1;		

        	BoltzFactor = exp(-arg1);

    }
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	
}//end fnB()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
	system("clear");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tCISC Style Boltzmann Factor Calculation\n");
		printf("2\tRISC Style Boltzmann Factor Calculation\n");
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
	
			case '2': fnB();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

} // End Main

