// LookUp01.c : implementation file
//
// This program compares the performance of explicit computations of
// the 'reduced' Lennard Jones energy to the use of a Look-Up Table.
//
// The Look-up Table assignment routine is in fnA().
// The Explicit calculation routine is in fnB().
// The Look-Up Table calculation routine is in fnC().


#include <sys/time.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

//defines
#define NumIts 10000000		//number of iterations

//global declarations
//integers
int bINIT;	 //flag = true if LUT has been initialized

//doubles
double EnergyLUT[5700];  //The Look-Up Table

//functions
void fnA();			
void fnB();			
void fnC();



//initialize the look up table
void fnA()
{

	//local declarations
	//integers
	int counter;			// iteration counter
	
	//doubles
	double r;			// distance to compute energy
	double r6;			// r^6
	double r12;			// r^12
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;
	
	//display entry message
	printf("\nInitializing Look-Up Table ...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(counter=50; counter <= 5657; ++counter){
    
		r = (double)counter * 0.001;
		
		r6 = r * r * r * r * r * r;
		r12 = r6 * r6;

		EnergyLUT[counter] = 1/r12 - 1/r6;  

        }
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nLook-Up Table Initialized in ", ElapsedTime, " seconds\n");
	
	//set flag to 'true'
	bINIT = 1;
	
}//end fnA()

// interative LJ Energy by actual computation
void fnB()
{

	//general local declarations
	//integers
	int counter;			//iteration counter

	//doubles
	double r;                 //distance
	double r6;		  //r^6
	double r12;		  //r^12
    	double energy;            //energy
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

	//just a value to use
	r = 1.023;
	
	//display entry message
	printf("\nStarting Iterative Explicit LJ Energy calculation...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(counter=1; counter <= NumIts; ++counter){
    
		r6 = r * r * r * r * r * r;
		r12 = r6 * r6;
		
		energy = 1/r12 - 1/r6;

        }
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nComputed LJ Energy in ", ElapsedTime, " seconds\n");
	
}//end fnB()

// interative LJ Energy by Look-Up Table
void fnC()
{

	//general local declarations
	//integers
	int counter;			//iteration counter
	int rindex;			//entry value for energy LUT


	//doubles
    	double energy;            //energy
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

	
	//display entry message
	printf("\nStarting Iterative LJ Energy via LUT ...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	//assign the distance index
	rindex = 1023;
	
	for(counter=1; counter <= NumIts; ++counter){
    
		energy = EnergyLUT[rindex];

        }
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nLooked Up Energy in ", ElapsedTime, " seconds\n");
	
}//end fnC()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//initialize bINIT to NOT INITIALIZED
	bINIT = 0;
	
	//clear terminal
	system("clear");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tInitialize the Look-Up Table\n");
		
		if(bINIT){
			printf("2\tIterative Explicit Computation of LJ Energy\n");
			printf("3\tIterative LJ Energy via Look-Up Table\n");
			printf("q\tEnd this program\n");
		}
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
	
			case '2': fnB();
				break;
				
			case '3': fnC();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

} // End Main

double calc_ljenergy (double R)
{

	double R1;		//these would be used internally in the 
	double R2;		//optimized lj energy computation

	R1 = 0;
	R2 = 0;

	return(0);

}
