// Par02.c : implementation file
//

// This program compares the 4x4 matrix mult. using straight x87 floating
// point code to use of 3dNow! and SSE SIMD code.  If the run-time processor
// does not support SIMD, the SIMD routines simply exit with a 
// message (CPUid MUST be supported, however...I did not test for
// that support before making the calls to CPUid).
//
// The x87 routine is in fnA()
// The 3dNow! routine is in fnB()
// The SSE routine is in fnC()


#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>

//defines
#define NumIts 50000000		//number of iterations

//functions
void fnA();			
void fnB();
void fnC();			


// perform matrix multiplication using x87 code.
void fnA()
{

	//general local declarations
	//integers
	int counter;		// iteration counter

	//floats
	float a[4][4];		//matrix a
	float b[4][4];		//matrix b
	float c[4][4];		//c = a x b
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

	//assign some values to the matrices
	//matrix a in row major ('regular') order
	a[0][0] = (float)3.654;
	a[0][1] = (float)8.234;
	a[0][2] = (float)3.874;
	a[0][3] = (float)5.792;
	a[1][0] = (float)2.234;
	a[1][1] = (float)8.201;
	a[1][2] = (float)8.333;
	a[1][3] = (float)0.677;
	a[2][0] = (float)5.555;
	a[2][1] = (float)6.633;
	a[2][2] = (float)4.298;
	a[2][3] = (float)9.002;
	a[3][0] = (float)4.094;
	a[3][1] = (float)6.773;
	a[3][2] = (float)1.202;
	a[3][3] = (float)2.118;

	//matrix b in row major ('regular') order
	b[0][0] = (float)3.654;
	b[0][1] = (float)2.348;
	b[0][2] = (float)8.743;
	b[0][3] = (float)7.925;
	b[1][0] = (float)2.342;
	b[1][1] = (float)2.018;
	b[1][2] = (float)3.338;
	b[1][3] = (float)6.770;
	b[2][0] = (float)5.555;
	b[2][1] = (float)6.336;
	b[2][2] = (float)2.984;
	b[2][3] = (float)0.029;
	b[3][0] = (float)0.944;
	b[3][1] = (float)7.736;
	b[3][2] = (float)2.021;
	b[3][3] = (float)1.182;

	
	//display entry message
	printf("\nComputing 4x4 Matrix Multiplication using only x87 ...\n");
	
	//get start time
	gettimeofday(&_tstart, &tz);
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//4x4 matrix multiplication, compiles to straight
		//x87 code
		//j = 0
		c[0][0] = a[0][0] * b[0][0] + a[0][1] * b[1][0] + a[0][2] * b[2][0] + a[0][3] * b[3][0];
		c[0][1] = a[0][0] * b[0][1] + a[0][1] * b[1][1] + a[0][2] * b[2][1] + a[0][3] * b[3][1];
		c[0][2] = a[0][0] * b[0][2] + a[0][1] * b[1][2] + a[0][2] * b[2][2] + a[0][3] * b[3][2];
		c[0][3] = a[0][0] * b[0][3] + a[0][1] * b[1][3] + a[0][2] * b[2][3] + a[0][3] * b[3][3];
	
		//j = 1
		c[1][0] = a[1][0] * b[0][0] + a[1][1] * b[1][0] + a[1][2] * b[2][0] + a[1][3] * b[3][0];
		c[1][1] = a[1][0] * b[0][1] + a[1][1] * b[1][1] + a[1][2] * b[2][1] + a[1][3] * b[3][1];
		c[1][2] = a[1][0] * b[0][2] + a[1][1] * b[1][2] + a[1][2] * b[2][2] + a[1][3] * b[3][2];
		c[1][3] = a[1][0] * b[0][3] + a[1][1] * b[1][3] + a[1][2] * b[2][3] + a[1][3] * b[3][3];
		
		//j = 2
		c[2][0] = a[2][0] * b[0][0] + a[2][1] * b[1][0] + a[2][2] * b[2][0] + a[2][3] * b[3][0];
		c[2][1] = a[2][0] * b[0][1] + a[2][1] * b[1][1] + a[2][2] * b[2][1] + a[2][3] * b[3][1];
		c[2][2] = a[2][0] * b[0][2] + a[2][1] * b[1][2] + a[2][2] * b[2][2] + a[2][3] * b[3][2];
		c[2][3] = a[2][0] * b[0][3] + a[2][1] * b[1][3] + a[2][2] * b[2][3] + a[2][3] * b[3][3];

		//j = 3
		c[3][0] = a[3][0] * b[0][0] + a[3][1] * b[1][0] + a[3][2] * b[2][0] + a[3][3] * b[3][0];
		c[3][1] = a[3][0] * b[0][1] + a[3][1] * b[1][1] + a[3][2] * b[2][1] + a[3][3] * b[3][1];
		c[3][2] = a[3][0] * b[0][2] + a[3][1] * b[1][2] + a[3][2] * b[2][2] + a[3][3] * b[3][2];
		c[3][3] = a[3][0] * b[0][3] + a[3][1] * b[1][3] + a[3][2] * b[2][3] + a[3][3] * b[3][3];

    	}
	
	//stop the timer
	gettimeofday(&_tstop, &tz);

	//get seconds from the structures
	t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
	t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
	//compute the elapsed time
	ElapsedTime = (float)(t2 - t1);

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	printf("%s%g%s", "c[2,3] ",c[2][3],"\n");
	
}//end fnA()

// perform matrix multiplication in ASSEMBLY using 3dNow! SIMD instructions
void fnB()
{

	//general local declarations
	//integers
	int counter;			//iteration counter
	int bAMDOK;			//cpu supports 3DNow! instructions


	//floats
	float a[4][4];			//matrix a
	float b[4][4];			//matrix b
	float c[4][4];			//c = a x b 
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

	//initialize AMD flag to zero
	bAMDOK = 0;

	//test for 3DNow! capability in cpu
	//this is adapted from code in AMD 3DNow! Tech. Manual
	asm volatile("pushl	%%eax\n\t"
		"pushl	%%edx\n\t"
		"xor	%%eax, %%eax\n\t"
		"CPUid\n\t"
		"test	%%eax, %%eax\n\t"
		"jz	No3dNow\n\t"	
		"movl	$0x80000000, %%eax\n\t"
		"CPUid\n\t"
		"cmp	$0x80000001, %%eax\n\t"
		"jl	No3dNow\n\t"
		"movl	$0x80000001, %%eax\n\t"
		"CPUid\n\t"
		"test	$0x80000000, %%edx\n\t"
		"jnz	Yes3dNow\n\t"
		"jmp	No3dNow\n"
"Yes3dNow:	\n\t"
		"movl	$1, %0\n\t"
		"jmp	EndCpuTest\n"
"No3dNow:	\n\t" 
		"movl	$0,%0\n"
"EndCpuTest:	\n\t"	 
		"popl	%%edx\n\t"
		"popl	%%eax"
		:"=g" (bAMDOK)
		:
		:"%eax","%ebx","ecx","%edx"
		
	);

	if(bAMDOK){

		//display entry message
		printf("\n4x4 Matrix Multiplication using 3dNow! SIMD Code...\n");
	
		//initialize the two matrices with some made up values
		a[0][0] = (float)3.654;
		a[0][1] = (float)8.234;
		a[0][2] = (float)3.874;
		a[0][3] = (float)5.792;
		a[1][0] = (float)2.234;
		a[1][1] = (float)8.201;
		a[1][2] = (float)8.333;
		a[1][3] = (float)0.677;
		a[2][0] = (float)5.555;
		a[2][1] = (float)6.633;
		a[2][2] = (float)4.298;
		a[2][3] = (float)9.002;
		a[3][0] = (float)4.094;
		a[3][1] = (float)6.773;
		a[3][2] = (float)1.202;
		a[3][3] = (float)2.118;

		//store b matrix in COLUMN MAJOR order
		b[0][0] = (float)3.654;
		b[1][0] = (float)2.348;
		b[2][0] = (float)8.743;
		b[3][0] = (float)7.925;
		b[0][1] = (float)2.342;
		b[1][1] = (float)2.018;
		b[2][1] = (float)3.338;
		b[3][1] = (float)6.770;
		b[0][2] = (float)5.555;
		b[1][2] = (float)6.336;
		b[2][2] = (float)2.984;
		b[3][2] = (float)0.029;
		b[0][3] = (float)0.944;
		b[1][3] = (float)7.736;
		b[2][3] = (float)2.021;
		b[3][3] = (float)1.182;
		
		//get start time
		gettimeofday(&_tstart, &tz);
	
		//set fp registers to SIMD mode
		asm ("femms\n\t");
	
		for(counter=1; counter <= NumIts; ++counter){
    
			//3DNow! dot product, using extended assembly lang.
			asm volatile(
			
				//prefetch B matrix into L1
				"prefetchnta	512(%1)\n\t"
						
				"movq    (%0), %%mm0\n\t"
				"movq   8(%0), %%mm1\n\t"
				"movq    (%1), %%mm2\n\t"
				"movq   8(%1), %%mm3\n\t"
				"pfmul   %%mm0, %%mm2\n\t"
				"movq  16(%1), %%mm4\n\t"
				"movq  24(%1), %%mm5\n\t"
				"pfmul   %%mm1, %%mm3\n\t"
				"movq  32(%1), %%mm6\n\t"
				"movq  40(%1), %%mm7\n\t"
				"pfmul   %%mm0, %%mm4\n\t"
				"pfadd   %%mm3, %%mm2\n\t"
				"pfacc   %%mm2, %%mm2\n\t"
				"movd    %%mm2, (%2)\n\t"
				"pfmul   %%mm1, %%mm5\n\t"
				"movq  48(%1), %%mm2\n\t"
				"movq  56(%1), %%mm3\n\t"
				"pfmul   %%mm0, %%mm6\n\t"
				"pfadd   %%mm5, %%mm4\n\t"
				"pfacc   %%mm4, %%mm4\n\t"
				"movd    %%mm4, 4(%2)\n\t"
				"pfmul   %%mm1, %%mm7\n\t"
				"pfadd   %%mm7, %%mm6\n\t"
				"pfacc   %%mm6, %%mm6\n\t"
				"movd    %%mm6, 8(%2)\n\t"
				"pfmul   %%mm0, %%mm2\n\t"
				"pfmul   %%mm1, %%mm3\n\t"
				"movq   16(%0), %%mm0\n\t"
				"movq   24(%0), %%mm1\n\t"
				"pfadd   %%mm3, %%mm2\n\t"
				"pfacc   %%mm2, %%mm2\n\t"
				"movd    %%mm2, 12(%2)\n\t"
				
				"movq    (%1), %%mm2\n\t"
				"movq   8(%1), %%mm3\n\t"
				"pfmul   %%mm0, %%mm2\n\t"
				"movq  16(%1), %%mm4\n\t"
				"movq  24(%1), %%mm5\n\t"
				"pfmul   %%mm1, %%mm3\n\t"
				"movq  32(%1), %%mm6\n\t"
				"movq  40(%1), %%mm7\n\t"
				"pfmul   %%mm0, %%mm4\n\t"
				"pfadd   %%mm3, %%mm2\n\t"
				"pfacc   %%mm2, %%mm2\n\t"
				"movd    %%mm2, 16(%2)\n\t"
				"pfmul   %%mm1, %%mm5\n\t"
				"movq  48(%1), %%mm2\n\t"
				"movq  56(%1), %%mm3\n\t"
				"pfmul   %%mm0, %%mm6\n\t"
				"pfadd   %%mm5, %%mm4\n\t"
				"pfacc   %%mm4, %%mm4\n\t"
				"movd    %%mm4, 20(%2)\n\t"
				"pfmul   %%mm1, %%mm7\n\t"
				"pfadd   %%mm7, %%mm6\n\t"
				"pfacc   %%mm6, %%mm6\n\t"
				"movd    %%mm6, 24(%2)\n\t"
				"pfmul   %%mm0, %%mm2\n\t"
				"pfmul   %%mm1, %%mm3\n\t"
				"movq   32(%0), %%mm0\n\t"
				"movq   40(%0), %%mm1\n\t"
				"pfadd   %%mm3, %%mm2\n\t"
				"pfacc   %%mm2, %%mm2\n\t"
				"movd    %%mm2, 28(%2)\n\t"
				
				"movq    (%1), %%mm2\n\t"
				"movq   8(%1), %%mm3\n\t"
				"pfmul   %%mm0, %%mm2\n\t"
				"movq  16(%1), %%mm4\n\t"
				"movq  24(%1), %%mm5\n\t"
				"pfmul   %%mm1, %%mm3\n\t"
				"movq  32(%1), %%mm6\n\t"
				"movq  40(%1), %%mm7\n\t"
				"pfmul   %%mm0, %%mm4\n\t"
				"pfadd   %%mm3, %%mm2\n\t"
				"pfacc   %%mm2, %%mm2\n\t"
				"movd    %%mm2, 32(%2)\n\t"
				"pfmul   %%mm1, %%mm5\n\t"
				"movq  48(%1), %%mm2\n\t"
				"movq  56(%1), %%mm3\n\t"
				"pfmul   %%mm0, %%mm6\n\t"
				"pfadd   %%mm5, %%mm4\n\t"
				"pfacc   %%mm4, %%mm4\n\t"
				"movd    %%mm4, 36(%2)\n\t"
				"pfmul   %%mm1, %%mm7\n\t"
				"pfadd   %%mm7, %%mm6\n\t"
				"pfacc   %%mm6, %%mm6\n\t"
				"movd    %%mm6, 40(%2)\n\t"
				"pfmul   %%mm0, %%mm2\n\t"
				"pfmul   %%mm1, %%mm3\n\t"
				"movq   48(%0), %%mm0\n\t"
				"movq   56(%0), %%mm1\n\t"
				"pfadd   %%mm3, %%mm2\n\t"
				"pfacc   %%mm2, %%mm2\n\t"
				"movd    %%mm2, 44(%2)\n\t"
				
				"movq    (%1), %%mm2\n\t"
				"movq   8(%1), %%mm3\n\t"
				"pfmul   %%mm0, %%mm2\n\t"
				"movq  16(%1), %%mm4\n\t"
				"movq  24(%1), %%mm5\n\t"
				"pfmul   %%mm1, %%mm3\n\t"
				"movq  32(%1), %%mm6\n\t"
				"movq  40(%1), %%mm7\n\t"
				"pfmul   %%mm0, %%mm4\n\t"
				"pfadd   %%mm3, %%mm2\n\t"
				"pfacc   %%mm2, %%mm2\n\t"
				"movd    %%mm2, 48(%2)\n\t"
				"pfmul   %%mm1, %%mm5\n\t"
				"movq  48(%1), %%mm2\n\t"
				"movq  56(%1), %%mm3\n\t"
				"pfmul   %%mm0, %%mm6\n\t"
				"pfadd   %%mm5, %%mm4\n\t"
				"pfacc   %%mm4, %%mm4\n\t"
				"movd    %%mm4, 52(%2)\n\t"
				"pfmul   %%mm1, %%mm7\n\t"
				"pfadd   %%mm7, %%mm6\n\t"
				"pfacc   %%mm6, %%mm6\n\t"
				"movd    %%mm6, 56(%2)\n\t"
				"pfmul   %%mm0, %%mm2\n\t"
				"pfmul   %%mm1, %%mm3\n\t"
				"pfadd   %%mm3, %%mm2\n\t"
				"pfacc   %%mm2, %%mm2\n\t"
				"movd    %%mm2, 60(%2)\n\t"
				
				: // no output registers //
				:"r" (a), "r" (b), "r" (c)
				:"memory"
					
			);
			
        
    		}
		

		//clear fp registers from SIMD mode
		asm ("femms\n\t");
	
		//stop the timer
		gettimeofday(&_tstop, &tz);

		//get seconds from the structures
		t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
		t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
		//compute the elapsed time
		ElapsedTime = (float)(t2 - t1);
		
		//display the result
		printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
		printf("%s%g%s", "c[2][3] = ", c[2][3], "\n\n");
	}
	
	else{
	
		printf("3dNow instructions not available on this CPU\n\n");
		
	}
	
	
}//end fnB()

// perform matrix mult. in ASSEMBLY using SSE SIMD instructions
// this is actually implemented as looped matrix-vector multiplication
// as an optimization
void fnC()
{

	//general local declarations
	//integers
	int counter;			//iteration counter
	int bSSEOK;			//cpu supports SSE instructions

	//floats
	float a[4][4];			//matrix a
	float b[4][4];			//matrix b
	float c[4][4];			//c = a x b
	
	//variables for time execution time
	//floats
	float ElapsedTime;	//ticks during loop
	
	//doubles
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//structures needed for basic timing functions
	static struct timeval _tstart, _tstop;
	static struct timezone tz;

	//initialize SSE flag to zero
	bSSEOK = 0;

	//test for SSE capability in cpu
	asm volatile(
	
		"pushl	%%eax\n\t"
		"pushl	%%edx\n\t"
		"xor	%%eax, %%eax\n\t"
		"CPUid\n\t"
		"test	%%eax, %%eax\n\t"
		"jz	NoSSE\n\t"	
		"movl	$0x00000001, %%eax\n\t"
		"CPUid\n\t"
		"test	$0x02000000, %%edx\n\t"
		"jnz	YesSSE\n\t"
		"jmp	NoSSE\n"
"YesSSE:	\n\t"
		"movl	$1, %0\n\t"
		"jmp	EndP3Test\n"
"NoSSE	:	\n\t" 
		"movl	$0,%0\n"
"EndP3Test:	\n\t"	 
		"popl	%%edx\n\t"
		"popl	%%eax"
		:"=g" (bSSEOK)
		:
		:"%eax","%ebx","%ecx","%edx"
		
	);

	if(bSSEOK){

		//display entry message
		printf("\n4x4 Matrix Mult.using SSE SIMD Code...\n");
	
		//initialize the two matrices with some made up values
		a[0][0] = (float)3.654;
		a[0][1] = (float)8.234;
		a[0][2] = (float)3.874;
		a[0][3] = (float)5.792;
		a[1][0] = (float)2.234;
		a[1][1] = (float)8.201;
		a[1][2] = (float)8.333;
		a[1][3] = (float)0.677;
		a[2][0] = (float)5.555;
		a[2][1] = (float)6.633;
		a[2][2] = (float)4.298;
		a[2][3] = (float)9.002;
		a[3][0] = (float)4.094;
		a[3][1] = (float)6.773;
		a[3][2] = (float)1.202;
		a[3][3] = (float)2.118;

		//matrix b in row major ('regular') order
		b[0][0] = (float)3.654;
		b[0][1] = (float)2.348;
		b[0][2] = (float)8.743;
		b[0][3] = (float)7.925;
		b[1][0] = (float)2.342;
		b[1][1] = (float)2.018;
		b[1][2] = (float)3.338;
		b[1][3] = (float)6.770;
		b[2][0] = (float)5.555;
		b[2][1] = (float)6.336;
		b[2][2] = (float)2.984;
		b[2][3] = (float)0.029;
		b[3][0] = (float)0.944;
		b[3][1] = (float)7.736;
		b[3][2] = (float)2.021;
		b[3][3] = (float)1.182;		
		
		//get start time
		gettimeofday(&_tstart, &tz);
	
		for(counter=1; counter <= NumIts; ++counter){
    
			//SSE 4x4 matrix multiply.
			
// adapted from	http://www.cortstratton.org/content/tutorials/HugiCode.html
			
			asm volatile(
			
				// load B matrix into SIMD registers
				"movaps	    (%0), %%xmm4\n\t"
				"movaps	  16(%0), %%xmm5\n\t"
				"movaps	  32(%0), %%xmm6\n\t"
				"movaps	  48(%0), %%xmm7\n\t"
				: // no output registers //
				:"r" (b)
					
			);
			
			
			//iterate through A matrix as row vectors
			//using unrolled loop

			   asm volatile(
			   
			
				// load first row vector into reg 0-3
				// and broadcast
				"movaps   (%0), %%xmm0\n\t"
				"movaps   %%xmm0, %%xmm1\n\t"
				"movaps   %%xmm0, %%xmm2\n\t"
				"movaps   %%xmm0, %%xmm3\n\t"
				"shufps   $0x00, %%xmm0, %%xmm0\n\t"
				"shufps   $0x55, %%xmm1, %%xmm1\n\t"
				"shufps   $0xaa, %%xmm2, %%xmm2\n\t"
				"shufps   $0xff, %%xmm3, %%xmm3\n\t"
				
				//multiply the elements
				"mulps    %%xmm4, %%xmm0\n\t"
				"mulps    %%xmm5, %%xmm1\n\t"
				"mulps    %%xmm6, %%xmm2\n\t"
				"mulps    %%xmm7, %%xmm3\n\t"
				
				//add the products and store result
				"addps    %%xmm0, %%xmm1\n\t"
				"addps    %%xmm3, %%xmm2\n\t"
				"addps    %%xmm2, %%xmm1\n\t"
				"movaps   %%xmm1, (%1)\n\t"
				
				// load second row vector into reg 0-3
				// and broadcast
				"movaps   16(%0), %%xmm0\n\t"
				"movaps   %%xmm0, %%xmm1\n\t"
				"movaps   %%xmm0, %%xmm2\n\t"
				"movaps   %%xmm0, %%xmm3\n\t"
				"shufps   $0x00, %%xmm0, %%xmm0\n\t"
				"shufps   $0x55, %%xmm1, %%xmm1\n\t"
				"shufps   $0xaa, %%xmm2, %%xmm2\n\t"
				"shufps   $0xff, %%xmm3, %%xmm3\n\t"
				
				//multiply the elements
				"mulps    %%xmm4, %%xmm0\n\t"
				"mulps    %%xmm5, %%xmm1\n\t"
				"mulps    %%xmm6, %%xmm2\n\t"
				"mulps    %%xmm7, %%xmm3\n\t"
				
				//add the products and store result
				"addps    %%xmm0, %%xmm1\n\t"
				"addps    %%xmm3, %%xmm2\n\t"
				"addps    %%xmm2, %%xmm1\n\t"
				"movaps   %%xmm1, 16(%1)\n\t"
				
				// load third row vector into reg 0-3
				// and broadcast
				"movaps   32(%0), %%xmm0\n\t"
				"movaps   %%xmm0, %%xmm1\n\t"
				"movaps   %%xmm0, %%xmm2\n\t"
				"movaps   %%xmm0, %%xmm3\n\t"
				"shufps   $0x00, %%xmm0, %%xmm0\n\t"
				"shufps   $0x55, %%xmm1, %%xmm1\n\t"
				"shufps   $0xaa, %%xmm2, %%xmm2\n\t"
				"shufps   $0xff, %%xmm3, %%xmm3\n\t"
				
				//multiply the elements
				"mulps    %%xmm4, %%xmm0\n\t"
				"mulps    %%xmm5, %%xmm1\n\t"
				"mulps    %%xmm6, %%xmm2\n\t"
				"mulps    %%xmm7, %%xmm3\n\t"
				
				//add the products and store result
				"addps    %%xmm0, %%xmm1\n\t"
				"addps    %%xmm3, %%xmm2\n\t"
				"addps    %%xmm2, %%xmm1\n\t"
				"movaps   %%xmm1, 32(%1)\n\t"
				
				// load fourth row vector into reg 0-3
				// and broadcast
				"movaps   48(%0), %%xmm0\n\t"
				"movaps   %%xmm0, %%xmm1\n\t"
				"movaps   %%xmm0, %%xmm2\n\t"
				"movaps   %%xmm0, %%xmm3\n\t"
				"shufps   $0x00, %%xmm0, %%xmm0\n\t"
				"shufps   $0x55, %%xmm1, %%xmm1\n\t"
				"shufps   $0xaa, %%xmm2, %%xmm2\n\t"
				"shufps   $0xff, %%xmm3, %%xmm3\n\t"
				
				//multiply the elements
				"mulps    %%xmm4, %%xmm0\n\t"
				"mulps    %%xmm5, %%xmm1\n\t"
				"mulps    %%xmm6, %%xmm2\n\t"
				"mulps    %%xmm7, %%xmm3\n\t"
				
				//add the products and store result
				"addps    %%xmm0, %%xmm1\n\t"
				"addps    %%xmm3, %%xmm2\n\t"
				"addps    %%xmm2, %%xmm1\n\t"
				"movaps   %%xmm1, 48(%1)"
				: // no output registers //
				:"r" (a), "r" (c)
				
			   );
        
    		}
		
	
		//stop the timer
		gettimeofday(&_tstop, &tz);

		//get seconds from the structures
		t1 = (double)_tstart.tv_sec + (double)_tstart.tv_usec * 0.000001;
		t2 = (double)_tstop.tv_sec + (double)_tstop.tv_usec * 0.000001;
	
		//compute the elapsed time
		ElapsedTime = (float)(t2 - t1);

		//display the result
		printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
		printf("%s%g%s","c[2][3] = ",c[2][3],"\n\n");
	}
	
	else{
	
		printf("SSE instructions not available on this CPU\n\n");
		
	}
	
	
}//end fnC()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
	system("clear");
	
	do {
		
		//present menu to user
		printf("\nEnter the Number of the routine to run, and press <Enter> \n");
		printf("1\t4x4 Matrix Multiplication using x87 instructions\n");
		printf("2\t4x4 Matrix Multiplication using 3dNow! SIMD Instructions\n");
		printf("3\t4x4 Matrix Multiplication using SSE SIMD Instructions\n");
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
	
			case '2': fnB();
				break;
				
			case '3': fnC();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

} // End Main

