// Function05.c : implementation file
//
// This program demonstrates the use of global data rather than passing
// parameters to functions.  No work is done in the computational
// functions.
//
// The function call using local data, parameter passed by value is in fnA().
// The function call using local data, parameter passed by reference is in fnB().
// The function call using global data is in fnC()

#include <windows.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

//defines
#define NumIts 50000000		//number of iterations

double gR;		//global distance
double gE;		//global energy

//functions
void fnA();			
void fnB();
void fnC();			

//receives reference
double calc_ljenergyRef(double *Rptr);

//receives value
double calc_ljenergyVal(double R);

//"works" on global data
int calc_ljenergyglobal();


//call by passing a single double value
void fnA()
{

	//general local declarations
	//integers
	int counter;			  // iteration counter

	//doubles
	double r;                 //distance, a double to pass to function
    double energy;            //energy, holder for function return value
	
	//variables for time execution time
	//doubles
	double ElapsedTime;	//ticks during loop
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//just a value to pass
    r = 1.234;
	
	//display entry message
	printf("\nCalling Function by value, local data ...\n");
	
	//get start time
	t1 = GetTickCount();
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//call the function, passing a double and
        //returning a double
        energy = calc_ljenergyVal(r);   

    }
	
	//stop the timer
	t2 = GetTickCount();
	
	//compute the elapsed time
	ElapsedTime = (t2 - t1) * 0.001;

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");

}//end fnA()

//calls the function passing a reference
void fnB()
{
	//general local declarations
	//integers
	int counter;			  // iteration counter

	//doubles
	double r;                 //distance, a double to pass to function
    double energy;            //energy, holder for function return value
	
	//variables for time execution time
	//doubles
	double ElapsedTime;	//ticks during loop
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//just a value to pass
    r = 1.234;
	
	//display entry message
	printf("\nCalling Function by reference, local data ...\n");
	
	//get start time
	t1 = GetTickCount();
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//call the function, passing a refence and
        //returning a double
        energy = calc_ljenergyRef(&r);   

    }
	
	//stop the timer
	t2 = GetTickCount();
	
	//compute the elapsed time
	ElapsedTime = (t2 - t1) * 0.001;

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");

}  // End fnB()

//calls the function using global data
void fnC()
{
	//general local declarations
	//integers
	int counter;			  // iteration counter
	int dummy;				  // return a value
	
	//variables for time execution time
	//doubles
	double ElapsedTime;	//ticks during loop
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//just a value to pass
    gR = 1.234;
	
	//display entry message
	printf("\nCalling Function using global data ...\n");
	
	//get start time
	t1 = GetTickCount();
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//call the function, using global data and
        //returning an integer (say a success code)
        dummy = calc_ljenergyglobal();   

    }
	
	//stop the timer
	t2 = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (t2 - t1) * 0.001;

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");

}  // End fnC()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
	system("cls");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tCall Function passing parameter by value and using local data\n");
		printf("2\tCall Function passing parameter by reference and using local data\n");
		printf("3\tCall Function that uses global data for parameters and data\n");
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
	
			case '2': fnB();
				break;

			case '3': fnC();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

} // End Main

//receives the data value
double calc_ljenergyVal (double R)
{

	double En;

	En = R;

	return (0);

}

//receives the data reference
double calc_ljenergyRef (double *Rptr)
{

	double En;

	En = *Rptr;

	return (En);

}

int calc_ljenergyglobal()
{

	gE = gR;

	return(0);

}
