
//RISC05 compares the combined distance and energy calculation using an
//unnecessary sqrt(r^2) to the calculation without square root of the square.


#include <windows.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

//defines
#define NumIts 10000000		//number of iterations

//functions
void fnCISC();			//With sqrt(r^2)
void fnRISC();			//WithOut sqrt(r^2)

//computes distance and energy in with unneeded sqrt(r^2)
void fnCISC()
{

	//local declarations
	//integers
	int i;			//iteration counter
	
	
	//doubles
	double x1 = 1.234;	//some values to use for coordinates
	double x2 = 2.675;		
	double y1 = 0.045;
	double y2 = 2.073;
	double tempx;		//deltax, then deltax^2
	double tempy;		//deltay, then deltay^2
	double r;		//distance between particles
	double r6;		//r^6
	double r12;		//r^12
	double energy;		//energy

	//values for timing functions
	double ElapsedTime;	//ticks during loop	
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//display entry message
	printf("\nComputing CISC Style Distance and Energy...\n");
	
	//get start time
	t1 = GetTickCount();
	
	for(i=1; i<= NumIts; ++i){
	
		//compute deltax and deltay
		tempx = x1 - x2;
		tempy = y1 - y2;
		
		//compute deltax^2 and deltay^2
		tempx *= tempx;
		tempy *= tempy;
		
		//compute r
		r = sqrt(tempx + tempy);
		
		//compute r^6
		r6 = r * r * r * r * r * r;
		
		//compute r^12
		r12 = r6 * r6;

		//compute the energy
		energy = 1/r12 - 1/r6;

	}
	
	//stop the timer
	t2 = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (t2 - t1) * 0.001;

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	printf("%s%g%s", "Energy = ", energy, " \n\n");
	
}//end fnCISC()

void fnRISC()
{

	//local declarations
	//integers
	int i;			//iteration counter
		
	//doubles
	double x1 = 1.234;	//some values to use for coordinates
	double x2 = 2.675;		
	double y1 = 0.045;
	double y2 = 2.073;
	double tempx;		//deltax, then deltax^2
	double tempy;		//deltay, then deltay^2
	double r;		//(distance between particles)^2
	double r6;		//r^6
	double r12;		//r^12
	double energy;		//computed energy
	
	//values for timing functions
	double ElapsedTime;	//ticks during loop	
	double t1;		//ticks at start of loop
	double t2;		//ticks at loop end
	
	//display entry message
	printf("\nComputing LJ Energies in RISC Style...\n");
	
	//get start time
	t1 = GetTickCount();
	
	for(i=1; i<= NumIts; ++i){
	
		//compute deltax and deltay
		tempx = x1 - x2;
		tempy = y1 - y2;
		
		//compute deltax^2 and deltay^2
		tempx *= tempx;
		tempy *= tempy;
		
		//compute distance^2
		r = tempx + tempy;
		
		//compute r^6, which is (r^2)^3
		r6 = r * r * r;
		
		//compute r^12
		r12 = r6 * r6;
		
		//compute energy
		energy = 1/r12 - 1/r6;
		
	}
	
	//stop the timer
	t2 = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (t2 - t1) * 0.001;

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	printf("%s%g%s", "Energy = ", energy, " \n\n");
	
}//end fnRISC()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
	system("cls");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tDistance and LJ Energies in CISC Style\n");
		printf("2\tDistance and LJ Energies in RISC Style\n");
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);

		//inchar = getchar();

		//call the user selected function
		switch	(inchar){

			case '1': fnCISC();
				break;
	
			case '2': fnRISC();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

}
