// LookUp03.c : implementation file
//
// This program compares the performance of explicit computations of
// the several common functions to the use of a Look-Up Table.
//
// The Look-up Table assignment routine is in fnA().
// The Explicit sin(x) routine is in fnB().
// The Explicit ln(x) routine is in fnC().
// The Explicit pow(x,y) routine is in fnD().
// The Look-Up Table sin(x) routine is in fnE().
// The Look-Up Table ln(x) routine is in fnF().
// The Look-Up Table pow(x,y) routine is in fnG().

#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

//defines
#define NumIts 50000000		//number of iterations

//global declarations
//integers
int bINIT;	 //flag = true if LUT has been initialized

//doubles
double Angle = 1.31266;		// an angle at which to get sine(angle)
double Base = 6.345;		// test values for x^y:  6.345^4.78
double Expon = 4.78;
double LogArg = 5.32;		// a value at which to get ln(LogArg)

double SinLU[70001];		//look up table for sin(x)
double LogLU[70001];		//look up table for ln(x)
double exponLU[70001];		//look up table for x^y

//functions
void fnA();			
void fnB();			
void fnC();
void fnD();
void fnE();
void fnF();
void fnG();


//initialize the look up table
void fnA()
{

	//local declarations
	//integers
	int counter;			// iteration counter
	
	//doubles
	double luAngle;			//angle for sin(angle)
	double luExpon;			//exponent for x^y function
	double luLogArg;		//argument for ln(x) function
	
	//variables for time execution time
	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop
	
	//display entry message
	printf("\nInitializing Look-Up Tables ...\n");
	
	//get start time
	starttime = GetTickCount();
	
	for(counter=1; counter <= 70000; ++counter){
    
		//compute angle from counter, and assign sin() array
		luAngle = (double)counter / (double)10000;
        	SinLU[counter] = sin(luAngle);
        
		//compute exponent from counter, and assign x^y array
        	luExpon = (double)counter / (double)10000;
        	exponLU[counter] = pow(Base, luExpon);
    
		//compute argument from counter, and assign ln(x) array
        	luLogArg = (double)counter / (double)10000;
        	LogLU[counter] = log(luLogArg);
  

        }
	
	//stop the timer
	stoptime = GetTickCount();
	
	//compute the elapsed time
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result
	printf("%s%g%s", "\nLook-Up Tables Initialized in ", ElapsedTime, " seconds\n");
	
	//set flag to 'true'
	bINIT = 1;
	
}//end fnA()

// Explicit sin(x)
void fnB()
{

	//general local declarations
	//integers
	int counter;			//iteration counter

	//doubles
	double Value;		//return value for sin(x) function
	
	//variables for time execution time
	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop
	
	//display entry message
	printf("\nStarting Iterative Explicit sin(x) calculation...\n");
	
	//get start time
	starttime = GetTickCount();
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//explicit computation of sin(x)
		Value = sin(Angle);

        }
	
	//stop the timer
	stoptime = GetTickCount();
	
	//compute the elapsed time
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result
	printf("%s%g%s", "\nComputed sin(x) in ", ElapsedTime, " seconds\n");
	
}//end fnB()

// Explicit ln(x)
void fnC()
{

	//general local declarations
	//integers
	int counter;			//iteration counter

	//doubles
	double Value;		//return value for log(x) function
	
	//variables for time execution time
	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop
	
	//display entry message
	printf("\nStarting Iterative Explicit ln(x) calculation...\n");
	
	//get start time
	starttime = GetTickCount();
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//explicitly compute the ln(x)
		Value = log(LogArg);

        }
	
	//stop the timer
	stoptime = GetTickCount();
	
	//compute the elapsed time
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result
	printf("%s%g%s", "\nComputed ln(x) in ", ElapsedTime, " seconds\n");
	
}//end fnC()

// Explicit pow(x,y)
void fnD()
{

	//general local declarations
	//integers
	int counter;			//iteration counter

	//doubles
	double Value;		//return value for log(x) function
	
	//variables for time execution time
	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop
	
	//display entry message
	printf("\nStarting Iterative Explicit pow(x,y) calculation...\n");
	
	//get start time
	starttime = GetTickCount();
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//explicitly compute x^y
		Value = pow(Base,Expon);

        }
	
	//stop the timer
	stoptime = GetTickCount();
	
	//compute the elapsed time
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result
	printf("%s%g%s", "\nComputed pow(x,y) in ", ElapsedTime, " seconds\n");
	
}//end fnD()

// interative sin(x) by Look-Up Table
void fnE()
{

	//general local declarations
	//integers
	int counter;		//iteration counter
	int AngleIndex;		//lookup table index

	//doubles
   	double Value;		//return value for lookup sin(x)
	
	//variables for time execution time
	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop
	
	//display entry message
	printf("\nStarting Iterative sin(x) via LUT ...\n");
	
	//get start time
	starttime = GetTickCount();
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//compute the index from desired angle
		AngleIndex = (int)(Angle * (double)10000);

		//look-up sin(x)
		Value = SinLU[AngleIndex];
		
        }
	
	//stop the timer
	stoptime = GetTickCount();
	
	//compute the elapsed time
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result
	printf("%s%g%s", "\nLooked Up sin(x) in ", ElapsedTime, " seconds\n");
	
}//end fnE()

// interative ln(x) by Look-Up Table
void fnF()
{

	//general local declarations
	//integers
	int counter;		//iteration counter
	int LogIndex;		//lookup table index

	//doubles
   	double Value;		//return value for lookup sin(x)
	
	//variables for time execution time
	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop
	
	//display entry message
	printf("\nStarting Iterative ln(x) via LUT ...\n");
	
	//get start time
	starttime = GetTickCount();
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//compute the index from desired argument
		LogIndex = (int)(LogArg * (double)10000);

		//look up ln(x)
		Value = LogLU[LogIndex];
		
        }
	
	//stop the timer
	stoptime = GetTickCount();
	
	//compute the elapsed time
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result
	printf("%s%g%s", "\nLooked Up ln(x) in ", ElapsedTime, " seconds\n");
	
}//end fnF()

// interative pow(x,y) by Look-Up Table
void fnG()
{

	//general local declarations
	//integers
	int counter;		//iteration counter
	int PwrIndex;		//lookup table index

	//doubles
   	double Value;		//return value for lookup sin(x)
	
	//variables for time execution time
	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop
	
	//display entry message
	printf("\nStarting Iterative pow(x,y) via LUT ...\n");
	
	//get start time
	starttime = GetTickCount();
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//compute index from desired exponent
		PwrIndex = (int)(Expon * (double)10000);

		//look up pow(x,y)
		Value = exponLU[PwrIndex];
		
        }
	
	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result
	printf("%s%g%s", "\nLooked Up pow(x,y) in ", ElapsedTime, " seconds\n");
	
}//end fnG()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//initialize bINIT to NOT INITIALIZED
	bINIT = 0;
	
	//clear terminal
	system("cls");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tInitialize the Look-Up Table\n");
		
		if(bINIT){
			printf("2\tIterative Explicit sin(x)\n");
			printf("3\tIterative Explicit ln(x)\n");
			printf("4\tIterative Explicit pow(x,y)\n");
			printf("5\tIterative sin(x) via Look-Up Table\n");
			printf("6\tIterative ln(x) via Look-Up Table\n");
			printf("7\tIterative pow(x,y) via Look-Up Table\n");
			printf("q\tEnd this program\n");
		}
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
	
			case '2': fnB();
				break;
				
			case '3': fnC();
				break;
				
			case '4': fnD();
				break;
				
			case '5': fnE();
				break;
				
			case '6': fnF();
				break;
				
			case '7': fnG();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

} // End Main
