// LookUp04.c : implementation file
//
// This program compares the performance of explicit computations of
// the combined distance and LJ energy to the use of a Look-Up Tables
// for these functions.
//
// The Look-up Table assignment routine is in fnA().
// The Explicit calculation routine is in fnB().
// The Look-Up Table calculation routine is in fnC().

#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

//defines
#define NumIts 100000000		//number of iterations
#define x_1 		103					//just some values to use
#define x_2 		373					//for coordinates of two particles
#define y_1			271
#define y_2			144
#define depth		10.22				//well depth for He -- He
#define sigma		258					//E(sigma)=0, for He -- He
#define sigma6		2.9492951E14		//sigma^6
#define sigma12		8.6983418E28		//sigma^12


//global declarations
//integers
int r[2831][2831];			//lookup table for distance

int bINIT;				//flag = true if LUT has been initialized

//doubles
double e[2831];				//lookup table for energy

//functions
void fnA();			
void fnB();			
void fnC();


//initialize the look up table
void fnA()
{

	//local declarations
	//integers
	int counter;			//counter for E and deltax
	int yCounter;			//counter for deltay
	
	
	//doubles
	double rvalue;			//actual r for E(r)
					//will also be used for deltax
	double r6;			//rvalue^6
	double r12;			//rvalue^12
	double deltay;			//y2-y1, then (y2-y1)^2
	
	//variables for time execution time
	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop
	
	//display entry message
	printf("\nInitializing Look-Up Table ...\n");
	
	//get start time
	starttime = GetTickCount();
	
	for(counter=1; counter <= 2830; ++counter){
    
		//convert index to double
		rvalue = (double)(counter);

		if (rvalue>0){
			//compute r^6
			r6 = rvalue * rvalue * rvalue * rvalue * rvalue * rvalue;

			//compute r^12
			r12 = r6 * r6;

			//compute E(r)
			e[counter] = sigma12/r12 - sigma6/r6;
			e[counter] *= depth;
		}

		//compute r(deltax, deltay), using rvalue for deltax
		//compute deltax^2
		rvalue *= rvalue;

		for(yCounter=0; yCounter <= 2830; ++yCounter){

			//convert to double
			deltay = (double)yCounter;

			//compute deltay^2
			deltay *= deltay;

			//compute sum
			deltay = deltay + rvalue;

			//and take square root
			//only need integer part, since the r(x,y) entries
			//are used as indices to use to enter e(r) array
			r[counter][yCounter] = (int)sqrt(deltay);
		}  

        }
	
	//stop the timer
	stoptime = GetTickCount();
	
	//compute the elapsed time
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result
	printf("%s%g%s", "\nLook-Up Table Initialized in ", ElapsedTime, " seconds\n");
	
	//set flag to 'true'
	bINIT = 1;
	
}//end fnA()

// interative distance and energy by actual computation
void fnB()
{

	//general local declarations
	//integers
	int counter;			//iteration counter

	//doubles
	double delx;			//delta x
	double dely;			//delta y
	double r_2;			//distance, r^2
	double r_6;			//r^6
	double r_12;			//r^12
	double energy;			//computed energy
	
	//variables for time execution time
	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop
	
	//display entry message
	printf("\nStarting Iterative Combined Distance and Energy calculation...\n");
	
	//get start time
	starttime = GetTickCount();
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//compute deltax and deltay from the coordinates
		delx = (double)(x_2-x_1);
		dely = (double)(y_2-y_1);

		//compute r^2 using RISC algorithm from Ch.8
		delx = delx * delx;
		dely = dely * dely;

		r_2 = delx + dely;

		//compute energy via RISC algorithm from Ch.8
		//using r^2
		r_6 = r_2 * r_2 * r_2;
		r_12 = r_6 * r_6;

		energy = sigma12/r_12 - sigma6/r_6;
		energy *= depth;

        }
	
	//stop the timer
	stoptime = GetTickCount();
	
	//compute the elapsed time
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result
	printf("%s%g%s", "\nCombined Explicit Calculatino in ", ElapsedTime, " seconds\n");
	
}//end fnB()

// interative Boltzmann Factor by Look-Up Table
void fnC()
{

	//general local declarations
	//integers
	int counter;			//iteration counter
	int delx;			//deltax
	int dely;			//deltay
	int EIndex;			//looked up r value, used
						//as index to the E lookup table

	//doubles
   	double energy;                 //computed energy
	
	//variables for time execution time
	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop
	
	//display entry message
	printf("\nStarting Iterative Combined Distance and Energy via LUT ...\n");
	
	//get start time
	starttime = GetTickCount();
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//compute the indices to lookup the distance, r
		delx = labs(x_1-x_2);
		dely = labs(y_1-y_2);

		//lookup the distance
		EIndex = r[delx][dely];

		//lookup the energy
		energy = e[EIndex];
		
        }
	
	//stop the timer
	stoptime = GetTickCount();
	
	//compute the elapsed time
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result
	printf("%s%g%s", "\nLooked Up Distance and Energy in ", ElapsedTime, " seconds\n");
	
}//end fnC()

//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//initialize bINIT to NOT INITIALIZED
	bINIT = 0;
	
	//clear terminal
	system("cls");
	
	do {
		
		//present menu to user
		printf("Enter the Number of the routine to run, and press <Enter> \n");
		printf("1\tInitialize the Look-Up Table\n");
		
		if(bINIT){
			printf("2\tIterative Combined Distance and Energy Factor\n");
			printf("3\tIterative Combined Distance and Energy via Look-Up Table\n");
			printf("q\tEnd this program\n");
		}
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
	
			case '2': fnB();
				break;
				
			case '3': fnC();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

} // End Main
