// Par02.c : implementation file
//
// This program compares the 4x4 matrix mult. using straight x87 floating
// point code to use of 3dNow! and SSE SIMD code.  If the run-time processor
// does not support SIMD, the SIMD routines simply exit with a 
// message (CPUid MUST be supported, however...I did not test for
// that support before making the calls to CPUid).
//
// The x87 routine is in fnA()
// The 3dNow! routine is in fnB()
// The SSE routine is in fnC()

#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <stdlib.h>

//defines
#define NumIts 50000000		//number of iterations

//functions
void fnA();			
void fnB();			

// perform matrix multiplication using x87 code as compiled
// from C unrolled loop as demo Loop02.
void fnA()
{

	//general local declarations
	//integers
	int counter;			  // iteration counter

	//floats
	float a[4][4];		//matrix a
	float b[4][4];		//matrix b
	float c[4][4];		//resultant product result
	
	//variables for time execution time
	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop

	//assign some values to the matrices
	//matrix a in row major ('regular') order
	a[0][0] = (float)3.654;
	a[0][1] = (float)8.234;
	a[0][2] = (float)3.874;
	a[0][3] = (float)5.792;
	a[1][0] = (float)2.234;
	a[1][1] = (float)8.201;
	a[1][2] = (float)8.333;
	a[1][3] = (float)0.677;
	a[2][0] = (float)5.555;
	a[2][1] = (float)6.633;
	a[2][2] = (float)4.298;
	a[2][3] = (float)9.002;
	a[3][0] = (float)4.094;
	a[3][1] = (float)6.773;
	a[3][2] = (float)1.202;
	a[3][3] = (float)2.118;

	//matrix b in row major ('regular') order
	b[0][0] = (float)3.654;
	b[0][1] = (float)2.348;
	b[0][2] = (float)8.743;
	b[0][3] = (float)7.925;
	b[1][0] = (float)2.342;
	b[1][1] = (float)2.018;
	b[1][2] = (float)3.338;
	b[1][3] = (float)6.770;
	b[2][0] = (float)5.555;
	b[2][1] = (float)6.336;
	b[2][2] = (float)2.984;
	b[2][3] = (float)0.029;
	b[3][0] = (float)0.944;
	b[3][1] = (float)7.736;
	b[3][2] = (float)2.021;
	b[3][3] = (float)1.182;
		
	//display entry message
	printf("\nComputing Unrolled Matrix Mult. using only x87 ...\n");
	
	//get start time
	starttime = GetTickCount();
	
	for(counter=1; counter <= NumIts; ++counter){
    
		//4x4 matrix multiplication, compiles to straight
		//x87 code
		//j = 0
		c[0][ 0] = a[0][ 0] * b[0][ 0] + a[0][ 1] * b[1][ 0] + a[0][ 2] * b[2][ 0] + a[0][ 3] * b[3][ 0];
		c[0][ 1] = a[0][ 0] * b[0][ 1] + a[0][ 1] * b[1][ 1] + a[0][ 2] * b[2][ 1] + a[0][ 3] * b[3][ 1];
		c[0][ 2] = a[0][ 0] * b[0][ 2] + a[0][ 1] * b[1][ 2] + a[0][ 2] * b[2][ 2] + a[0][ 3] * b[3][ 2];
		c[0][ 3] = a[0][ 0] * b[0][ 3] + a[0][ 1] * b[1][ 3] + a[0][ 2] * b[2][ 3] + a[0][ 3] * b[3][ 3];
	
		//j = 1
		c[1][ 0] = a[1][ 0] * b[0][ 0] + a[1][ 1] * b[1][ 0] + a[1][ 2] * b[2][ 0] + a[1][ 3] * b[3][ 0];
		c[1][ 1] = a[1][ 0] * b[0][ 1] + a[1][ 1] * b[1][ 1] + a[1][ 2] * b[2][ 1] + a[1][ 3] * b[3][ 1];
		c[1][ 2] = a[1][ 0] * b[0][ 2] + a[1][ 1] * b[1][ 2] + a[1][ 2] * b[2][ 2] + a[1][ 3] * b[3][ 2];
		c[1][ 3] = a[1][ 0] * b[0][ 3] + a[1][ 1] * b[1][ 3] + a[1][ 2] * b[2][ 3] + a[1][ 3] * b[3][ 3];
		
		//j = 2
		c[2][ 0] = a[2][ 0] * b[0][ 0] + a[2][ 1] * b[1][ 0] + a[2][ 2] * b[2][ 0] + a[2][ 3] * b[3][ 0];
		c[2][ 1] = a[2][ 0] * b[0][ 1] + a[2][ 1] * b[1][ 1] + a[2][ 2] * b[2][ 1] + a[2][ 3] * b[3][ 1];
		c[2][ 2] = a[2][ 0] * b[0][ 2] + a[2][ 1] * b[1][ 2] + a[2][ 2] * b[2][ 2] + a[2][ 3] * b[3][ 2];
		c[2][ 3] = a[2][ 0] * b[0][ 3] + a[2][ 1] * b[1][ 3] + a[2][ 2] * b[2][ 3] + a[2][ 3] * b[3][ 3];

		//j = 3
		c[3][ 0] = a[3][ 0] * b[0][ 0] + a[3][ 1] * b[1][ 0] + a[3][ 2] * b[2][ 0] + a[3][ 3] * b[3][ 0];
		c[3][ 1] = a[3][ 0] * b[0][ 1] + a[3][ 1] * b[1][ 1] + a[3][ 2] * b[2][ 1] + a[3][ 3] * b[3][ 1];
		c[3][ 2] = a[3][ 0] * b[0][ 2] + a[3][ 1] * b[1][ 2] + a[3][ 2] * b[2][ 2] + a[3][ 3] * b[3][ 2];
		c[3][ 3] = a[3][ 0] * b[0][ 3] + a[3][ 1] * b[1][ 3] + a[3][ 2] * b[2][ 3] + a[3][ 3] * b[3][ 3];
		
   	}
	
	//stop the timer
	stoptime = GetTickCount();
	
	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;

	//display the result
	printf("%s%g%s", "\nElapsed Time = ", ElapsedTime, " seconds\n");
	printf("%s%g%s", "c[2][3] = ", c[2][3], "\n\n");
	
}//end fnA()

// perform matrix multiplication in ASSEMBLY using 3dNow! SIMD instructions
void fnB()
{

	//general local declarations
	//integers
	int counter;			//iteration counter
	int bAMDOK;			//cpu supports 3DNow! instructions

	//floats
	float a[4][4];			//matrix a, 4-d
	float b[4][4];			//matrix b, 4-d
	float c[4][4];			//matrix product result
	
	//variables for time execution time
	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop

	//initialize AMD flag to zero
	bAMDOK = 0;

	//test for 3DNow! capability in cpu
	//this is adapted from code in AMD 3DNow! Tech. Manual
	_asm {

			push	eax
			push	edx
			xor		eax, eax		; CPUid function Vendor ID
			CPUid
			test	eax, eax
			jz		$No3dNow		
			mov		eax, 80000000h	; CPUid function Largest Ext. Val
			CPUid
			cmp		eax, 80000001h
			jl		$No3dNow
			mov		eax, 80000001h	; CPUid function Sig. + features
			CPUid
			test	edx, 80000000h
			jnz		$Yes3dNow
			jmp		$No3dNow
$Yes3dNow:	mov		bAMDOK, 1		; bAMDOK = true
			jmp		$EndCpuTest
$No3dNow:	mov		bAMDOK, 0		; bAMDOK = false
$EndCpuTest:		pop		edx
			pop		eax

	}

	if(bAMDOK){

		//assign some values to the matrices
		//matrix a in row major ('regular') order
		a[0][0] = (float)3.654;
		a[0][1] = (float)8.234;
		a[0][2] = (float)3.874;
		a[0][3] = (float)5.792;
		a[1][0] = (float)2.234;
		a[1][1] = (float)8.201;
		a[1][2] = (float)8.333;
		a[1][3] = (float)0.677;
		a[2][0] = (float)5.555;
		a[2][1] = (float)6.633;
		a[2][2] = (float)4.298;
		a[2][3] = (float)9.002;
		a[3][0] = (float)4.094;
		a[3][1] = (float)6.773;
		a[3][2] = (float)1.202;
		a[3][3] = (float)2.118;

		//matrix b in COLUMN MAJOR order!!
		b[0][0] = (float)3.654;
		b[1][0] = (float)2.348;
		b[2][0] = (float)8.743;
		b[3][0] = (float)7.925;
		b[0][1] = (float)2.342;
		b[1][1] = (float)2.018;
		b[2][1] = (float)3.338;
		b[3][1] = (float)6.770;
		b[0][2] = (float)5.555;
		b[1][2] = (float)6.336;
		b[2][2] = (float)2.984;
		b[3][2] = (float)0.029;
		b[0][1] = (float)0.944;
		b[1][3] = (float)7.736;
		b[2][3] = (float)2.021;
		b[3][3] = (float)1.182;

		//start the timer
		starttime = GetTickCount(); 	

		//switch registers to mmx mode
		_asm	femms

		for(counter=1; counter <= NumIts; ++counter){

			
			//SIMD 3dNow 4x4 matrix multiplication
			//this may not be fully optimized in terms of 
			//pairing and data reuse.
			//NOTE: matrix b MUST be COLUMN MAJOR for this
			//algorithm to work as written!!!
			_asm{

				;compute c_00-c_03
				movq	mm0, a[0]
				movq	mm1, a[8]
				movq	mm2, b[0]
				movq	mm3, b[8]
				pfmul	mm2, mm0
				movq	mm4, b[16]
				movq	mm5, b[24]
				pfmul	mm3, mm1
				movq	mm6, b[32]
				movq	mm7, b[40]
				pfmul	mm4, mm0
				pfadd	mm2, mm3
				pfacc	mm2, mm2
				movq	c[0], mm2
				pfmul	mm5, mm1
				movq	mm3, b[56]
				movq	mm2, b[48]
				pfmul	mm6, mm0
				pfadd	mm4, mm5
				pfacc	mm4, mm4
				movq	c[4], mm4
				pfmul	mm7, mm1
				pfadd	mm6, mm7
				pfacc	mm6, mm6
				movq	c[8], mm6
				pfmul	mm2, mm0
				movq	mm0, a[16]
				pfmul	mm3, mm1
				movq	mm1, a[24]
				pfadd	mm2, mm3
				pfacc	mm2, mm2
				movq	c[12], mm2
	
				;compute c_10-c_13
				movq	mm2, b[0]
				movq	mm3, b[8]
				pfmul	mm2, mm0
				movq	mm4, b[16]
				movq	mm5, b[24]
				pfmul	mm3, mm1
				movq	mm6, b[32]
				movq	mm7, b[40]
				pfmul	mm4, mm0
				pfadd	mm2, mm3
				pfacc	mm2, mm2
				movq	c[16], mm2
				pfmul	mm5, mm1
				movq	mm3, b[56]
				movq	mm2, b[48]
				pfmul	mm6, mm0
				pfadd	mm4, mm5
				pfacc	mm4, mm4
				movq	c[20], mm4
				pfmul	mm7, mm1
				pfadd	mm6, mm7
				pfacc	mm6, mm6
				movq	c[24], mm6
				pfmul	mm2, mm0	
				movq	mm0, a[32]
				pfmul	mm3, mm1
				movq	mm1, a[40]
				pfadd	mm2, mm3
				pfacc	mm2, mm2
				movq	c[28], mm2

				;compute c_20-c_23
				movq	mm2, b[0]
				movq	mm3, b[8]
				pfmul	mm2, mm0
				movq	mm4, b[16]
				movq	mm5, b[24]
				pfmul	mm3, mm1
				movq	mm6, b[32]
				movq	mm7, b[40]
				pfmul	mm4, mm0
				pfadd	mm2, mm3
				pfacc	mm2, mm2
				movq	c[32], mm2
				pfmul	mm5, mm1
				movq	mm3, b[56]
				movq	mm2, b[48]
				pfmul	mm6, mm0
				pfadd	mm4, mm5
				pfacc	mm4, mm4
				movq	c[36], mm4
				pfmul	mm7, mm1
				pfadd	mm6, mm7
				pfacc	mm6, mm6
				movq	c[40], mm6
				pfmul	mm2, mm0
				movq	mm0, a[48]
				pfmul	mm3, mm1
				movq	mm1, a[56]
				pfadd	mm2, mm3
				pfacc	mm2, mm2
				movq	c[44], mm2

				;compute c_30-c_33
				movq	mm2, b[0]
				movq	mm3, b[8]
				pfmul	mm2, mm0
				movq	mm4, b[16]
				movq	mm5, b[24]
				pfmul	mm3, mm1
				movq	mm6, b[32]
				movq	mm7, b[40]
				pfmul	mm4, mm0
				pfadd	mm2, mm3
				pfacc	mm2, mm2
				movq	c[48], mm2
				pfmul	mm5, mm1
				movq	mm3, b[56]
				movq	mm2, b[48]
				pfmul	mm6, mm0
				pfadd	mm4, mm5
				pfacc	mm4, mm4
				movq	c[52], mm4
				pfmul	mm7, mm1
				pfadd	mm6, mm7
				pfacc	mm6, mm6
				movq	c[56], mm6
				pfmul	mm2, mm0
				pfmul	mm3, mm1
				pfadd	mm2, mm3
				pfacc	mm2, mm2
				movd	c[60], mm2

			}

		}

		//switch registers back to x87 mode
		_asm	femms
	
		//stop the timer
		stoptime = GetTickCount();		
	
		//compute the elapsed time, in seconds
		ElapsedTime = (stoptime - starttime) * .001;

		//display the result in a message box
		printf("Using 3DNow!, performed 4x4 matrix multiplication %g", ElapsedTime);
		printf("%s", " seconds.\n");
		printf("%s%g%s", "c[2][3] = ", c[2][3], "\n\n");
		
	}
		
	else {

		printf("This processor does not support 3dNow! Instructions\n");
			
	}
		
}//end fnB()

// perform dot product in ASSEMBLY using SSE SIMD instructions
void fnC()
{

	//general local declarations
	//integers
	int counter;		//iteration counter
	int bSSEOK;			//cpu supports SSE instructions

	//floats
	float a[4][4];			//matrix a
	float b[4][4];			//matrix b
	float c[4][4];			//matrix c = a x b
	
	//variables for time execution time
	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at loop exit
	double ElapsedTime;		//system ticks during loop

	//initialize SSE flag to zero
	bSSEOK = 0;

	//test for SSE capability in cpu
	_asm {

			push	eax
			push	edx
			xor		eax, eax		; CPUid function Vendor ID
			CPUid
			test	eax, eax
			jz		$NoSSE		
			mov		eax, 00000001h	; CPUid function Largest Ext. Val
			CPUid
			test	eax, 02000000h
			jl		$YesSSE
			jmp		$NoSSE
$YesSSE:	mov		bSSEOK, 1		; bSSEOK = true
			jmp		$EndCpuTest
$NoSSE:		mov		bSSEOK, 0		; bSSEOK = false
$EndCpuTest:		pop		edx
			pop		eax

	}

	if(bSSEOK){

		//initialize the two matrices with some made up values
		a[0][0] = (float)3.654;
		a[0][1] = (float)8.234;
		a[0][2] = (float)3.874;
		a[0][3] = (float)5.792;
		a[1][0] = (float)2.234;
		a[1][1] = (float)8.201;
		a[1][2] = (float)8.333;
		a[1][3] = (float)0.677;
		a[2][0] = (float)5.555;
		a[2][1] = (float)6.633;
		a[2][2] = (float)4.298;
		a[2][3] = (float)9.002;
		a[3][0] = (float)4.094;
		a[3][1] = (float)6.773;
		a[3][2] = (float)1.202;
		a[3][3] = (float)2.118;

		//matrix b in row major ('regular') order
		b[0][0] = (float)3.654;
		b[0][1] = (float)2.348;
		b[0][2] = (float)8.743;
		b[0][3] = (float)7.925;
		b[1][0] = (float)2.342;
		b[1][1] = (float)2.018;
		b[1][2] = (float)3.338;
		b[1][3] = (float)6.770;
		b[2][0] = (float)5.555;
		b[2][1] = (float)6.336;
		b[2][2] = (float)2.984;
		b[2][3] = (float)0.029;
		b[3][0] = (float)0.944;
		b[3][1] = (float)7.736;
		b[3][2] = (float)2.021;
		b[3][3] = (float)1.182;	

		//start the timer
		starttime = GetTickCount(); 	

		for(counter=1; counter <= NumIts; ++counter){
    
			//SSE 4x4 matrix multiply.
			
// adapted from	http://www.cortstratton.org/content/tutorials/HugiCode.html
			
			_asm {
			
				// load B matrix into SIMD registers
				movaps	  xmm4, b[0]
				movaps	  xmm5, b[16]
				movaps	  xmm6, b[32]
				movaps	  xmm7, b[48]	
			
				//iterate through A matrix as row vectors
				//using unrolled loop
					   
				// load first row vector into reg 0-3
				// and broadcast
				movaps   xmm0, a[0]
				movaps   xmm1, xmm0
				movaps   xmm2, xmm0
				movaps   xmm3, xmm0
				shufps   xmm0, xmm0, 0x00 
				shufps   xmm1, xmm1, 0x55 
				shufps   xmm2, xmm2, 0xaa 
				shufps   xmm3, xmm3, 0xff
				
				//multiply the elements
				mulps    xmm0, xmm4
				mulps    xmm1, xmm5
				mulps    xmm2, xmm6
				mulps    xmm3, xmm7
				
				//add the products and store result
				addps    xmm1, xmm0
				addps    xmm2, xmm3
				addps    xmm1, xmm2
				movaps   c[0], xmm1

				// load second row vector into reg 0-3
				// and broadcast
				movaps   xmm0, a[16]
				movaps   xmm1, xmm0
				movaps   xmm2, xmm0
				movaps   xmm3, xmm0
				shufps   xmm0, xmm0, 0x00 
				shufps   xmm1, xmm1, 0x55 
				shufps   xmm2, xmm2, 0xaa 
				shufps   xmm3, xmm3, 0xff
				
				//multiply the elements
				mulps    xmm0, xmm4
				mulps    xmm1, xmm5
				mulps    xmm2, xmm6
				mulps    xmm3, xmm7
				
				//add the products and store result
				addps    xmm1, xmm0
				addps    xmm2, xmm3
				addps    xmm1, xmm2
				movaps   c[16], xmm1

				// load third row vector into reg 0-3
				// and broadcast
				movaps   xmm0, a[32]
				movaps   xmm1, xmm0
				movaps   xmm2, xmm0
				movaps   xmm3, xmm0
				shufps   xmm0, xmm0, 0x00 
				shufps   xmm1, xmm1, 0x55 
				shufps   xmm2, xmm2, 0xaa 
				shufps   xmm3, xmm3, 0xff
				
				//multiply the elements
				mulps    xmm0, xmm4
				mulps    xmm1, xmm5
				mulps    xmm2, xmm6
				mulps    xmm3, xmm7
				
				//add the products and store result
				addps    xmm1, xmm0
				addps    xmm2, xmm3
				addps    xmm1, xmm2
				movaps   c[32], xmm1

				// load fourth row vector into reg 0-3
				// and broadcast
				movaps   xmm0, a[48]
				movaps   xmm1, xmm0
				movaps   xmm2, xmm0
				movaps   xmm3, xmm0
				shufps   xmm0, xmm0, 0x00 
				shufps   xmm1, xmm1, 0x55 
				shufps   xmm2, xmm2, 0xaa 
				shufps   xmm3, xmm3, 0xff
				
				//multiply the elements
				mulps    xmm0, xmm4
				mulps    xmm1, xmm5
				mulps    xmm2, xmm6
				mulps    xmm3, xmm7
				
				//add the products and store result
				addps    xmm1, xmm0
				addps    xmm2, xmm3
				addps    xmm1, xmm2
				movaps   c[48], xmm1
					
			   }
        
    		}
		//stop the timer
		stoptime = GetTickCount();		
	
		//compute the elapsed time, in seconds
		ElapsedTime = (stoptime - starttime) * .001;

		//display the result in a message box
		printf("Using SSE, performed matrix mult. in %g", ElapsedTime);
		printf("%s", " seconds.\n");
		printf("%s%g%s", "c[2][3] = ", c[2][3], "\n\n");

		
	}
		
	else {

		printf("This processor does not support SSE Instructions\n");
			
	}
	
	
}//end fnC()


//entry function
int main()
{
	//local declarations
	//integers
	int bTERM;		//menu flag

	//character strings
	char inchar;		//input character
	

	//initialize bTERM to NOT TERMINATE
	bTERM = 0;
	
	//clear terminal
	system("cls");
	
	do {
		
		//present menu to user
		printf("\nEnter the Number of the routine to run, and press <Enter> \n");
		printf("1\t4x4 Matrix Multiplication using x87 instructions\n");
		printf("2\t4x4 Matrix Multiplication using 3dNow! SIMD Instructions\n");
		printf("3\t4x4 Matrix Multiplication using SSE SIMD Instructions\n");
		printf("q\tEnd this program\n");
	
		//get user input
		scanf("%s", &inchar);

		//call the user selected function
		switch	(inchar){

			case '1': fnA();
				break;
	
			case '2': fnB();
				break;

			case '3': fnC();
				break;
		
			//select the terminate function	
			case 'q': printf("Ending This Program\n");
				
				//set flag to terminate
				bTERM = 1;
				break;

		}
		
	} while(!bTERM);

	return 0;

} // End Main

