// OS02Dlg.cpp : implementation file
//

#include "stdafx.h"
#include "OS02.h"
#include "OS02Dlg.h"
#include "windows.h"
#include "windowsx.h"
#include "stdio.h"
#include "stdlib.h"

//Defines
#define NumIts 100000000		//number of iterations
#define sigma  (float)258		//lennard jones sigma parameter
#define depth  (float)10.22		//lennard jones depth parameter


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// COS02Dlg dialog

COS02Dlg::COS02Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(COS02Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(COS02Dlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void COS02Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COS02Dlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(COS02Dlg, CDialog)
	//{{AFX_MSG_MAP(COS02Dlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON1, OnButton1)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COS02Dlg message handlers

BOOL COS02Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void COS02Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR COS02Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}


//working function for the program. Note this contains essentially 
//what is in main() in the console app
void COS02Dlg::OnButton1() 
{
	//local declarations
	//integers
	int i;		//iteration counter
	int j;		//output string pointer
	int xk;
	int rnum;	//next random number in sequence

	//floats
	float r;			//distance between particles
	float r_6;			//distance^6
	float r_12;			//distance^12
	float sigma_6;		//sigma^6
	float sigma_12;		//sigma^12
	float energy;		//computed energy

	//doubles
	double starttime;	//ticks at start of loop
	double stoptime;	//ticks at end of loop
	double elapsedtime;	//seconds during loop

	//character strings
	char buffer[200];		//output string printer
	
	//seed the random number generator
	xk = 200;

	//start the timer
	starttime = GetTickCount();
	
	//compute sigma^6, per Chapter 8
	sigma_6 = sigma * sigma * sigma * sigma * sigma * sigma;

	//compute sigma^12, per Chapter 8
	sigma_12 = sigma_6 * sigma_6;

	//start iteration
	for(i=0; i<NumIts; ++i){

		//get a random r between 0 and 1, per Chapter 6
		//using a= 25 and m= 16777216
		rnum = (25 * xk) % 16777216;
		r = (float)rnum / (float)16777216;

		//increment 'last' random number
		xk = rnum;

		//get r between 1 and 7
		r *= 7;
		r += 1;

		//compute r^6, per Chapter 8
		r_6 = r * r * r * r * r * r;
	
		//compute r^12, per Chapter 8
		r_12 = r_6 * r_6;

		//compute energy
		energy = sigma_12/r_12 - sigma_6/r_6;
		energy *= depth;

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	elapsedtime = (stoptime - starttime) * 0.001;

	//build output string and display result
	j = sprintf(buffer,"Elapsed time for MFC Windows App: %g",elapsedtime);
	j += sprintf(buffer+j, "%s", "seconds.\n\n");

	MessageBox(buffer, "Done with Iterative Pair Energy Calculation",
				MB_OK | MB_ICONEXCLAMATION);
	
}
