// OS03.cpp : Defines the entry point for the application.
//

#define WIN32_LEAN_AND_MEAN

#include "stdafx.h"
#include "windows.h"
#include "windowsx.h"
#include "stdio.h"
#include "stdlib.h"

//Defines
#define NumIts 100000000		//number of iterations
#define sigma  (float)258		//lennard jones sigma parameter
#define depth  (float)10.22		//lennard jones depth parameter


int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	//local declarations
	//integers
	int i;		//iteration counter
	int j;		//output string pointer
	int rnum;	//next random number
	int xk;		//last random number

	//floats
	float r;			//distance between particles
	float r_6;			//distance^6
	float r_12;			//distance^12
	float sigma_6;		//sigma^6
	float sigma_12;		//sigma^12
	float energy;		//computed energy

	//doubles
	double starttime;	//ticks at start of loop
	double stoptime;	//ticks at end of loop
	double elapsedtime;	//seconds during loop

	//character strings
	char buffer[200];		//output string printer
	
	//seed the random number generator
	xk = 200;

	//start the timer
	starttime = GetTickCount();
	
	//compute sigma^6, per Chapter 8
	sigma_6 = sigma * sigma * sigma * sigma * sigma * sigma;

	//compute sigma^12, per Chapter 8
	sigma_12 = sigma_6 * sigma_6;

	//start iteration
	for(i=0; i<NumIts; ++i){

		//get a random r between 0 and 1, per Chapter 6
		//using a= 25 and m= 16777216
		rnum = (25 * xk) % 16777216;
		r = (float)rnum / (float)16777216;

		//increment 'last' random number
		xk = rnum;

		//get r between 0 and 7
		r = r * 7 + 1;

		//compute r^6, per Chapter 8
		r_6 = r * r * r * r * r * r;
	
		//compute r^12, per Chapter 8
		r_12 = r_6 * r_6;

		//compute energy
		energy = sigma_12/r_12 - sigma_6/r_6;
		energy *= depth;

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	elapsedtime = (stoptime - starttime) * 0.001;

	//build output string and display result
	j = sprintf(buffer,"Elapsed time for Windows App (No MFC): %g",elapsedtime);
	j += sprintf(buffer+j, "%s", "seconds.\n\n");

	MessageBox(NULL, buffer, "Done with Iterative Pair Energy Calculation",
				MB_OK | MB_ICONEXCLAMATION);

	return 0;
}



