// Data07Dlg.cpp : implementation file
//
// This program demonstrates the use of caching structure (or class)
// member data in simple variables for repeated access, vs accessing
// such data directly.  The class is declared globally.
// Some simple math is done for demo purposes.
//
// The function CData07Dlg::OnButton1 is the direct member access.
// The function CData07Dlg::OnButton2 uses simple cache variables.

#include "stdafx.h"
#include "Data07.h"
#include "Data07Dlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//global declarations
const int cNumIts = 20000000;	//number of iterations

//define the class
class CAStructure
{

public:

	float x;
	float y;
	float z;

};

//instantiate the class
CAStructure	ThisStructure;


/////////////////////////////////////////////////////////////////////////////
// CData07Dlg dialog

CData07Dlg::CData07Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CData07Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CData07Dlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CData07Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CData07Dlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CData07Dlg, CDialog)
	//{{AFX_MSG_MAP(CData07Dlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON1, OnButton1)
	ON_BN_CLICKED(IDC_BUTTON2, OnButton2)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CData07Dlg message handlers

BOOL CData07Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CData07Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CData07Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

//do some simple math with structure.member access
void CData07Dlg::OnButton1() 
{
	//local declarations
	//integers
	int j;						//output string pointer
	int counter;				//iteration counter

	//float
	float a;					//results for some simple math
	float b;
	float c;

	//doubles
	double starttime;			//ticks at start of loop
	double stoptime;			//ticks at loop exit
	double ElapsedTime;			//ticks during loop

	//character strings
	char buffer[200];			//output string	

	//initialize structure values
	ThisStructure.x = (float)123.4;
	ThisStructure.y = (float)16.453;
	ThisStructure.z = (float)-0.00345;


	//start the timer
    starttime = GetTickCount();

	for(counter=1; counter <=cNumIts; ++counter){
        
		//do some floating point math to force memory references
        a = ThisStructure.x + ThisStructure.y + ThisStructure.z;
        b = ThisStructure.x * ThisStructure.y * ThisStructure.z;
        c = ThisStructure.x * ThisStructure.x + ThisStructure.y * 
            ThisStructure.y + ThisStructure.z * ThisStructure.z;
    
    }
    
	//stop the timer
    stoptime = GetTickCount();
    
	//compute elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//build output string and display result in message box
	j = sprintf(buffer,"%s%d%s%g", "Computation ", cNumIts, " times with Member Access in ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds.\n");

	MessageBox(buffer,
					 "Computed with Direct Member Access",
					 MB_OK | MB_ICONEXCLAMATION);
	
}

//do some simple math using cached simple variables to access data
void CData07Dlg::OnButton2() 
{
	//local declarations
	//integers
	int j;						//output string pointer
	int counter;				//iteration counter

	//float
	float a;					//results for some simple math
	float b;
	float c;
	float xcache;					//the cache variables
	float ycache;
	float zcache;

	//doubles
	double starttime;			//ticks at start of loop
	double stoptime;			//ticks at loop exit
	double ElapsedTime;			//ticks during loop

	//character strings
	char buffer[200];			//output string

	//initialize structure values
	ThisStructure.x = (float)123.4;
	ThisStructure.y = (float)16.453;
	ThisStructure.z = (float)-0.00345;
	
	//start the timer
    starttime = GetTickCount();

	//cache the member data in simple variables
	xcache = ThisStructure.x;
	ycache = ThisStructure.y;
	zcache = ThisStructure.z;

	for(counter=1; counter <=cNumIts; ++counter){
        
		//do some floating point math to force memory references
        a = xcache + ycache + zcache;
        b = xcache * ycache * zcache;
        c = xcache * xcache + ycache * ycache + zcache * zcache;
    
    }
    
	//stop the timer
    stoptime = GetTickCount();
    
	//compute elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//build output string and display result in message box
	j = sprintf(buffer,"%s%d%s%g", "Computation ", cNumIts, " times with Cache Variable Access in ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds.\n");

	MessageBox(buffer,
					 "Computed with Cached Variable Access",
					 MB_OK | MB_ICONEXCLAMATION);	
}

