// Loop03Dlg.cpp : implementation file
//
// This program demonstrates the unrolled loops for n x n matrix 
// multiplication.  However, using the unrolled loops necessitates
// the use of a decision structure when variable values for n are 
// possible in a routine, whereas the generic, rolled version does
// not require a decision since the dimension can be sent as a parameter.
//
// Therefore, in this demo, the program is performing 4x4 and 20x20
// matrix multiplications in random order using a variety of rolled
// algorithms compared to the unrolled one.
//
// A random number array is initialized in CLoop03Dlg::OnButton1().
//
// The rolled, generic routine matrix_nxn(int Dimen) is called without
// decision structures in CLoop03Dlg::OnButton5().
//
// The rolled, generic routine matrix_nxn(int Dimen) is called with
// decision structures in CLoop03Dlg::OnButton2().

#include "stdafx.h"
#include "Loop03.h"
#include "Loop03Dlg.h"
#include "math.h"
#include "stdio.h"
#include <stdlib.h>


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// global declarations
// constants
static int cNumIts = 500000;		//number of iterations

//integers
int r[500001];		//random 4's and 8's

// floats
float a[21][21];		//Matrix A
float b[21][21];		//Matrix B
float c[21][21];		//Result Matrix C

//functions
BOOL EnableWindow(BOOL bEnable = TRUE);
int matrix_nxn(int Dimen);
int matrix_4x4_rolled();
int matrix_8x8_rolled();
int matrix_4x4_unrolled();
int matrix_8x8_unrolled();


/////////////////////////////////////////////////////////////////////////////
// CLoop03Dlg dialog

CLoop03Dlg::CLoop03Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CLoop03Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CLoop03Dlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CLoop03Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLoop03Dlg)
	DDX_Control(pDX, IDC_BUTTON5, m_button5);
	DDX_Control(pDX, IDC_BUTTON4, m_button4);
	DDX_Control(pDX, IDC_BUTTON3, m_button3);
	DDX_Control(pDX, IDC_BUTTON2, m_button2);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CLoop03Dlg, CDialog)
	//{{AFX_MSG_MAP(CLoop03Dlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON1, OnButton1)
	ON_BN_CLICKED(IDC_BUTTON2, OnButton2)
	ON_BN_CLICKED(IDC_BUTTON5, OnButton5)
	ON_BN_CLICKED(IDC_BUTTON3, OnButton3)
	ON_BN_CLICKED(IDC_BUTTON4, OnButton4)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLoop03Dlg message handlers

BOOL CLoop03Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CLoop03Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CLoop03Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CLoop03Dlg::OnButton1() 
{

	//local declarations
	//integers
	int rcounter;				//counter used to initialize random array
	int rowcounter;
	int colcounter;
	int randomval;


	//initialize the random number array
	for(rcounter = 1; rcounter <= cNumIts; ++rcounter){

		//rand() returns an integer < about 32000
		randomval = rand();
		
		//want about half being 20, half being 4
		if(randomval < 16000) r[rcounter] = 4;
		else r[rcounter] = 8;
		
	}


	//initialize the A and B matrices to random values
	for(rowcounter=1; rowcounter <= 8; ++rowcounter){

		for(colcounter=1; colcounter <= 8; ++colcounter){

			a[rowcounter][colcounter] = (float)rand()/RAND_MAX;
			b[rowcounter][colcounter] = (float)rand()/RAND_MAX;

		}

	}

	m_button2.EnableWindow();  // Enable button controls;
	m_button3.EnableWindow();
	m_button4.EnableWindow();
	m_button5.EnableWindow();	

}

//generic nxn matrix multiplication with decision structure
void CLoop03Dlg::OnButton2() 
{

	//local declarations
	//integers
	int counter;		//iteration counter
	int dummy;			//holder for return of function call
	int j;				//character pointer

	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at end of loop
	double ElapsedTime;		//system ticks during loop

	//character strings
	char buffer[200];		//output string

	starttime = GetTickCount();		//start the timer
	
	//call the implicit function with decision structure
	for(counter = 1; counter <= cNumIts; ++counter){

		//decision to determine which dimension to send
		if(r[counter]=8) dummy = matrix_nxn(8);
		else dummy = matrix_nxn(4);

	}

	stoptime = GetTickCount();		//stop the timer
    
	//compute the elapsed time, in seconds
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result in a message box
	j=sprintf(buffer, "Called Implicit nxn function with decisions 500,000 times %g", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.\n");

	MessageBox(buffer, "Done with Implicit Call with Decisions",
				MB_OK | MB_ICONEXCLAMATION);
	


}

	
// generic nxn matrix multiplication call without decision structures
void CLoop03Dlg::OnButton5() 
{

	//local declarations
	//integers
	int counter;		//iteration counter
	int dummy;			//holder for return of function call
	int j;				//character pointer

	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at end of loop
	double ElapsedTime;		//system ticks during loop

	//character strings
	char buffer[200];		//output string

	starttime = GetTickCount();		//start the timer
	
	//call the implicit function without decision structure
	for(counter = 1; counter <= cNumIts; ++counter){

		//no decision need be made, since r array holds the 
		//dimension itself
		dummy = matrix_nxn(r[counter]);

	}

	stoptime = GetTickCount();		//stop the timer
    
	//compute the elapsed time, in seconds
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result in a message box
	j=sprintf(buffer, "Called Implicit nxn function without decisions 500,000 times %g", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.\n");

	MessageBox(buffer, "Done with Implicit Call w/o Decisions",
				MB_OK | MB_ICONEXCLAMATION);
	
	

}


//this function calls the EXPLICIT ROLLED-UP matrix multiplications
//since the calls involve explict dimensions, some form of decision
//must be made
void CLoop03Dlg::OnButton3() 
{
	
	//local declarations
	//integers
	int counter;		//iteration counter
	int dummy;			//holder for return of function call
	int j;				//character pointer

	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at end of loop
	double ElapsedTime;		//system ticks during loop

	//character strings
	char buffer[200];		//output string

	starttime = GetTickCount();		//start the timer
	
	//call the explicit functions without decision structure
	for(counter = 1; counter <= cNumIts; ++counter){

		//decision to determine which dimension to send
		if(r[counter]=8) dummy = matrix_8x8_rolled();
		else dummy = matrix_4x4_rolled();

	}

	stoptime = GetTickCount();		//stop the timer
    
	//compute the elapsed time, in seconds
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result in a message box
	j=sprintf(buffer, "Called Explicit rolled functions with decisions 500,000 times %g", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.\n");

	MessageBox(buffer, "Done with Explicit Rolled Calls with Decisions",
				MB_OK | MB_ICONEXCLAMATION);
	
	
}


//function to repeatedly call the unrolled matrix multiplcation routines
void CLoop03Dlg::OnButton4() 
{

	//local declarations
	//integers
	int counter;		//iteration counter
	int dummy;			//holder for return of function call
	int j;				//character pointer

	//doubles
	double starttime;		//system ticks at start of loop
	double stoptime;		//system ticks at end of loop
	double ElapsedTime;		//system ticks during loop

	//character strings
	char buffer[200];		//output string

	starttime = GetTickCount();		//start the timer
	
	//call the explicit unrolled functions with decision structure
	for(counter = 1; counter <= cNumIts; ++counter){

		//decision to determine which dimension to send
		if(r[counter]=8) dummy = matrix_8x8_unrolled();
		else dummy = matrix_4x4_unrolled();

	}

	stoptime = GetTickCount();		//stop the timer
    
	//compute the elapsed time, in seconds
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result in a message box
	j=sprintf(buffer, "Called Explicit Unrolled functions with decisions 500,000 times %g", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.\n");

	MessageBox(buffer, "Done with Explicit Unrolled Calls with Decisions",
				MB_OK | MB_ICONEXCLAMATION);
	

}


// Generic nxn matrix multiplication; dimension n is passed as integer
// the actual matrices are globals; actual function in application
// would better be coded more general with pointers.
int matrix_nxn(int Dimen)
{

	//local declarations
	int i;			//column counter
	int j;			//row counter
	int k;			//internal counter for matrix multiplication

	for(i=1; i<=Dimen; ++i){

		for(j=1; j<=Dimen; ++j){

			c[i][j] = 0;

			for(k=1; k<=Dimen; ++k){

				c[i][j] += a[i][k] * b[k][j];

			}

		}

	}

	return(0);

}

//Explicit 4x4 Rolled-Up matrix multiplication.  The dimension is explicit,
//so does not need to be passed.  The matrices are globals.  
int matrix_4x4_rolled()
{

	//local declarations
	int i;
	int j;
	int k;

	for(i=1; i <= 4; ++i){

		for(j=1; j <= 4; ++j){

			c[i][j] = 0;

			for(k=1; k <= 4; ++k){

				c[i][j] += a[i][k] * b[k][j];

			}

		}

	}

	return(0);

}


//Explicit 20x20 Rolled-Up matrix multiplication.  The dimension is explicit,
//so does not need to be passed.  The matrices are globals.  
int matrix_8x8_rolled()
{

	//local declarations
	int i;
	int j;
	int k;

	for(i=1; i <= 8; ++i){

		for(j=1; j <= 8; ++j){

			c[i][j] = 0;

			for(k=1; k <= 8; ++k){

				c[i][j] += a[i][k] * b[k][j];

			}

		}

	}

	return(0);

}

//Explicit, Unrolled 4x4 matrix multiplication
int matrix_4x4_unrolled()
{

	//j = 1
    c[1][ 1] = a[1][ 1] * b[1][ 1] + a[1][ 2] * b[2][ 1] + a[1][ 3] * b[3][ 1] + a[1][ 4] * b[4][ 1];
    c[1][ 2] = a[1][ 1] * b[1][ 2] + a[1][ 2] * b[2][ 2] + a[1][ 3] * b[3][ 2] + a[1][ 4] * b[4][ 2];
    c[1][ 3] = a[1][ 1] * b[1][ 3] + a[1][ 2] * b[2][ 3] + a[1][ 3] * b[3][ 3] + a[1][ 4] * b[4][ 3];
    c[1][ 4] = a[1][ 1] * b[1][ 4] + a[1][ 2] * b[2][ 4] + a[1][ 3] * b[3][ 4] + a[1][ 4] * b[4][ 4];
    
    //j = 2
    c[2][ 1] = a[2][ 1] * b[1][ 1] + a[2][ 2] * b[2][ 1] + a[2][ 3] * b[3][ 1] + a[2][ 4] * b[4][ 1];
    c[2][ 2] = a[2][ 1] * b[1][ 2] + a[2][ 2] * b[2][ 2] + a[2][ 3] * b[3][ 2] + a[2][ 4] * b[4][ 2];
    c[2][ 3] = a[2][ 1] * b[1][ 3] + a[2][ 2] * b[2][ 3] + a[2][ 3] * b[3][ 3] + a[2][ 4] * b[4][ 3];
    c[2][ 4] = a[2][ 1] * b[1][ 4] + a[2][ 2] * b[2][ 4] + a[2][ 3] * b[3][ 4] + a[2][ 4] * b[4][ 4];
    
    //j=3
    c[3][ 1] = a[3][ 1] * b[1][ 1] + a[3][ 2] * b[2][ 1] + a[3][ 3] * b[3][ 1] + a[3][ 4] * b[4][ 1];
    c[3][ 2] = a[3][ 1] * b[1][ 2] + a[3][ 2] * b[2][ 2] + a[3][ 3] * b[3][ 2] + a[3][ 4] * b[4][ 2];
    c[3][ 3] = a[3][ 1] * b[1][ 3] + a[3][ 2] * b[2][ 3] + a[3][ 3] * b[3][ 3] + a[3][ 4] * b[4][ 3];
    c[3][ 4] = a[3][ 1] * b[1][ 4] + a[3][ 2] * b[2][ 4] + a[3][ 3] * b[3][ 4] + a[3][ 4] * b[4][ 4];
    
    //j=4
    c[4][ 1] = a[4][ 1] * b[1][ 1] + a[4][ 2] * b[2][ 1] + a[4][ 3] * b[3][ 1] + a[4][ 4] * b[4][ 1];
    c[4][ 2] = a[4][ 1] * b[1][ 2] + a[4][ 2] * b[2][ 2] + a[4][ 3] * b[3][ 2] + a[4][ 4] * b[4][ 2];
    c[4][ 3] = a[4][ 1] * b[1][ 3] + a[4][ 2] * b[2][ 3] + a[4][ 3] * b[3][ 3] + a[4][ 4] * b[4][ 3];
    c[4][ 4] = a[4][ 1] * b[1][ 4] + a[4][ 2] * b[2][ 4] + a[4][ 3] * b[3][ 4] + a[4][ 4] * b[4][ 4];
    

	return(0);

}


//8x8 unrolled matrix multiplication
int matrix_8x8_unrolled()
{

	//i=1
    c[1][ 1] = a[1][1] * b[1][1] + a[1][2] * b[2][1] + a[1][3] * b[3][1] + a[1][4] * b[4][1]  
	 +	 a[1][ 5] * b[5][ 1] + a[1][ 6] * b[6][ 1] + a[1][ 7] * b[7][ 1] + a[1][ 8] * b[8][ 1];
    
    c[1][ 2] = a[1][ 1] * b[1][ 2] + a[1][ 2] * b[2][ 2] + a[1][ 3] * b[3][ 2] + a[1][ 4] * b[4][ 2]
     +   a[1][ 5] * b[5][ 2] + a[1][ 6] * b[6][ 2] + a[1][ 7] * b[7][ 2] + a[1][ 8] * b[8][ 2];
    
    c[1][ 3] = a[1][ 1] * b[1][ 3] + a[1][ 2] * b[2][ 3] + a[1][ 3] * b[3][ 3] + a[1][ 4] * b[4][ 3]
     +   a[1][ 5] * b[5][ 3] + a[1][ 6] * b[6][ 3] + a[1][ 7] * b[7][ 3] + a[1][ 8] * b[8][ 3];
    
    c[1][ 4] = a[1][ 1] * b[1][ 4] + a[1][ 2] * b[2][ 4] + a[1][ 3] * b[3][ 4] + a[1][ 4] * b[4][ 4]
     +  a[1][ 4] * b[5][ 4] + a[1][ 6] * b[6][ 4] + a[1][ 7] * b[7][ 4] + a[1][ 8] * b[8][ 4];
    
    c[1][ 5] = a[1][ 1] * b[1][ 5] + a[1][ 2] * b[2][ 5] + a[1][ 3] * b[3][ 5] + a[1][ 4] * b[4][ 5]
     +   a[1][ 5] * b[5][ 5] + a[1][ 6] * b[6][ 5] + a[1][ 7] * b[7][ 5] + a[1][ 8] * b[8][ 5];
    
    c[1][ 6] = a[1][ 1] * b[1][ 6] + a[1][ 2] * b[2][ 6] + a[1][ 3] * b[3][ 6] + a[1][ 4] * b[4][ 6]
     +   a[1][ 5] * b[5][ 6] + a[1][ 6] * b[6][ 6] + a[1][ 7] * b[7][ 6] + a[1][ 8] * b[8][ 6];
    
    c[1][ 7] = a[1][ 1] * b[1][ 7] + a[1][ 2] * b[2][ 7] + a[1][ 3] * b[3][ 7] + a[1][ 4] * b[4][ 7]
     +   a[1][ 5] * b[5][ 7] + a[1][ 6] * b[6][ 7] + a[1][ 7] * b[7][ 7] + a[1][ 8] * b[8][ 7];
    
    c[1][ 8] = a[1][ 1] * b[1][ 8] + a[1][ 2] * b[2][ 8] + a[1][ 3] * b[3][ 8] + a[1][ 4] * b[4][ 8]
     +   a[1][ 5] * b[5][ 8] + a[1][ 6] * b[6][ 8] + a[1][ 7] * b[7][ 8] + a[1][ 8] * b[8][ 8];
    
    //i=2
    
    c[2][ 1] = a[2][ 1] * b[1][ 1] + a[2][ 2] * b[2][ 1] + a[2][ 3] * b[3][ 1] + a[2][ 4] * b[4][ 1]
     +   a[2][ 5] * b[5][ 1] + a[2][ 6] * b[6][ 1] + a[2][ 7] * b[7][ 1] + a[2][ 8] * b[8][ 1];
    
    c[2][ 2] = a[2][ 1] * b[1][ 2] + a[2][ 2] * b[2][ 2] + a[2][ 3] * b[3][ 2] + a[2][ 4] * b[4][ 2]
     +   a[2][ 5] * b[5][ 2] + a[2][ 6] * b[6][ 2] + a[2][ 7] * b[7][ 2] + a[2][ 8] * b[8][ 2];
    
    c[2][ 3] = a[2][ 1] * b[1][ 3] + a[2][ 2] * b[2][ 3] + a[2][ 3] * b[3][ 3] + a[2][ 4] * b[4][ 3]
     +   a[2][ 5] * b[5][ 3] + a[2][ 6] * b[6][ 3] + a[2][ 7] * b[7][ 3] + a[2][ 8] * b[8][ 3];
    
    c[2][ 4] = a[2][ 1] * b[1][ 4] + a[2][ 2] * b[2][ 4] + a[2][ 3] * b[3][ 4] + a[2][ 4] * b[4][ 4]
     +   a[2][ 4] * b[5][ 4] + a[2][ 6] * b[6][ 4] + a[2][ 7] * b[7][ 4] + a[2][ 8] * b[8][ 4];
    
    c[2][ 5] = a[2][ 1] * b[1][ 5] + a[2][ 2] * b[2][ 5] + a[2][ 3] * b[3][ 5] + a[2][ 4] * b[4][ 5]
     +   a[2][ 5] * b[5][ 5] + a[2][ 6] * b[6][ 5] + a[2][ 7] * b[7][ 5] + a[2][ 8] * b[8][ 5];
    
    c[2][ 6] = a[2][ 1] * b[1][ 6] + a[2][ 2] * b[2][ 6] + a[2][ 3] * b[3][ 6] + a[2][ 4] * b[4][ 6]
     +   a[2][ 5] * b[5][ 6] + a[2][ 6] * b[6][ 6] + a[2][ 7] * b[7][ 6] + a[2][ 8] * b[8][ 6];
    
    c[2][ 7] = a[2][ 1] * b[1][ 7] + a[2][ 2] * b[2][ 7] + a[2][ 3] * b[3][ 7] + a[2][ 4] * b[4][ 7]
     +   a[2][ 5] * b[5][ 7] + a[2][ 6] * b[6][ 7] + a[2][ 7] * b[7][ 7] + a[2][ 8] * b[8][ 7];
    
    c[2][ 8] = a[2][ 1] * b[1][ 8] + a[2][ 2] * b[2][ 8] + a[2][ 3] * b[3][ 8] + a[2][ 4] * b[4][ 8]
     +   a[2][ 5] * b[5][ 8] + a[2][ 6] * b[6][ 8] + a[2][ 7] * b[7][ 8] + a[2][ 8] * b[8][ 8];

    //i=3
    
    c[3][ 1] = a[3][ 1] * b[1][ 1] + a[3][ 2] * b[2][ 1] + a[3][ 3] * b[3][ 1] + a[3][ 4] * b[4][ 1]
     +   a[3][ 5] * b[5][ 1] + a[3][ 6] * b[6][ 1] + a[3][ 7] * b[7][ 1] + a[3][ 8] * b[8][ 1];

    c[3][ 2] = a[3][ 1] * b[1][ 2] + a[3][ 2] * b[2][ 2] + a[3][ 3] * b[3][ 2] + a[3][ 4] * b[4][ 2]
     +   a[3][ 5] * b[5][ 2] + a[3][ 6] * b[6][ 2] + a[3][ 7] * b[7][ 2] + a[3][ 8] * b[8][ 2];
    
    c[3][ 3] = a[3][ 1] * b[1][ 3] + a[3][ 2] * b[2][ 3] + a[3][ 3] * b[3][ 3] + a[3][ 4] * b[4][ 3]
     +   a[3][ 5] * b[5][ 3] + a[3][ 6] * b[6][ 3] + a[3][ 7] * b[7][ 3] + a[3][ 8] * b[8][ 3];
    
    c[3][ 4] = a[3][ 1] * b[1][ 4] + a[3][ 2] * b[2][ 4] + a[3][ 3] * b[3][ 4] + a[3][ 4] * b[4][ 4]
     +   a[3][ 4] * b[5][ 4] + a[3][ 6] * b[6][ 4] + a[3][ 7] * b[7][ 4] + a[3][ 8] * b[8][ 4];
    
    c[3][ 5] = a[3][ 1] * b[1][ 5] + a[3][ 2] * b[2][ 5] + a[3][ 3] * b[3][ 5] + a[3][ 4] * b[4][ 5]
     +   a[3][ 5] * b[5][ 5] + a[3][ 6] * b[6][ 5] + a[3][ 7] * b[7][ 5] + a[3][ 8] * b[8][ 5];
    
    c[3][ 6] = a[3][ 1] * b[1][ 6] + a[3][ 2] * b[2][ 6] + a[3][ 3] * b[3][ 6] + a[3][ 4] * b[4][ 6]
     +   a[3][ 5] * b[5][ 6] + a[3][ 6] * b[6][ 6] + a[3][ 7] * b[7][ 6] + a[3][ 8] * b[8][ 6];
    
    c[3][ 7] = a[3][ 1] * b[1][ 7] + a[3][ 2] * b[2][ 7] + a[3][ 3] * b[3][ 7] + a[3][ 4] * b[4][ 7]
     +   a[3][ 5] * b[5][ 7] + a[3][ 6] * b[6][ 7] + a[3][ 7] * b[7][ 7] + a[3][ 8] * b[8][ 7];
    
    c[3][ 8] = a[3][ 1] * b[1][ 8] + a[3][ 2] * b[2][ 8] + a[3][ 3] * b[3][ 8] + a[3][ 4] * b[4][ 8]
     +   a[3][ 5] * b[5][ 8] + a[3][ 6] * b[6][ 8] + a[3][ 7] * b[7][ 8] + a[3][ 8] * b[8][ 8];

    //i=4
    
    c[4][ 1] = a[4][ 1] * b[1][ 1] + a[4][ 2] * b[2][ 1] + a[4][ 3] * b[3][ 1] + a[4][ 4] * b[4][ 1]
     +   a[4][ 5] * b[5][ 1] + a[4][ 6] * b[6][ 1] + a[4][ 7] * b[7][ 1] + a[4][ 8] * b[8][ 1];
    
    c[4][ 2] = a[4][ 1] * b[1][ 2] + a[4][ 2] * b[2][ 2] + a[4][ 3] * b[3][ 2] + a[4][ 4] * b[4][ 2]
     +   a[4][ 5] * b[5][ 2] + a[4][ 6] * b[6][ 2] + a[4][ 7] * b[7][ 2] + a[4][ 8] * b[8][ 2];
    
    c[4][ 3] = a[4][ 1] * b[1][ 3] + a[4][ 2] * b[2][ 3] + a[4][ 3] * b[3][ 3] + a[4][ 4] * b[4][ 3]
     +   a[4][ 5] * b[5][ 3] + a[4][ 6] * b[6][ 3] + a[4][ 7] * b[7][ 3] + a[4][ 8] * b[8][ 3];
    
    c[4][ 4] = a[4][ 1] * b[1][ 4] + a[4][ 2] * b[2][ 4] + a[4][ 3] * b[3][ 4] + a[4][ 4] * b[4][ 4]
     +   a[4][ 4] * b[5][ 4] + a[4][ 6] * b[6][ 4] + a[4][ 7] * b[7][ 4] + a[4][ 8] * b[8][ 4];
    
    c[4][ 5] = a[4][ 1] * b[1][ 5] + a[4][ 2] * b[2][ 5] + a[4][ 3] * b[3][ 5] + a[4][ 4] * b[4][ 5]
     +   a[4][ 5] * b[5][ 5] + a[4][ 6] * b[6][ 5] + a[4][ 7] * b[7][ 5] + a[4][ 8] * b[8][ 5];
    
    c[4][ 6] = a[4][ 1] * b[1][ 6] + a[4][ 2] * b[2][ 6] + a[4][ 3] * b[3][ 6] + a[4][ 4] * b[4][ 6]
     +   a[4][ 5] * b[5][ 6] + a[4][ 6] * b[6][ 6] + a[4][ 7] * b[7][ 6] + a[4][ 8] * b[8][ 6];
    
    c[4][ 7] = a[4][ 1] * b[1][ 7] + a[4][ 2] * b[2][ 7] + a[4][ 3] * b[3][ 7] + a[4][ 4] * b[4][ 7]
     +   a[4][ 5] * b[5][ 7] + a[4][ 6] * b[6][ 7] + a[4][ 7] * b[7][ 7] + a[4][ 8] * b[8][ 7];
    
    c[4][ 8] = a[4][ 1] * b[1][ 8] + a[4][ 2] * b[2][ 8] + a[4][ 3] * b[3][ 8] + a[4][ 4] * b[4][ 8]
     +   a[4][ 5] * b[5][ 8] + a[4][ 6] * b[6][ 8] + a[4][ 7] * b[7][ 8] + a[4][ 8] * b[8][ 8];
    
    //i=5
    
    c[5][ 1] = a[5][ 1] * b[1][ 1] + a[5][ 2] * b[2][ 1] + a[5][ 3] * b[3][ 1] + a[5][ 4] * b[4][ 1]
     +   a[5][ 5] * b[5][ 1] + a[5][ 6] * b[6][ 1] + a[5][ 7] * b[7][ 1] + a[5][ 8] * b[8][ 1];
    
    c[5][ 2] = a[5][ 1] * b[1][ 2] + a[5][ 2] * b[2][ 2] + a[5][ 3] * b[3][ 2] + a[5][ 4] * b[4][ 2]
     +   a[5][ 5] * b[5][ 2] + a[5][ 6] * b[6][ 2] + a[5][ 7] * b[7][ 2] + a[5][ 8] * b[8][ 2];
    
    c[5][ 3] = a[5][ 1] * b[1][ 3] + a[5][ 2] * b[2][ 3] + a[5][ 3] * b[3][ 3] + a[5][ 4] * b[4][ 3]
     +   a[5][ 5] * b[5][ 3] + a[5][ 6] * b[6][ 3] + a[5][ 7] * b[7][ 3] + a[5][ 8] * b[8][ 3];
    
    c[5][ 4] = a[5][ 1] * b[1][ 4] + a[5][ 2] * b[2][ 4] + a[5][ 3] * b[3][ 4] + a[5][ 4] * b[4][ 4]
     +   a[5][ 4] * b[5][ 4] + a[5][ 6] * b[6][ 4] + a[5][ 7] * b[7][ 4] + a[5][ 8] * b[8][ 4];
    
    c[5][ 5] = a[5][ 1] * b[1][ 5] + a[5][ 2] * b[2][ 5] + a[5][ 3] * b[3][ 5] + a[5][ 4] * b[4][ 5]
     +   a[5][ 5] * b[5][ 5] + a[5][ 6] * b[6][ 5] + a[5][ 7] * b[7][ 5] + a[5][ 8] * b[8][ 5];
    
    c[5][ 6] = a[5][ 1] * b[1][ 6] + a[5][ 2] * b[2][ 6] + a[5][ 3] * b[3][ 6] + a[5][ 4] * b[4][ 6]
     +   a[5][ 5] * b[5][ 6] + a[5][ 6] * b[6][ 6] + a[5][ 7] * b[7][ 6] + a[5][ 8] * b[8][ 6];
    
    c[5][ 7] = a[5][ 1] * b[1][ 7] + a[5][ 2] * b[2][ 7] + a[5][ 3] * b[3][ 7] + a[5][ 4] * b[4][ 7]
     +   a[5][ 5] * b[5][ 7] + a[5][ 6] * b[6][ 7] + a[5][ 7] * b[7][ 7] + a[5][ 8] * b[8][ 7];
    
    c[5][ 8] = a[5][ 1] * b[1][ 8] + a[5][ 2] * b[2][ 8] + a[5][ 3] * b[3][ 8] + a[5][ 4] * b[4][ 8]
     +   a[5][ 5] * b[5][ 8] + a[5][ 6] * b[6][ 8] + a[5][ 7] * b[7][ 8] + a[5][ 8] * b[8][ 8];

    //i=6
    
    c[6][ 1] = a[6][ 1] * b[1][ 1] + a[6][ 2] * b[2][ 1] + a[6][ 3] * b[3][ 1] + a[6][ 4] * b[4][ 1]
     +   a[6][ 5] * b[5][ 1] + a[6][ 6] * b[6][ 1] + a[6][ 7] * b[7][ 1] + a[6][ 8] * b[8][ 1];
    
    c[6][ 2] = a[6][ 1] * b[1][ 2] + a[6][ 2] * b[2][ 2] + a[6][ 3] * b[3][ 2] + a[6][ 4] * b[4][ 2]
     +   a[6][ 5] * b[5][ 2] + a[6][ 6] * b[6][ 2] + a[6][ 7] * b[7][ 2] + a[6][ 8] * b[8][ 2];
    
    c[6][ 3] = a[6][ 1] * b[1][ 3] + a[6][ 2] * b[2][ 3] + a[6][ 3] * b[3][ 3] + a[6][ 4] * b[4][ 3]
     +   a[6][ 5] * b[5][ 3] + a[6][ 6] * b[6][ 3] + a[6][ 7] * b[7][ 3] + a[6][ 8] * b[8][ 3];
    
    c[6][ 4] = a[6][ 1] * b[1][ 4] + a[6][ 2] * b[2][ 4] + a[6][ 3] * b[3][ 4] + a[6][ 4] * b[4][ 4]
     +   a[6][ 4] * b[5][ 4] + a[6][ 6] * b[6][ 4] + a[6][ 7] * b[7][ 4] + a[6][ 8] * b[8][ 4];
    
    c[6][ 5] = a[6][ 1] * b[1][ 5] + a[6][ 2] * b[2][ 5] + a[6][ 3] * b[3][ 5] + a[6][ 4] * b[4][ 5]
     +   a[6][ 5] * b[5][ 5] + a[6][ 6] * b[6][ 5] + a[6][ 7] * b[7][ 5] + a[6][ 8] * b[8][ 5];
    
    c[6][ 6] = a[6][ 1] * b[1][ 6] + a[6][ 2] * b[2][ 6] + a[6][ 3] * b[3][ 6] + a[6][ 4] * b[4][ 6]
     +   a[6][ 5] * b[5][ 6] + a[6][ 6] * b[6][ 6] + a[6][ 7] * b[7][ 6] + a[6][ 8] * b[8][ 6];
    
    c[6][ 7] = a[6][ 1] * b[1][ 7] + a[6][ 2] * b[2][ 7] + a[6][ 3] * b[3][ 7] + a[6][ 4] * b[4][ 7]
     +   a[6][ 5] * b[5][ 7] + a[6][ 6] * b[6][ 7] + a[6][ 7] * b[7][ 7] + a[6][ 8] * b[8][ 7];
    
    c[6][ 8] = a[6][ 1] * b[1][ 8] + a[6][ 2] * b[2][ 8] + a[6][ 3] * b[3][ 8] + a[6][ 4] * b[4][ 8]
     +   a[6][ 5] * b[5][ 8] + a[6][ 6] * b[6][ 8] + a[6][ 7] * b[7][ 8] + a[6][ 8] * b[8][ 8];

    //i=7
    
    c[7][ 1] = a[7][ 1] * b[1][ 1] + a[7][ 2] * b[2][ 1] + a[7][ 3] * b[3][ 1] + a[7][ 4] * b[4][ 1]
     +   a[7][ 5] * b[5][ 1] + a[7][ 6] * b[6][ 1] + a[7][ 7] * b[7][ 1] + a[7][ 8] * b[8][ 1];
    
    c[7][ 2] = a[7][ 1] * b[1][ 2] + a[7][ 2] * b[2][ 2] + a[7][ 3] * b[3][ 2] + a[7][ 4] * b[4][ 2]
     +   a[7][ 5] * b[5][ 2] + a[7][ 6] * b[6][ 2] + a[7][ 7] * b[7][ 2] + a[7][ 8] * b[8][ 2];
    
    c[7][ 3] = a[7][ 1] * b[1][ 3] + a[7][ 2] * b[2][ 3] + a[7][ 3] * b[3][ 3] + a[7][ 4] * b[4][ 3]
     +   a[7][ 5] * b[5][ 3] + a[7][ 6] * b[6][ 3] + a[7][ 7] * b[7][ 3] + a[7][ 8] * b[8][ 3];
    
    c[7][ 4] = a[7][ 1] * b[1][ 4] + a[7][ 2] * b[2][ 4] + a[7][ 3] * b[3][ 4] + a[7][ 4] * b[4][ 4]
     +   a[7][ 4] * b[5][ 4] + a[7][ 6] * b[6][ 4] + a[7][ 7] * b[7][ 4] + a[7][ 8] * b[8][ 4];
    
    c[7][ 5] = a[7][ 1] * b[1][ 5] + a[7][ 2] * b[2][ 5] + a[7][ 3] * b[3][ 5] + a[7][ 4] * b[4][ 5]
     +   a[7][ 5] * b[5][ 5] + a[7][ 6] * b[6][ 5] + a[7][ 7] * b[7][ 5] + a[7][ 8] * b[8][ 5];
    
    c[7][ 6] = a[7][ 1] * b[1][ 6] + a[7][ 2] * b[2][ 6] + a[7][ 3] * b[3][ 6] + a[7][ 4] * b[4][ 6]
     +   a[7][ 5] * b[5][ 6] + a[7][ 6] * b[6][ 6] + a[7][ 7] * b[7][ 6] + a[7][ 8] * b[8][ 6];
    
    c[7][ 7] = a[7][ 1] * b[1][ 7] + a[7][ 2] * b[2][ 7] + a[7][ 3] * b[3][ 7] + a[7][ 4] * b[4][ 7]
     +   a[7][ 5] * b[5][ 7] + a[7][ 6] * b[6][ 7] + a[7][ 7] * b[7][ 7] + a[7][ 8] * b[8][ 7];
    
    c[7][ 8] = a[7][ 1] * b[1][ 8] + a[7][ 2] * b[2][ 8] + a[7][ 3] * b[3][ 8] + a[7][ 4] * b[4][ 8]
     +   a[7][ 5] * b[5][ 8] + a[7][ 6] * b[6][ 8] + a[7][ 7] * b[7][ 8] + a[7][ 8] * b[8][ 8];

    //i=8
    
    c[8][ 1] = a[8][ 1] * b[1][ 1] + a[8][ 2] * b[2][ 1] + a[8][ 3] * b[3][ 1] + a[8][ 4] * b[4][ 1]
     +   a[8][ 5] * b[5][ 1] + a[8][ 6] * b[6][ 1] + a[8][ 7] * b[7][ 1] + a[8][ 8] * b[8][ 1];
    
    c[8][ 2] = a[8][ 1] * b[1][ 2] + a[8][ 2] * b[2][ 2] + a[8][ 3] * b[3][ 2] + a[8][ 4] * b[4][ 2]
     +   a[8][ 5] * b[5][ 2] + a[8][ 6] * b[6][ 2] + a[8][ 7] * b[7][ 2] + a[8][ 8] * b[8][ 2];
    
    c[8][ 3] = a[8][ 1] * b[1][ 3] + a[8][ 2] * b[2][ 3] + a[8][ 3] * b[3][ 3] + a[8][ 4] * b[4][ 3]
     +   a[8][ 5] * b[5][ 3] + a[8][ 6] * b[6][ 3] + a[8][ 7] * b[7][ 3] + a[8][ 8] * b[8][ 3];
    
    c[8][ 4] = a[8][ 1] * b[1][ 4] + a[8][ 2] * b[2][ 4] + a[8][ 3] * b[3][ 4] + a[8][ 4] * b[4][ 4]
     +   a[8][ 4] * b[5][ 4] + a[8][ 6] * b[6][ 4] + a[8][ 7] * b[7][ 4] + a[8][ 8] * b[8][ 4];
    
    c[8][ 5] = a[8][ 1] * b[1][ 5] + a[8][ 2] * b[2][ 5] + a[8][ 3] * b[3][ 5] + a[8][ 4] * b[4][ 5]
     +   a[8][ 5] * b[5][ 5] + a[8][ 6] * b[6][ 5] + a[8][ 7] * b[7][ 5] + a[8][ 8] * b[8][ 5];
    
    c[8][ 6] = a[8][ 1] * b[1][ 6] + a[8][ 2] * b[2][ 6] + a[8][ 3] * b[3][ 6] + a[8][ 4] * b[4][ 6]
     +   a[8][ 5] * b[5][ 6] + a[8][ 6] * b[6][ 6] + a[8][ 7] * b[7][ 6] + a[8][ 8] * b[8][ 6];
    
    c[8][ 7] = a[8][ 1] * b[1][ 7] + a[8][ 2] * b[2][ 7] + a[8][ 3] * b[3][ 7] + a[8][ 4] * b[4][ 7]
     +   a[8][ 5] * b[5][ 7] + a[8][ 6] * b[6][ 7] + a[8][ 7] * b[7][ 7] + a[8][ 8] * b[8][ 7];
    
    c[8][ 8] = a[8][ 1] * b[1][ 8] + a[8][ 2] * b[2][ 8] + a[8][ 3] * b[3][ 8] + a[8][ 4] * b[4][ 8]
     +   a[8][ 5] * b[5][ 8] + a[8][ 6] * b[6][ 8] + a[8][ 7] * b[7][ 8] + a[8][ 8] * b[8][ 8];

    return(0);

}
