// Loop04Dlg.cpp : implementation file
//
// This program compares the execution times for a repetitive Simpson's
// rule integration with the summation loop fully rolled up or partially
// unrolled.  The technique of partial unrolling is useful when full
// unrolling is not feasible.
//
// Recall that the integral, I, of a function is divided into 2n steps
// (Simpson's Rule requires an even number of steps) can be computed
// by
//
//				I = h/3 * (s0 + 2s1 + 4s0)
//
// where h = (b-a)/2n, s0 = f(0) + f(n), s1 = f(1) + f(3) + f(5) + ...
// f(2n-2) and s2 = f(2) + f(4) + f(6) + ... + f(2n - 1)

#include "stdafx.h"
#include "Loop04.h"
#include "Loop04Dlg.h"
#include "math.h"
#include "stdio.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// global declarations
static int cNumIts = 50000;				//number of iterations

float f[10000];							//function f(x)

/////////////////////////////////////////////////////////////////////////////
// CLoop04Dlg dialog

CLoop04Dlg::CLoop04Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CLoop04Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CLoop04Dlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CLoop04Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLoop04Dlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CLoop04Dlg, CDialog)
	//{{AFX_MSG_MAP(CLoop04Dlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON1, OnButton1)
	ON_BN_CLICKED(IDC_BUTTON2, OnButton2)
	ON_BN_CLICKED(IDC_BUTTON3, OnButton3)
	ON_BN_CLICKED(IDC_BUTTON4, OnButton4)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLoop04Dlg message handlers

BOOL CLoop04Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CLoop04Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CLoop04Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}


// Rolled up Simpson's Rule Function
void CLoop04Dlg::OnButton1() 
{

	//local declarations
	//integers
	int SimpCounter;		//simpson's rule summation counter
	int ItCounter;			//iteration counter
	int Counter;			//function assignment counter
	int j;					//output string pointer

	//floats
	float h_3;				//simpson's rule parameters
	float S0;
	float S1;
	float S2;
	float Integral;			//result
	
	//doubles
	double argX;			//x for assignment
	double starttime;		//ticks at start of main loop
	double stoptime;		//ticks at main loop exit
	double ElapsedTime;		//ticks during main loop

	//character strings
	char buffer[200];		//output string

	//assign the function f(x) = x^5
	for(Counter=1; Counter <= 10000; ++Counter){

		argX = (double)Counter * 0.0001;
		f[Counter] = (float)pow(argX, 5);

	}

	//start the timer
	starttime = GetTickCount();

	for(ItCounter=1; ItCounter <= cNumIts; ++ItCounter){

		//h/3, where h = (b-a)/n
		h_3 = (float)0.999 / 30000;

		//sum the first and last terms
		S0 = f[0] + f[9999];

		//zero values for running summations
		S1 = 0;
		S2 = 0;

		//sum terms except first and last
		for(SimpCounter = 1; SimpCounter <= 9997; SimpCounter=SimpCounter + 2){

			S1 += f[SimpCounter];
			S2 += f[SimpCounter + 1];

		}

		// 'odd' terms * 2
		S1 *= 2;

		//'even' terms * 4
		S2 *= 4;

		//total summation
		Integral = S0 + S1 + S2;

		//multiply by h/3
		Integral *= h_3;

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;

	//build the output the string and display in message box
	j=sprintf(buffer, "Using Rolled Loop, integrated x^5 50,000 times by Simpson's Rule in %g", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.\n");
	j += sprintf(buffer + j, "Integral = %g", Integral);

	MessageBox(buffer, "Done with Repetitive Simpson's Rule Integration",
				MB_OK | MB_ICONEXCLAMATION);

}


// Simpson's Rule Function unrolled by five terms
void CLoop04Dlg::OnButton2() 
{
	
	//local declarations
	//integers
	int SimpCounter;		//simpson's rule loop counter
	int ItCounter;			//iteration counter
	int Counter;			//counter to assign function
	int j;					//output string pointer

	//floats
	float h_3;				//simpson's rule parameters
	float S0;
	float S1;
	float S2;
	float Integral;			//result
	
	//doubles
	double argX;			//x for x^5 assignment
	double starttime;		//ticks at start of main loop
	double stoptime;		//ticks at main loop exit
	double ElapsedTime;		//ticks during loop

	//character strings
	char buffer[200];		//output string

	//assign the function f(x) = x^5
	for(Counter=1; Counter <= 10000; ++Counter){

		argX = (double)Counter * 0.0001;
		f[Counter] = (float)pow(argX, 5);

	}

	//start the timer
	starttime = GetTickCount();

	for(ItCounter=1; ItCounter <= cNumIts; ++ItCounter){

		//h/3, where h=(b-1)/n
		h_3 = (float)0.999 /30000;

		//sum the first and last terms
		S0 = f[0] + f[9999];

		//zero the values for the running summations
		S1 = 0;
		S2 = 0;

		//do the unrolled summation
		for(SimpCounter = 1; SimpCounter <= 9981; SimpCounter = SimpCounter + 10){

			S1 += f[SimpCounter] + f[SimpCounter + 2] + f[SimpCounter + 4] + f[SimpCounter + 6] + f[SimpCounter + 8];
			S2 += f[SimpCounter + 1] + f[SimpCounter + 3] + f[SimpCounter + 5] + f[SimpCounter + 7] + f[SimpCounter + 9];

		}

		//clean up extra terms
		S1 += f[9991] + f[9993] + f[9995] + f[9997];
		S2 += f[9992] + f[9994] + f[9996] + f[9998];

		// 'odd; terms * 2
		S1 *= 2;

		// 'even' terms * 4
		S2 *= 4;

		// total summation
		Integral = S0 + S1 + S2;
		
		//multiply by h/3
		Integral *= h_3;

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;

	//build the output string and display in a message box
	j=sprintf(buffer, "Using Loop unrolled by five, integrated x^5 50,000 times by Simpson's Rule in %g", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.\n");
	j += sprintf(buffer + j, "Integral = %g", Integral);

	MessageBox(buffer, "Done with Repetitive Simpson's Rule Integration",
				MB_OK | MB_ICONEXCLAMATION);
	
}

//Simpson's rule integration with loop unrolled by four terms
void CLoop04Dlg::OnButton3() 
{
	//local declarations
	//integers
	int SimpCounter;		//simpson's rule loop counter
	int ItCounter;			//iteration counter
	int Counter;			//counter to assign function
	int j;					//output string pointer

	//floats
	float h_3;				//simpson's rule parameters
	float S0;
	float S1;
	float S2;
	float Integral;			//result
	
	//doubles
	double argX;			//x for x^5 assignment
	double starttime;		//ticks at start of main loop
	double stoptime;		//ticks at main loop exit
	double ElapsedTime;		//ticks during loop

	//character strings
	char buffer[200];		//output string

	//assign the function f(x) = x^5
	for(Counter=1; Counter <= 10000; ++Counter){

		argX = (double)Counter * 0.0001;
		f[Counter] = (float)pow(argX, 5);

	}

	//start the timer
	starttime = GetTickCount();

	for(ItCounter=1; ItCounter <= cNumIts; ++ItCounter){

		//h/3, where h=(b-1)/n
		h_3 = (float)0.999 /30000;

		//sum the first and last terms
		S0 = f[0] + f[9999];

		//zero the values for the running summations
		S1 = 0;
		S2 = 0;

		//do the unrolled summation
		for(SimpCounter = 1; SimpCounter <= 9985; SimpCounter = SimpCounter + 8){

			S1 += f[SimpCounter] + f[SimpCounter + 2] + f[SimpCounter + 4] + f[SimpCounter + 6];
			S2 += f[SimpCounter + 1] + f[SimpCounter + 3] + f[SimpCounter + 5] + f[SimpCounter + 7];

		}

		//clean up extra terms
		S1 += f[9993] + f[9995] + f[9997];
		S2 += f[9994] + f[9996] + f[9998];

		// 'odd; terms * 2
		S1 *= 2;

		// 'even' terms * 4
		S2 *= 4;

		// total summation
		Integral = S0 + S1 + S2;
		
		//multiply by h/3
		Integral *= h_3;

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;

	//build the output string and display in a message box
	j=sprintf(buffer, "Using Loop Unrolled by four terms, integrated x^5 50,000 times by Simpson's Rule in %g", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.\n");
	j += sprintf(buffer + j, "Integral = %g", Integral);

	MessageBox(buffer, "Done with Repetitive Simpson's Rule Integration",
				MB_OK | MB_ICONEXCLAMATION);
	
}

// Simpson's rule integration with loop unrolled by eight terms
void CLoop04Dlg::OnButton4() 
{
	//local declarations
	//integers
	int SimpCounter;		//simpson's rule loop counter
	int ItCounter;			//iteration counter
	int Counter;			//counter to assign function
	int j;					//output string pointer

	//floats
	float h_3;				//simpson's rule parameters
	float S0;
	float S1;
	float S2;
	float Integral;			//result
	
	//doubles
	double argX;			//x for x^5 assignment
	double starttime;		//ticks at start of main loop
	double stoptime;		//ticks at main loop exit
	double ElapsedTime;		//ticks during loop

	//character strings
	char buffer[200];		//output string

	//assign the function f(x) = x^5
	for(Counter=1; Counter <= 10000; ++Counter){

		argX = (double)Counter * 0.0001;
		f[Counter] = (float)pow(argX, 5);

	}

	//start the timer
	starttime = GetTickCount();

	for(ItCounter=1; ItCounter <= cNumIts; ++ItCounter){

		//h/3, where h=(b-1)/n
		h_3 = (float)0.999 /30000;

		//sum the first and last terms
		S0 = f[0] + f[9999];

		//zero the values for the running summations
		S1 = 0;
		S2 = 0;

		//do the unrolled summation
		for(SimpCounter = 1; SimpCounter <= 9969; SimpCounter = SimpCounter + 16){

			S1 += f[SimpCounter] + f[SimpCounter + 2] + f[SimpCounter + 4] + f[SimpCounter + 6] + f[SimpCounter + 8]
					+ f[SimpCounter + 10] + f[SimpCounter + 12] + f[SimpCounter + 14];
			S2 += f[SimpCounter + 1] + f[SimpCounter + 3] + f[SimpCounter + 5] + f[SimpCounter + 7] + f[SimpCounter + 9]
					+ f[SimpCounter + 11] + f[SimpCounter + 13] + f[SimpCounter + 15];

		}

		//clean up extra terms
		S1 += f[9985] + f[9987] + f[9989] + f[9991] + f[9993] + f[9995] + f[9997];
		S2 += f[9986] + f[9988] + f[9990] + f[9992] + f[9994] + f[9996] + f[9998];

		// 'odd; terms * 2
		S1 *= 2;

		// 'even' terms * 4
		S2 *= 4;

		// total summation
		Integral = S0 + S1 + S2;
		
		//multiply by h/3
		Integral *= h_3;

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;

	//build the output string and display in a message box
	j=sprintf(buffer, "Using Loop Unrolled by eight terms, integrated x^5 50,000 times by Simpson's Rule in %g", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.\n");
	j += sprintf(buffer + j, "Integral = %g", Integral);

	MessageBox(buffer, "Done with Repetitive Simpson's Rule Integration",
				MB_OK | MB_ICONEXCLAMATION);
	
	
}
