// RISC03Dlg.cpp : implementation file
//
// This program demonstrates the use of the RISC style coding
// to compute both the 2d distance and the LJ Energy.

#include "stdafx.h"
#include "RISC03.h"
#include "RISC03Dlg.h"
#include "stdio.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CRISC03Dlg dialog

CRISC03Dlg::CRISC03Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CRISC03Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CRISC03Dlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CRISC03Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRISC03Dlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CRISC03Dlg, CDialog)
	//{{AFX_MSG_MAP(CRISC03Dlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON1, OnRISC)
	ON_BN_CLICKED(IDCISC, OnCisc)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRISC03Dlg message handlers

BOOL CRISC03Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CRISC03Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CRISC03Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CRISC03Dlg::OnOK() 
{
	// TODO: Add extra validation here

	


	
	CDialog::OnOK();
}

// Compute r and E via RISC style algorithm
void CRISC03Dlg::OnRISC() 
{

	// local declarations
	// integers
	int j;							//output string pointer
	int counter;					//iteration counter

	// doubles
	double x1 = 1.234;				//entry value coordinates
	double x2 = 2.675;				//with some values to use
	double y1 = 0.045;
	double y2 = 2.073;
	double tempx;					//deltax, then (deltax)^2
	double tempy;					//deltay, then (deltay)^2
	double r;						//computed distance
	double denom1;					//distance^6
	double denom2;					//distance^12
	double Energy;					//computed energy
	double starttime;				//system ticks at start of loop
	double stoptime;				//system ticks at loop exit
	double ElapsedTime;				//system ticks during loop

	//character strings
	char buffer[200];

	//start the timer
	starttime = GetTickCount();

	for(counter = 1; counter <= 10000000; ++counter){

		//compute distance in RISC style
		tempx = x1 - x2;
		tempx = tempx * tempx;
		tempy = y1 - y2;
		tempy = tempy * tempy;

		r= sqrt(tempx + tempy);

		//use the computed distance to compute LJ energy in RISC style
		denom1 = r * r * r * r * r * r;
		denom2 = denom1 * denom1;

		Energy = 1/denom2 - 1/denom1;

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;

	//build the output string and display in a message box
	j = sprintf(buffer, "Computed r and E 10,000,000 times in RISC style in ");
	j += sprintf(buffer + j, "%g ", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.");

	MessageBox(buffer, "Done with RISC Computation",
				MB_OK | MB_ICONEXCLAMATION);
	


}

// Compute r and E via CISC style algorithm
void CRISC03Dlg::OnCisc() 
{
	
	// local declarations
	// integers
	int j;							//output string pointer
	int counter;					//interation counter

	// doubles
	double x1 = 1.234;				//entry value coordinates
	double x2 = 2.675;				//with some values to use
	double y1 = 0.045;
	double y2 = 2.073;
	double r;						//computed distance
	double Energy;					//computed energy
	double starttime;				//system ticks at start of loop
	double stoptime;				//system ticks at loop exit
	double ElapsedTime;				//system ticks during loop

	//character strings
	char buffer[200];				//output string

	//start the timer
	starttime = GetTickCount();

	for(counter = 1; counter <= 10000000; ++counter){

		//compute the distance in CISC style
		r = sqrt(pow(x1-x2,2) + pow(y1-y2,2));

		//use the computed distance to compute the
		//energy in CISC style
		Energy = 1/pow(r,12) - 1/pow(2,6);

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;

	//build the output string and display the result in a message box
	j = sprintf(buffer, "Computed r and E 10,000,000 times in CISC style in ");
	j += sprintf(buffer + j, "%g ", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.");

	MessageBox(buffer, "Done with CISC Computation",
				MB_OK | MB_ICONEXCLAMATION);

}
