// LookUp02Dlg.cpp : implementation file
//
// This program demonstrates the Look Up Table
// technique for the Boltzmann factor
//
// The Look Up Table is initialized in CLookUp02Dlg::OnInitializeTable() 
// The explicit calculation is in CLookUp02Dlg::OnCompute()
// The look up table calculation is in CLookUp02Dlg::OnLookUp()

#include "stdafx.h"
#include "LookUp02.h"
#include "LookUp02Dlg.h"
#include "math.h"
#include "stdio.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//globals
//integers
int cNumIts = 10000000;
    
//doubles
double k = 1.381E-23;	// Boltzmann constant
double Temp = 100;		// Temperature
double Energy = 4e-21;	// energy
double Expon[10000];	// look up array


/////////////////////////////////////////////////////////////////////////////
// CLookUp02Dlg dialog

CLookUp02Dlg::CLookUp02Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CLookUp02Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CLookUp02Dlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CLookUp02Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLookUp02Dlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CLookUp02Dlg, CDialog)
	//{{AFX_MSG_MAP(CLookUp02Dlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDLookUp, OnLookUp)
	ON_BN_CLICKED(IDCompute, OnCompute)
	ON_BN_CLICKED(IDInitializeTable, OnInitializeTable)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLookUp02Dlg message handlers

BOOL CLookUp02Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CLookUp02Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CLookUp02Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

// look up the Boltzmann factor
void CLookUp02Dlg::OnLookUp() 
{

	//local declarations
	//integers
	int j;				// output string pointer
	int Counter;		// iteration counter
	int ExIndex;		// look up argument index
	
	//doubles
	double BoltzFact;	// looked up Boltzmann factor
	double starttime;	// ticks at start of loop
	double stoptime;	// ticks at loop exit
	double ElapsedTime;	// loop time in seconds
	
	//character string
	char buffer[200];	// output string

	// start the timer
	starttime = GetTickCount();

	for(Counter=1; Counter <=10000000; ++Counter){
		
		// compute the index from the real argument
		ExIndex = (int)((Energy/(k * Temp)) * 100);

		BoltzFact = Expon[ExIndex];

	}
	
	//stop the timer
	stoptime = GetTickCount();

	//compute the loop time in seconds
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//build the output string and display in message box
	j = sprintf(buffer,"Looked Up Boltzmann Factor 10,000,000 times in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds. ");
	j += sprintf(buffer + j, "\nBoltzmann Factor = %.15g ", BoltzFact);

	MessageBox(buffer,
					 "Boltzmann Factor Look-Up",
					 MB_OK | MB_ICONEXCLAMATION);
	
}

// explicitly compute the Boltzmann factor
void CLookUp02Dlg::OnCompute() 
{

	//local declarations
	//integers
	int j;				// output string pointer 
	int Counter;		// iteration counter
	
	//doubles
	double BoltzFact;	// computed Boltzmann factor
	double starttime;	// ticks at start of loop
	double stoptime;	// ticks at loop exit
	double ElapsedTime;	// loop time in seconds
	
	//character string
	char buffer[200];	// output string

	//start the timer
	starttime = GetTickCount();

	for(Counter=1; Counter <=10000000; ++Counter){

		//compute the Boltzmann factor
		BoltzFact = exp(-Energy / (k * Temp));

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the loop time in seconds
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//build the output string and display in a message box
	j = sprintf(buffer,"Computed Boltzmann Factor 10,000,000 times in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds. ");
	j += sprintf(buffer + j, "\nBoltzmann Factor = %.15g ", BoltzFact);

	MessageBox(buffer,
					 "Explicit Boltzmann Factor Calculation",
					 MB_OK | MB_ICONEXCLAMATION);

}

// initialize the lookup table
void CLookUp02Dlg::OnInitializeTable() 
{

	//local declarations
	//integers
	int j;				// output string pointer
	int Counter;		// look up array index
	
	//doubles
	double arg1;		// exponential function argument
	double starttime;	// ticks at start of loop
	double stoptime;	// ticks at loop exit
	double ElapsedTime;	// loop time in seconds
	
	//character string
	char buffer[200];	// output string

	//start the timer
	starttime = GetTickCount();

	for(Counter=1; Counter <=10000; ++Counter){

		//compute the argument
		arg1 = (double)Counter/(double)100;

		//assign the function at index for argument
		Expon[Counter] = exp(-arg1);

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute loop time in seconds
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//build the output string and display in a message box
	j = sprintf(buffer,"Initialized Boltzmann Look-Up Table in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds. ");
	
	MessageBox(buffer,
					 "Inialized Look Up Table",
					 MB_OK | MB_ICONEXCLAMATION);

}
