// LookUp04Dlg.cpp : implementation file
//
// Demo LookUp Table technique for combined distance (r) and Lennard 
// Jones pair energy calculation.  The LJ parameters used are for Helium.
//
// The working functions are:
//
//			CLookUp04Dlg::OnButton1		Inialization of look up tables
//			CLookUp04Dlg::OnButton2		r and E by explicit computation
//			CLookUp04Dlg::OnButton3		r and E looked up
//
#include "stdafx.h"
#include "LookUp04.h"
#include "LookUp04Dlg.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//defines
#define cNumIts		100000000			//number of iterations
#define x_1 		103					//just some values to use
#define x_2 		373					//for coordinates of two particles
#define y_1			271
#define y_2			144
#define depth		10.22				//well depth for He -- He
#define sigma		258					//E(sigma)=0, for He -- He
#define sigma6		2.9492951E14		//sigma^6
#define sigma12		8.6983418E28		//sigma^12

//global declarations
int r[2831][2831];					//lookup table for distance

//doubles
double e[2831];						//lookup table for energy

/////////////////////////////////////////////////////////////////////////////
// CLookUp04Dlg dialog

CLookUp04Dlg::CLookUp04Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CLookUp04Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CLookUp04Dlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CLookUp04Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLookUp04Dlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CLookUp04Dlg, CDialog)
	//{{AFX_MSG_MAP(CLookUp04Dlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON1, OnButton1)
	ON_BN_CLICKED(IDC_BUTTON2, OnButton2)
	ON_BN_CLICKED(IDC_BUTTON3, OnButton3)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLookUp04Dlg message handlers

BOOL CLookUp04Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CLookUp04Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CLookUp04Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

//initialize look up tables; r is in picometers
void CLookUp04Dlg::OnButton1() 
{
	//local declarations
	//longs					
	int j;						//output string pointer
	int Counter;				//loop counter for E intitialization
	int yCounter;				//counter for deltay
	
	//doubles
	double rvalue;				//actual r for E(r)
								//will also be used for deltax
	double r6;					//rvalue^6
	double r12;					//rvalue^12
	double deltay;				//y2-y1, then (y2-y1)^2
	double starttime;			//'ticks' at start of loop
	double stoptime;			//'ticks' at loop exit
	double ElapsedTime;			//'ticks' during loop
	
	//character strings
	char buffer[200];			//output string

	//start the timer
	starttime = GetTickCount();

	//compute E(r) from 1 to 2380 pm (0.01 to 23.80 angstroms)
	//and r(deltax, deltay)
	for(Counter=0; Counter <= 2830; ++Counter){

		//convert index to double
		rvalue = double(Counter);

		if (rvalue>0){
			//compute r^6
			r6 = rvalue * rvalue * rvalue * rvalue * rvalue * rvalue;

			//compute r^12
			r12 = r6 * r6;

			//compute E(r)
			e[Counter] = sigma12/r12 - sigma6/r6;
			e[Counter] = e[Counter] * depth;
		}

		//compute r(deltax, deltay), using rvalue for deltax
		//compute deltax^2
		rvalue = rvalue * rvalue;

		for(yCounter=0; yCounter <= 2830; ++yCounter){

			//convert to double
			deltay = (double)yCounter;

			//compute deltay^2
			deltay = deltay * deltay;

			//compute sum
			deltay = deltay + rvalue;

			//and take square root
			//only need integer part, since the r(x,y) entries
			//are used as indices to use to enter e(r) array
			r[Counter][yCounter] = (int)sqrt(deltay);
		}
	}



	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//construct output string and display the result in message box
	j = sprintf(buffer,"Initialized r and E arrays in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds. ");

	MessageBox(buffer,
					 "r and E LookUp Table Initialization",
					 MB_OK | MB_ICONEXCLAMATION);

	
} // end CLookUp04Dlg::OnButton1

//compute r and E using the risc algorithm from Ch. 8
void CLookUp04Dlg::OnButton2() 
{
	//local declarations
	//longs					
	int j;						//output string pointer
	int Counter;				//iteration counter
	
	//doubles
	double delx;				//delta x
	double dely;				//delta y
	double r_2;					//distance, r^2
	double r_6;					//r^6
	double r_12;				//r^12
	double energy;				//computed energy
	double starttime;			//'ticks' at start of loop
	double stoptime;			//'ticks' at loop exit
	double ElapsedTime;			//'ticks' during loop
	
	//string characters
	char buffer[200];			//output string

	//start the timer
	starttime = GetTickCount();

	for(Counter=1; Counter <= cNumIts; ++Counter){

		//compute deltax and deltay from the coordinates
		delx = (double)(x_2-x_1);
		dely = (double)(y_2-y_1);

		//compute r^2 using RISC algorithm from Ch.8
		delx = delx * delx;
		dely = dely * dely;

		r_2 = delx + dely;

		//compute energy via RISC algorithm from Ch.8
		//using r^2
		r_6 = r_2 * r_2 * r_2;
		r_12 = r_6 * r_6;

		energy = sigma12/r_12 - sigma6/r_6;
		energy = energy * depth;

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//construct output string and display the result in message box
	j = sprintf(buffer,"Computed Energy by RISC style in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds. ");
	j += sprintf(buffer + j, "\nEnergy = %.15g ", energy);

	MessageBox(buffer,
					 "Explicit RISC Energy Calculation",
					 MB_OK | MB_ICONEXCLAMATION);
	
} // end CLookUp04Dlg::OnButton2

//'compute' r and E by lookup tables
void CLookUp04Dlg::OnButton3() 
{
	//local declarations
	//longs					
	int j;						//output string pointer
	int Counter;				//iteration counter
	int delx;					//deltax
	int dely;					//deltay
	int EIndex;					//looked up r value, used
								//as index to the E lookup table
	
	//doubles
	double energy;
	double starttime;			//'ticks' at start of loop
	double stoptime;			//'ticks' at loop exit
	double ElapsedTime;			//'ticks' during loop
	
	//string characters
	char buffer[200];			//output string

	//start the timer
	starttime = GetTickCount();

	for(Counter=1; Counter <= cNumIts; ++Counter){

		//compute the indices to lookup the distance, r
		delx = labs(x_1-x_2);
		dely = labs(y_1-y_2);

		//lookup the distance
		EIndex = r[delx][dely];

		//lookup the energy
		energy = e[EIndex];

	}

	//stop the timer
	stoptime = GetTickCount();

	//compute the elapsed time
	ElapsedTime = (stoptime - starttime) * 0.001;
	
	//construct output string and display the result in message box
	j = sprintf(buffer,"Looked-Up r and E in %g ",ElapsedTime);
	j += sprintf(buffer + j, "%s"," seconds. ");
	j += sprintf(buffer + j, "\nEnergy = %.15g ", energy);

	MessageBox(buffer,
					 "Energy by Look-Up of r and E",
					 MB_OK | MB_ICONEXCLAMATION);
	
} // end CLookUp04Dlg::OnButton3
