// Other02Dlg.cpp : implementation file
//
// This program demonstrates the elimination of nested loop code
// for an algorithm amenable to this optimization.
//
// In the Monte Carlo procedure, the total pair energy
// is not needed at each step; only the change in energy
// that results from moving a single particle is needed.
//
// The "Full Energy" using nested loops is computed in COther02Dlg::OnButton1()
// The energy using only "Single Particle Energy" is computed in COther02Dlg::OnButton2()
// Look Up Tables and other initializations are done in COther02Dlg::OnButton3() 

#include "stdafx.h"
#include "Other02.h"
#include "Other02Dlg.h"
#include "math.h"
#include "stdio.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define NUMITS   50000	   //number of iterations
#define NumInBox 20        //Number of particles in
#define BoxSide  2500      //Actual box is 2500pm x 2500pm
                           //precision is 10 pm.

//global declarations

//integers
int Rval[354][354];         //Look-Up Table Entry for precomputed distances
              
//floats
float Sigma;                //LJ Sigma parameter
float Depth;                //LJ Well depth parameter

//doubles
double Energy[354];         //Look-Up Table Entry for the 6 LJ Energies as
                            //as function of distance; distance mesh is 10pm
double TotalEnergy;         //Total Energy

//structures
struct particle {           //Structure to hold particle data; just a bit neater
                            //to store this way, and sets stage for OOP if desired
    float x;                //x coordinate
    float y;                //y coordinate
    
} Particles[20];


/////////////////////////////////////////////////////////////////////////////
// COther02Dlg dialog

COther02Dlg::COther02Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(COther02Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(COther02Dlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void COther02Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COther02Dlg)
	DDX_Control(pDX, IDC_BUTTON2, m_Button2);
	DDX_Control(pDX, IDC_BUTTON1, m_Button1);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(COther02Dlg, CDialog)
	//{{AFX_MSG_MAP(COther02Dlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON1, OnButton1)
	ON_BN_CLICKED(IDC_BUTTON2, OnButton2)
	ON_BN_CLICKED(IDC_BUTTON3, OnButton3)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COther02Dlg message handlers

BOOL COther02Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void COther02Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR COther02Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

// simulates energy part of Monte Carlo using 
// total LJ pair energy at each step
void COther02Dlg::OnButton1() 
{
	//local declarations
	//integers
	int counter;		//iteration counter
	int j;				//character pointer
	int Ipart;          //Particle I counter
    int Jpart;          //Particle J counter
    int deltaX;         //Difference between x of I and J particles
    int deltaY;         //Difference between y of I and J particles
    int R;              //R from Rval lookup table

	//floats
	float sXi;			//cache variable for x coordinate of particle I
	float sYi;			//cache variable for y coordinate of particle I
	float sXj;			//cache variable for x coordinate of particle J
	float sYj;			//cache variable for y coordinate of particle J
    
	//doubles
	double ThisEnergy;  //running variable for energy
	double starttime;	//system ticks at start of loop
	double stoptime;	//system ticks at end of loop
	double ElapsedTime;	//system ticks during loop

	//character strings
	char buffer[200];	//output string

	//start the timer
	starttime = GetTickCount();		
	
	for(counter = 1; counter <= NUMITS; ++counter){
                
	    //initialize to zero
		ThisEnergy = 0;

		//compute energy in nested loop
		for(Ipart = 0; Ipart < 19; ++Ipart){
    
			//cache x,y values
			sXi = Particles[Ipart].x;
			sYi = Particles[Ipart].y;
        
			for(Jpart = Ipart + 1; Jpart < 20; ++Jpart){
        
				//cache x,y values
				sXj = Particles[Jpart].x;
				sYj = Particles[Jpart].y;
            
				//compute x difference to 10 pm
				deltaX = (int)((fabs(sXi - sXj)) / 10);
            
				//compute y difference, to 10 pm
				deltaY = (int)((fabs(sYi - sYj)) / 10);
                
				//lookup distance rather than compute
				R = Rval[deltaX][deltaY];
        
				//lookup energy
				ThisEnergy = ThisEnergy + Energy[R];
        
			}
    
		}

	}

	//stop the timer
	stoptime = GetTickCount();		
    
	//compute the elapsed time, in seconds
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result in a message box
	j=sprintf(buffer, "50,000 full energy calculations in %g", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.\n");

	MessageBox(buffer, "Done Full Energy Calculation",
				MB_OK | MB_ICONEXCLAMATION);
	
	
}

// simulates energy part of Monte Carlo using 
// only single particle LJ pair energy at each step
void COther02Dlg::OnButton2() 
{
	//local declarations
	//integers
	int counter;		//iteration counter
	int j;				//character pointer
	int I;	            //Particle I counter
    int deltaX;         //Difference between x of I and J particles
    int deltaY;         //Difference between y of I and J particles
    int R;              //R from Rval lookup table
	int MovedIndex;		//index of moved particle

	//floats
	float sCurrentX;	//cache variable for x coordinate of particle I
	float sCurrentY;	//cache variable for y coordinate of particle I
	float sMovedX;		//cache for moved x
	float sMovedY;		//cache for moved y

	//doubles
	double ThisEnergy;  //running variable for energy
	double starttime;	//system ticks at start of loop
	double stoptime;	//system ticks at end of loop
	double ElapsedTime;	//system ticks during loop

	//character strings
	char buffer[200];	//output string

	//start the timer
	starttime = GetTickCount();		
	
	for(counter = 1; counter <= NUMITS; ++counter){
        
        //select a particle
		MovedIndex = (int)(rand()/RAND_MAX);

		//cache coordinates of selected particle
		sMovedX = Particles[MovedIndex].x;
		sMovedY = Particles[MovedIndex].y;

	    //initialize to zero
		ThisEnergy = 0;

		//compute energy in nested loop
		for(I = 0; I < 20; ++I){
    
			//cache x,y values
			sCurrentX = Particles[I].x;
			sCurrentY = Particles[I].y;
        
			if(I != MovedIndex){
        
				//compute x difference to 10 pm
				deltaX = (int)((fabs(sCurrentX - sMovedX)) / 10);
            
				//compute y difference, to 10 pm
				deltaY = (int)((fabs(sCurrentY - sMovedY)) / 10);
                
				//lookup distance rather than compute
				R = Rval[deltaX][deltaY];
        
				//lookup energy
				ThisEnergy = ThisEnergy + Energy[R];
        
			}
    
		}

	}

	//stop the timer
	stoptime = GetTickCount();		
    
	//compute the elapsed time, in seconds
    ElapsedTime = (stoptime - starttime) * .001;

	//display the result in a message box
	j=sprintf(buffer, "50,000 delta energy only calculations in %g", ElapsedTime);
	j += sprintf(buffer + j, "%s", " seconds.\n");

	MessageBox(buffer, "Done Delta Energy Calculation",
				MB_OK | MB_ICONEXCLAMATION);
	
	
}

//initialize the look up tables
void COther02Dlg::OnButton3() 
{

	//local declarations
    //integers
    int PCounter;    //particle counter
    int rCountx;     //x coordinate counter
    int rCounty;     //y coordinate counter
    
	//floats
    float tempx;     //cache variable for x
    float tempy;     //cache variable for y

    //doubles
    double R;         //Cache variable for distance
    double R6;        //Distance^6
    double R12;       //Distance^12
    double Sigma6;    //LJ Sigma^6
    double Sigma12;   //LJ Sigma^12
    

    for(rCountx = 0; rCountx < 354; ++rCountx){

		//distance mesh (or step) is 10 pm
        R = rCountx * 10;
        
        //compute distance^6
        R6 = R * R * R * R * R * R;
        
        //compute distance^12
        R12 = R6 * R6;
        
        //Compute Sigma^6
        Sigma6 = Sigma * Sigma * Sigma * Sigma * Sigma * Sigma;
        
        //compute Sigma^12
        Sigma12 = Sigma6 * Sigma6;
        
        //Compute LJ Energy
        Energy[rCountx] = Sigma12 / R12 - Sigma6 / R6;
        Energy[rCountx] = Energy[rCountx] * Depth;
        
		//distance mesh is 10 pm
        tempx = (float)rCountx * 10;
        
        //square "delta-x"
        tempx = tempx * tempx;
        
        for(rCounty=0; rCounty < 354; ++rCounty){
        
            //distance mesh is 10 pm
            tempy = (float)rCounty * 10;
            
            //square "delta-y"
            tempy = tempy * tempy;

            //store the distance
            Rval[rCountx][rCounty] = (int)(sqrt(tempx + tempy) * 0.1);
            
        }
    
    }
        
    //initialize random particle positions
    for(PCounter =0; PCounter < 20; ++PCounter){

        Particles[PCounter].x = (float)(rand()/RAND_MAX);
        Particles[PCounter].y = (float)(rand()/RAND_MAX);
        
    }

	//enable compute buttons after initializations
	m_Button1.EnableWindow();
	m_Button2.EnableWindow();

}
